#include "dbcrs.h"
#include "../../GEFEL/intools.h"
#include <string.h>

csec :: csec(void)
{
  type   = crsectypet(-1);
  nprop  = 0;
  prop   = NULL;
  propu  = NULL;
  ridx   = NULL;
  npropu = 0;
}

csec :: ~csec(void)
{
  for (long i=0; i < nprop; i++)
    delete [] prop[i];
  delete [] prop;
  delete [] propu;
  delete [] ridx;
}

long csec :: read(FILE *in)
{

  if (getint(in, (int &)type) || getlong(in, nprop))
  // reading cross-section type number and number of indeces
  {
    fprintf(stderr, "\nError reading cross-section parameters (type, num)\n");
    return(1);
  }
  // allocating apropriate ammount of memory
  prop = new char*[nprop];
  memset(prop, 0, sizeof(*prop)*nprop);
  propu = new long[nprop];
  memset(propu, 0, sizeof(*propu)*nprop);
  ridx = new long[nprop];
  memset(ridx, 0, sizeof(*ridx)*nprop);

  // Reading strings with cross-section properties for each cross-section index
  //  each line in the input file corresponds to one string with cross-section
  //  properties of one index
  for (long i=0; i < nprop; i++)
  {
    prop[i] = new char[1025];
    if (getstring(in, prop[i], 1024))
    {
      fprintf(stderr, "\nError reading cross-section parameters (properties string)\n");
      return(2);
    }
  }
  for (long i = 0; i < nprop; i++)
  {
    if (prop[i][strlen(prop[i])-1] == '\n')
    // replace last character with 0 if equal newline
      prop[i][strlen(prop[i])-1] = '\0';
  }
  return(0);
}



dbcrs :: dbcrs(void)
{
  numt = 0;
  crs = NULL;
}

dbcrs :: ~dbcrs(void)
{
  delete [] crs;
}

long dbcrs :: search_crs(crsectypet ce, long ci)
// Function searches database whether contains cross-section with
//  type number ce and type index number ci.
// Parameters :
//  ce - cross-section type number
//  ci - cross-section type index
// Returns :
//  If the cross-section with given parameters exists in the database function returns
//  index of given cross-section in the array mat else returns -1.
{
  long ret = -1;
  for (long i = 0; i < numt; i++)
  {
    if ((crs[i].type == ce) && ((ci >= 0) && (ci < crs[i].nprop)))
    {
      ret = i;
      break;
    }
  }
  return(ret);
}
