#include <string.h>
#include <math.h>
#include "edgeload.h"
#include "globprep.h"
#include "intools.h"
#include "mathem.h"

edgeload::edgeload()
{
   nlc = 0L;
   xa = ya = za = 0.0;
   xb = yb = zb = 0.0;

   tc   =  0L;
   //tf   = timefunct(-1);
   neq  =  0L;
   fa   =  fb   = 0.0;
   eqa  =  eqb  = NULL;
   fta  =  ftb  = NULL;
   taba =  tabb = NULL;
   lva  =  lvb  = NULL;
   trca  =  0.0;
   trcb  =  0.0;
}

edgeload::~edgeload()
{
  for (long i = 0; i < neq; i++)
  {
    delete eqa[i];
    delete eqb[i];
    delete [] fta[i];
    delete [] ftb[i];
  }
  delete [] eqa;
  delete [] eqb;
  delete [] fta;
  delete [] ftb;
  delete [] lva;
  delete [] lvb;
  //delete [] taba;//tady to pada
  //delete [] tabb;
}

long edgeload::read(FILE *in, long lc)
{
  long i;
  long tmp;
  Parser par;

  getlong(in, nlc);
  if ((nlc < 1) || (nlc > lc))
    return(2);
  getlong(in, tc);
  if ( ! ((tc == 2) || (tc==4) || (tc >= 30)))
  {
    fprintf(stderr, "\n\n Unknown type of condition is required in function edgeload::read().\n");
    fprintf(stderr, "(file %s, line %d)\n", __FILE__, __LINE__);
    return (1);
  }
  
  if ((tc == 2) || (tc >= 30)){
    getlong(in, tmp);
    tf = generalfunct (tmp);
    switch (tf)
      {
      case stat :
	break;
      case pars :
	neq = 1;
	eqa = new Equation *[neq];
	eqb = new Equation *[neq];
	fta = new char *[neq];
	ftb = new char *[neq];
	fta[0] = new char[256];
	ftb[0] = new char[256];
	break;
      case tab  :
	taba = new tablefunct;
	tabb = new tablefunct;
	getlong(in, i);
	tabb->asize = taba->asize = i;
	taba->x = new double[i];
	taba->y = new double[i];
	tabb->x = new double[i];
	tabb->y = new double[i];
	getlong(in, i);
	tabb->itype = taba->itype = interpoltype(i);
	break;
      case pars_set :
	getlong(in, neq);
	eqa = new Equation *[neq];
	eqb = new Equation *[neq];
	fta = new char *[neq];
	ftb = new char *[neq];
	lva = new double[neq];
	lvb = new double[neq];
	for (i = 0; i < neq; i++)
	  {
	    fta[i] = new char[256];
	    ftb[i] = new char[256];
	  }
	break;
      default :
	fprintf(stderr, "\n\n Unknown type of time function is required in function edgeload::read().\n");
	fprintf(stderr, "(file %s, line %d)\n", __FILE__, __LINE__);
	return(2);
      }
    // first point
    getdouble(in, xa);
    getdouble(in, ya);
    getdouble(in, za);
    switch (tf)
      {
      case stat :
	getdouble(in, fa);
	break;
      case pars :
	getstring(in, fta[0], 255);
	eqa[0] = par.TextToTree(fta[0]);
	if (eqa[0] == NULL)
	  {
	    fprintf(stderr, "\n\n Unable to parse function edgeload::read().\n");
	    fprintf(stderr, "(file %s, line %d)\n", __FILE__, __LINE__);
	    return (3);
	  }
	break;
      case tab  :
	for (i = 0; i < taba->asize; i++)
	  {
	    getdouble(in, taba->x[i]);
	    getdouble(in, taba->y[i]);
	  }
	break;
      case pars_set :
	for (i = 0; i < neq; i++)
	  {
	    getdouble(in, lva[i]);
	    getstring(in, fta[i], 255);
	    eqa[i] = par.TextToTree(fta[i]);
	    if (eqa[i] == NULL)
	      {
		fprintf(stderr, "\n\n Unable to parse function edgeload::read().\n");
		fprintf(stderr, "(file %s, line %d)\n", __FILE__, __LINE__);
		return (3);
	      }
	  }
	break;
      default :
	fprintf(stderr, "\n\n Unknown type of time function is required in function edgeload::read().\n");
	fprintf(stderr, "(file %s, line %d)\n", __FILE__, __LINE__);
	return (2);
      }
    if (tc >= 30)
      getdouble(in, trca);
    // second point
    getdouble(in, xb);
    getdouble(in, yb);
    getdouble(in, zb);
    switch (tf)
      {
      case stat :
	getdouble(in, fb);
	break;
      case pars :
	getstring(in, ftb[0], 255);
	eqb[0] = par.TextToTree(ftb[0]);
	if (eqb[0] == NULL)
	  {
	    fprintf(stderr, "\n\n Unable to parse function edgeload::read().\n");
	    fprintf(stderr, "(file %s, line %d)\n", __FILE__, __LINE__);
	    return (3);
	  }
	break;
      case tab  :
	for (i = 0; i < tabb->asize; i++)
	  {
	    getdouble(in, tabb->x[i]);
	    getdouble(in, tabb->y[i]);
	  }
	break;
      case pars_set :
	for (i = 0; i < neq; i++)
	  {
	    getdouble(in, lvb[i]);
	    getstring(in, ftb[i], 255);
	    eqb[i] = par.TextToTree(ftb[i]);
	    if (eqb[i] == NULL)
	      {
		fprintf(stderr, "\n\n Unable to parse function edgeload::read().\n");
		fprintf(stderr, "(file %s, line %d)\n", __FILE__, __LINE__);
		return (3);
	      }
	  }
	break;
      default :
	fprintf(stderr, "\n\n Unknown type of time function is required in function edgeload::read().\n");
	fprintf(stderr, "(file %s, line %d)\n", __FILE__, __LINE__);
	return (2);
      }
    if (tc >= 30)
      getdouble(in, trcb);
    return(0);
  }
  if (tc==4){
    getlong(in, ccid);
    return (0);
  }
  return 0;
}

long edgeload::getval(snode *tn, double a, double b, double &nv)
{
  double maxx, minx, maxy, miny, maxz, minz;
  (xa < xb) ? (maxx = xb, minx = xa) : (maxx = xa, minx = xb);
  (ya < yb) ? (maxy = yb, miny = ya) : (maxy = ya, miny = yb);
  (za < zb) ? (maxz = zb, minz = za) : (maxz = za, minz = zb);
  if ((tn->x < minx) || (tn->x > maxx))
    return(1);
  if ((tn->y < miny) || (tn->y > maxy))
    return(1);
  if ((tn->z < minz) || (tn->z > maxz))
    return(1);

  double l, ksi;
  l = sqrt(sqr(xb - xa) + sqr(yb - ya) + sqr(zb - za));
  ksi = sqrt(sqr(tn->x - xa) + sqr(tn->y - ya) + sqr(tn->z - za));
  nv = ksi / l * (b - a) + a;

  return(0);
}

long edgeload::getval(snode *tn, long i, char *eqtxt)
{
  double maxx, minx, maxy, miny, maxz, minz;
  (xa < xb) ? (maxx = xb, minx = xa) : (maxx = xa, minx = xb);
  (ya < yb) ? (maxy = yb, miny = ya) : (maxy = ya, miny = yb);
  (za < zb) ? (maxz = zb, minz = za) : (maxz = za, minz = zb);
  if ((tn->x < minx) || (tn->x > maxx))
    return(1);
  if ((tn->y < miny) || (tn->y > maxy))
    return(1);
  if ((tn->z < minz) || (tn->z > maxz))
    return(1);
  if (i > neq-1)
    return(2);
  double l, ksi;
  l = sqrt(sqr(xb - xa) + sqr(yb - ya) + sqr(zb - za));
  ksi = sqrt(sqr(tn->x - xa) + sqr(tn->y - ya) + sqr(tn->z - za));
//  nv = ksi / l * (fb[i] - fa[i]) + fa[i];
  sprintf(eqtxt, "%e*(", ksi/l);
  strcat(eqtxt, ftb[i]);
  sprintf(eqtxt+strlen(eqtxt),"-");
  strcat(eqtxt, fta[i]);
  sprintf(eqtxt+strlen(eqtxt),")+");
  strcat(eqtxt, fta[i]);

  return(0);
}
