#include "globprep.h"
#include "output.h"
#include <string.h>

long output(FILE *out, descrip *d)
{
  // Output data in mechtop format
  long err;
  long npropn, nprope;
  npropn = Npropn;
  nprope = Nprope;

  fprintf(stdout, "\n\nOutput of TRFEL input file :");
  fprintf(stdout, "\n writing nodes . . .");
  err = wr_nodes(out, npropn); // writing nodes, crsecs in the nodes and local systems
  fflush(out);
  if (err)
    return(7);
  fprintf(stdout, " O.K.");

  fprintf(stdout, "\n writing boundary conditions . . .");
  err = wr_bocon(out, npropn); // writing boundary conditions
  fflush(out);
  if (err)
    return(7);
  fprintf(stdout, " O.K.");

  fprintf(stdout, "\n writing elements . . .");
  err = wr_elements(out, nprope); // writing elements and code numbers on elements
  fflush(out);
  if (err)
    return(7);
  fprintf(stdout, " O.K.");

  if (!(d->seq))
  {
    fprintf(stdout, "\n writing global node numbers . . .");
    err = wr_globnodnum(out);
    if (err)
      return(7);
    fprintf(stdout, " O.K.");
  }

/*  err = wr_intpoints(out, nprope); // writing integration points of elements
                                   // with indeces of material parameters
  fflush(out);
  if (err)
    return(7);
*/
  fprintf(stdout, "\n writing materials . . .");
  err = wr_materials(out); // writing material parameters
  fflush(out);
  if (err)
    return(7);
  fprintf(stdout, " O.K.");

  fprintf(stdout, "\n writing cross-sections . . .");
  err = wr_crsecs(out);    // writing cross section parameters
  fflush(out);
  if (err)
    return(7);
  fprintf(stdout, " O.K.");

  fprintf(stdout, "\n writing load . . .");
  err = wr_load(out, npropn, nprope); 
  fflush(out);
  if (err)
    return(7);
  fprintf(stdout, " O.K.");

  if ((Tp->tprob == nonstationary_problem) || (Tp->tprob == nonlinear_nonstationary_problem))
  {
    fprintf(stdout, "\n writing initial conditions . . .");
    err = wr_initcond(out, npropn);
    fflush(out);
    if (err)
      return(7);
    fprintf(stdout, " O.K.");
  }

  fprintf(stdout, "\n\n");

  // deallocating used memory
  long i;
  for (i = 0; i < npropn; i++)
  {
    delete [] Boccn[i];
    delete Inic[i];
    delete Sl[i];
  }
  for (i = 0; i < nprope; i++)
  {
    delete [] Mel[i];
    delete [] Melid[i];
    delete Edgl[i];
    delete Boce[i];
  }
  delete [] Boccn;
  delete [] Nnp;
  delete [] Nep;
  delete [] Tel;
  delete [] Sel;
  delete [] Mel;
  delete [] Melid;
  delete [] Nmel;
  delete [] Ncs;
  delete [] Ncsi;
  delete [] Ecs;
  delete [] Ecsi;
  delete Dbmat;
  delete Dbcrs;
  delete [] Tet;
  delete [] Nmu;
  delete [] Ncu;
  delete [] Inic;
  delete [] Edgl;
  delete [] Sl;
  delete [] Surfbc;
  delete [] Boce;
  delete Top;
  return (0);
}



long wr_nodes(FILE *out, long nprop)
/**<
 Function writes section with description of nodes to the text file given by out.

 @param out - pointer to the opened text file, where the data will be written
 @param nprop - number of entries in the input property file

 @retval 0 - if succes
 @retval 1 - if node hasn't assigned ndof
 @retval 2 - if node hasn't assigned cross-section
*/
{
  long i, j, k, *ic, ncf, rindex;

  ic = new long[Top->nn];

  fprintf(out, "%ld\n", Top->nn);
  for (i = 0; i < Top->nn; i++)
  {
    for (j = 0, ncf = 0; j < nprop; j++)
    // loop over all properties from input
    {
      if (Top->nodes[i].prop[0] == Nnp[j])
      // test if number of property matchs number from input
      {
        if (Ncs[j])
        // test if any cross-section is assigned
        {
          if ((ic[i] = Dbcrs->search_crs(Ncs[j], Ncsi[j]-1)) < 0) // searching cs in the cs database
          {
            fprintf(stderr,
                "\nError - cross-section with number %d and index %ld in node %ld not found\n", Ncs[j], Ncsi[j], i+1);
            delete [] ic;
            return(2);
          }
          else
          {
            if (Dbcrs->crs[ic[i]].propu[Ncsi[j]-1] == 0)
              Dbcrs->crs[ic[i]].npropu++; // indicator whether this cs type is used
            if (Ncu[ic[i]] == 0)
              Ncu[ic[i]]++;   // indicator whether cs type from cs database is used in the given task
            Dbcrs->crs[ic[i]].propu[Ncsi[j]-1]++; // indicator whether this index of cs is used
          }
          ncf = 1;
          break;
        }
      }
    }
    if (ncf == 0)
    {
      fprintf(stderr, "\nError - node number %ld hasn't assigned cross-section\n", i+1);
      delete [] ic;
      return (2);
    }
  }
  for (j = 0; j < Dbcrs->numt; j++)
  {
    if (Dbcrs->crs[j].npropu)
    {
      rindex = 1;
      for (k = 0; k < Dbcrs->crs[j].nprop; k++)
      {
        if (Dbcrs->crs[j].propu[k])
        {
          Dbcrs->crs[j].ridx[k] = rindex;
          rindex++;
        }
      }
    }
  }
  Csrin = rindex;
  for (i = 0; i < Top->nn; i++)
  {
    // writing node number and coordinates
    fprintf(out, "%6ld % e % e % e %ld", i+1, Top->nodes[i].x, Top->nodes[i].y, Top->nodes[i].z, Ndofn);
    for (j = 0; j < nprop; j++)
    // loop over all properties from input
    {
      if (Top->nodes[i].prop[0] == Nnp[j])
      // test if number of property matchs number from input
      {
        if (Ncs[j])
        // test if any cross-section is assigned
        {
          fprintf(out, " %d %ld\n", Ncs[j], Dbcrs->crs[ic[i]].ridx[Ncsi[j]-1]);
          break;
        }
      }
    }
  }
  fprintf(out, "\n");
  delete [] ic;
  return(0);
}



/**
 Function writes section with description of boundary conditions of nodes
  to the text file given by out.

 @param out - pointer to the opened text file, where the data will be written
 @param nprop - number of entries in the input property file

 @retval 0 - if succes
 @retval 1 - if nodes haven't assigned any boundary condition
*/
long wr_bocon(FILE *out, long nprop)
{
  long i, j, k, nbc, tcn;

  nbc = 0;
  tcn = 0;
  Nspv = 0;
  Boccn = new long *[nprop];
  memset(Boccn, 0, nprop*sizeof(*Boccn));
  for (i = 0; i < Top->nn; i++)
  {
    for (j = 0; j < nprop; j++)
    // loop over all properties from input
    {
      if (Top->nodes[i].prop[0] == Nnp[j])
      // test if number of property matchs number from input
      {
        if (Boc[j])
        // test if any boundary condition is prescribed
        {
          nbc++;
          if (Boccn[j] == NULL)
          {
            Boccn[j] = new long[Ndofn];
            memset(Boccn[j], 0, Ndofn * sizeof(long));
            for (k = 0; k < Ndofn; k++)
            // loop for assigning bc code numbers
            // if direction has bc, increase cn
            {
              if (Boc[j]->dir[k] >= 0)
              {
                tcn++;
                Boccn[j][k] = tcn;
                Nspv++;
              }
            }
          }
        }
        if (Nccn[j])
        // test if any common code numbers are prescribed
          nbc++;
      }
    }
  }
  fprintf(out, "%ld\n", nbc);
  for (i = 0; i < Top->nn; i++)
  {
    for (j = 0; j < nprop; j++)
    // loop over all properties from input
    {
      if (Top->nodes[i].prop[0] == Nnp[j])
      // test if number of property matchs number from input
      {
        if ((Boc[j]) || (Nccn[j]))
        // test if any boundary condition is prescribed or any common code numbers are prescribed
        {
          fprintf(out, "%ld", i+1);
          for (k = 0; k < Ndofn; k++)
          // loop for writing bc
          // if direction has bc, write 0 else write 1
          {
            if (Boc[j] && Nccn[j])
            {
              if ((Boc[j]->dir[k] > 0) && (Nccn[j][k]))
              {
                fprintf(stderr, "\n\nError - node %ld has prescribed boundary condition and common code number\n", i+1);
                fprintf(stderr, " in the same direction\n.");
                fprintf(stderr, " function wr_bocon (input.cpp)\n.");
                return(2);
              }
              if ((Boc[j]->dir[k] < 0) && (Nccn[j][k] == 0))
              // no boundary condition nor common code number
              {
                fprintf(out, " 1");
                continue;
              }
            }
            if (Boc[j])
            {
              if (Boc[j]->dir[k] < 0)
              // no boundary condition
              {
                fprintf(out, " 1");
                continue;
              }
              else
              // prescribed value
              {
                fprintf(out, " %ld", -Boccn[j][k]);
                continue;
              }
            }
            if (Nccn[j])
            // common code number
            {
              if (Nccn[j][k])
              {
                fprintf(out, " %ld", Nccn[j][k]);
                continue;
              }
              if (Nccn[j][k] == 0)
              {
                fprintf(out, " 1");
                continue;
              }
            }
          }
          fprintf(out, "\n");
          break;
        }
      }
    }
  }
  fprintf(out, "\n");
  return(0);
}



/**
 Function writes section with description of elements to the text file given by out.
  @param out - pointer to the opened text file, where the data will be written
  @param nprop - number of entries in the input property file

  @retval 0 : on succes
  @retval 1 : if element hasn't assigned type
*/
long wr_elements(FILE *out, long nprop)
{
  long i, j, k, l;
  crsectypet *ce;
  long *ci, *ic;
  long rindex;
  elemtypet et;

  mattypet **me;
  long **mi, **im, *nm;

  me   = new mattypet* [Top->ne];
  memset(me, 0, sizeof(*me)*Top->ne);
  mi   = new long* [Top->ne];
  memset(mi, 0, sizeof(*mi)*Top->ne);
  im   = new long* [Top->ne];
  memset(im, 0, sizeof(*im)*Top->ne);
  nm   = new long  [Top->ne];

  Tet = new elemtypet[Top->ne];
  memset(Tet, 0, sizeof(*Tet)*Top->ne);
  ce   = new crsectypet [Top->ne];
  memset(ce, 0, sizeof(*ce)*Top->ne);
  ci   = new long  [Top->ne];
  memset(ci, 0, sizeof(*ci)*Top->ne);
  ic   = new long  [Top->ne];

  fprintf(out, "%ld\n", Top->ne);
  for (i = 0; i < Top->ne; i++)
  {
    for (j = 0, et = elemtypet(-1); j < nprop; j++)
    // loop over all properties from input
    {
      // test if number of property matchs number from input
      if ((Top->elements[i].prop == Nep[j]) && (Sel[j] == Top->elements[i].type))
      {
        if (Tel[j] >= 0)
        // assigning element type
        {
          et = Tel[j];
          break;
        }
      }
    }
    if (et < 0)
    {
      fprintf(stderr, "\nError - element number %ld hasn't assigned type\n", i+1);
      for (k = 0; k < Top->ne; k++)
      {
        delete [] me[k]; delete [] mi[k]; delete [] im[k];
      }
      delete [] me; delete [] mi; delete [] nm; delete [] im;
      delete [] ce; delete [] ci; delete [] ic;
      return(1);
    }
    for (j = 0; j < nprop; j++)
    // loop over all properties from input
    {
      if (Ecs[j])
      // assigning cross-section to the element with number i
      {
        ce[i] = Ecs[j];
        ci[i] = Ecsi[j];
      }
    }
    if (ce[i] > 0)
    {
      if ((ic[i] = Dbcrs->search_crs(ce[i], ci[i]-1)) < 0)
      {
        fprintf(stderr, "\nError - cross-section with number %d and index %ld not found\n", ce[i], ci[i]);
        for (k = 0; k < Top->ne; k++)
        {
          delete [] me[k]; delete [] mi[k]; delete [] im[k];
        }
        delete [] me; delete [] mi; delete [] nm; delete [] im;
        delete [] ce; delete [] ci; delete [] ic;
        return(2);
      }
      else
      {
        if (Dbcrs->crs[ic[i]].propu[ci[i]-1] == 0)
          Dbcrs->crs[ic[i]].npropu++; // indicator whether given cs type is used
        if (Ncu[ic[i]] == 0)
          Ncu[ic[i]]++; // indicator whether cs type from cs database is used in the given task
        Dbcrs->crs[ic[i]].propu[ci[i]-1]++;  // indicator whether given cs index is used
      }
    }
    for (j = 0; j < Dbcrs->numt; j++)
    {
      if (Dbcrs->crs[j].npropu)
      {
        rindex = 1;
        for (k = 0; k < Dbcrs->crs[j].nprop; k++)
        {
          if (Dbcrs->crs[j].propu[k])
          {
            if (Dbcrs->crs[j].ridx[k] != 0)
              rindex++;
          }
        }
        for (k = 0; k < Dbcrs->crs[j].nprop; k++)
        {
          if (Dbcrs->crs[j].propu[k])
          {
            if (Dbcrs->crs[j].ridx[k] == 0)
            {
              Dbcrs->crs[j].ridx[k] = rindex;
              rindex++;
            }
          }
        }
      }
    }
    nm[i] = 0;
    for (j = 0; j < nprop; j++)
    // loop over all properties from input
    {
      // test if number of property matchs number from input
      if (Top->elements[i].prop == Nep[j])
      {
        if (Mel[j])
        // assigning matrial to the element with number i
        {
          nm[i] = Nmel[j];
          me[i] = new mattypet[Nmel[j]];
          mi[i] = new long[Nmel[j]];
          im[i] = new long[Nmel[j]];
          for (k = 0; k < Nmel[j]; k++)
          {
            me[i][k] = Mel[j][k];
            mi[i][k] = Melid[j][k];
          }
        }
      }
    }
    if (me[i] ==  NULL) // unassigned element material
    {
      fprintf(stderr, "\nError - element %ld hasn't assigned material\n", i+1);
      for (k = 0; k < Top->ne; k++)
      {
        delete [] me[k]; delete [] mi[k]; delete [] im[k];
      }
      delete [] me; delete [] mi; delete [] nm; delete [] im;
      delete [] ce; delete [] ci; delete [] ic;
      return(3);
    }
    for (j = 0; j < nm[i]; j++)
    {
      // searching element material in the material database
      if ((im[i][j] = Dbmat->search_mat(me[i][j], mi[i][j]-1)) < 0)
      {
        fprintf(stderr, "\nError - material with number %d and index %ld not found\n", me[i][j], mi[i][j]);
        for (k = 0; k < Top->ne; k++)
        {
          delete [] me[k]; delete [] mi[k]; delete [] im[k];
        }
        delete [] me; delete [] mi; delete [] nm;
        delete [] ce; delete [] ci; delete [] ic;
        return(4);
      }
      else
      {
        if (Dbmat->mat[im[i][j]].propu[mi[i][j]-1] == 0)
          Dbmat->mat[im[i][j]].npropu++; // indicator whether given material type is used
        if (Nmu[im[i][j]] == 0)
          Nmu[im[i][j]]++;  // indicator whether material type from material database is used in the given task
        Dbmat->mat[im[i][j]].propu[mi[i][j]-1]++; // indicator whether given material index is used
      }
    }
    for (j = 0; j < Dbmat->numt; j++)
    {
      if (Dbmat->mat[j].npropu)
      {
        rindex = 1;
        for (k = 0; k < Dbmat->mat[j].nprop; k++)
        {
          if (Dbmat->mat[j].propu[k])
          {
            Dbmat->mat[j].ridx[k] = rindex;
            rindex++;
          }
        }
      }
    }
    Tt->elements[i].te = et;
    // writing type of element, element number
    fprintf(out, "%6ld ", i+1);
    fprintf(out, "%3d", et);
    Tet[i] = et;
    // writing element nodes
    for (j = 0; j < Top->elements[i].nne; j++)
      fprintf(out, " %6ld", Top->elements[i].nodes[j]+1);
    // no code numbers on element (code numbers not supported at this time)
    fprintf(out, " 0");
    if (ce[i] > 0)
      fprintf(out, " %d %ld", ce[i], Dbcrs->crs[ic[i]].ridx[ci[i]-1]); // if element has cs, write cs type and index
    else
      fprintf(out, " 0"); // writing element cross-section indicator
    if (nm[i] != (Tp->ntm)*(Tp->ntm))
    {
      fprintf(stderr, "\nError - incompatible number of materials on element %ld\n", i+1);
      for (k = 0; k < Top->ne; k++)
      {
        delete [] me[k]; delete [] mi[k]; delete [] im[k];
      }
      delete [] me; delete [] mi; delete [] nm; delete [] im;
      delete [] ce; delete [] ci; delete [] ic;
      return (5);
    }
    for (l = 0; l < nm[i]; l++)
      fprintf(out, " %d %ld", me[i][l], Dbmat->mat[im[i][l]].ridx[mi[i][l]-1]); // writing element material type and index
    fprintf(out, "\n");
  }
  fprintf(out, "\n");
  for (k = 0; k < Top->ne; k++)
  {
    delete [] me[k]; delete [] mi[k]; delete [] im[k];
  }
  delete [] me; delete [] mi; delete [] nm; delete [] im;
  delete [] ce; delete [] ci; delete [] ic;
  return(0);
}


/**<
 Function writes section with description of elements to the text file given by out.

 @param out - pointer to the opened text file, where the data will be written

 @retval 0 : on succes
*/
long wr_globnodnum(FILE *out)
{
  long i;
  if (Top->nsd>0){
    fprintf (out,"%ld %ld\n",Top->meshtype,Top->nsd);
    for (i=0;i<Top->nsd;i++){
      fprintf (out," %ld",Top->nnsd[i]);
    }
    fprintf (out,"\n");
  }
  for (i = 0; i < Top->nn; i++)
    fprintf(out, "%ld\n", Top->gnn[i]);
  return(0);
}

/**
 Function writes section with description of elements to the text file given by out.

  @param out - pointer to the opened text file, where the data will be written
  @param nprop - number of entries in the input property file

  @retval 0 : if succes
  @retval 1 : if element hasn't assigned material
  @retval 2 : if element hasn't assigned type
  @retval 3 : if element hasn't assigned cross-section
*/
/*
long wr_intpoints(FILE *out, long nprop)
{
  long i, j, k, l, ii, nip;
  mattype **me;
  long **mi, **im, *nm;
  crsectype *ce;
  long *ci, *ic;
  elemtype et = elemtype(-1);
  long rindex;

  me   = new mattype* [Top->ne];
  memset(me, 0, sizeof(*me)*Top->ne);
  mi   = new long* [Top->ne];
  memset(mi, 0, sizeof(*mi)*Top->ne);
  im   = new long* [Top->ne];
  memset(im, 0, sizeof(*im)*Top->ne);
  nm   = new long  [Top->ne];
  ce   = new crsectype [Top->ne];
  ci   = new long  [Top->ne];
  memset(ci, 0, sizeof(*ci)*Top->ne);
  ic   = new long  [Top->ne];

//  Nmu = 0;
  for (i = 0; i < Top->ne; i++)
  {
    nm[i] = 0;
    ce[i] = crsectype(-1);
    for (j = 0; j < nprop; j++)
    // loop over all properties from input
    {
      // test if number of property matchs number from input
      if (Top->elements[i].prop == Nep[j])
      {
        if (Mel[j])
        // assigning matrial to the element with number i
        {
          nm[i] = Nmel[j];
          me[i] = new mattype[Nmel[j]];
          mi[i] = new long[Nmel[j]];
          im[i] = new long[Nmel[j]];
          for (k = 0; k < Nmel[j]; k++)
          {
            me[i][k] = Mel[j][k];
            mi[i][k] = Melid[j][k];
          }
        }
        if (Ecs[j])
        // assigning cross-section to the element with number i
        {
          ce[i] = Ecs[j];
          ci[i] = Ecsi[j];
        }
      }
    }
    et = Tet[i];
    if (et < 0) // unassigned element type
    {
      fprintf(stderr, "\nError - element %ld hasn't assigned type\n", i+1);
      for (k = 0; k < Top->ne; k++)
      {
        delete [] me[k]; delete [] mi[k]; delete [] im[k];
      }
      delete [] me; delete [] mi; delete [] ce; delete [] nm;
      delete [] ci; delete [] im; delete [] ic;
      return(1);
    }
    if (me[i] ==  NULL) // unassigned element material
    {
      fprintf(stderr, "\nError - element %ld hasn't assigned material\n", i+1);
      for (k = 0; k < Top->ne; k++)
      {
        delete [] me[k]; delete [] mi[k]; delete [] im[k];
      }
      delete [] me; delete [] mi; delete [] ce; delete [] nm;
      delete [] ci; delete [] im; delete [] ic;
      return(2);
    }


    for (j = 0; j < nm[i]; j++)
    {
      // searching element material in the material database
      if ((im[i][j] = Dbmat->search_mat(me[i][j], mi[i][j]-1)) < 0)
      {
        fprintf(stderr,
                "\nError - material with number %d and index %ld not found\n", me[i][j], mi[i][j]);
        for (k = 0; k < Top->ne; k++)
        {
          delete [] me[k]; delete [] mi[k]; delete [] im[k];
        }
        delete [] me; delete [] mi; delete [] ce; delete [] nm;
        delete [] ci; delete [] im; delete [] ic;
        return(3);
      }
      else
      {
        if (Dbmat->mat[im[i][j]].propu[mi[i][j]-1] == 0)
          Dbmat->mat[im[i][j]].npropu++; // indicator whether given material type is used
        if (Nmu[im[i][j]] == 0)
          Nmu[im[i][j]]++;  // indicator whether material type from material database is used in the given task
        Dbmat->mat[im[i][j]].propu[mi[i][j]-1]++; // indicator whether given material index is used
      }
    }
    if (ce[i] > 0)
    {
      if ((ic[i] = Dbcrs->search_crs(ce[i], ci[i]-1)) < 0)
      {
        fprintf(stderr,
                "\nError - cross-section with number %d and index %ld not found\n", ce[i], ci[i]);
        for (k = 0; k < Top->ne; k++)
        {
          delete [] me[k]; delete [] mi[k]; delete [] im[k];
        }
        delete [] me; delete [] mi; delete [] ce; delete [] nm;
        delete [] ci; delete [] im; delete [] ic;
        return(4);
      }
      else
      {
        if (Dbcrs->crs[ic[i]].propu[ci[i]-1] == 0)
          Dbcrs->crs[ic[i]].npropu++; // indicator whether given cs type is used
        if (Ncu[ic[i]] == 0)
          Ncu[ic[i]]++; // indicator whether cs type from cs database is used in the given task
        Dbcrs->crs[ic[i]].propu[ci[i]-1]++;  // indicator whether given cs index is used
      }
    }
  }
  for (j = 0; j < Dbmat->numt; j++)
  {
    if (Dbmat->mat[j].npropu)
    {
      rindex = 1;
      for (k = 0; k < Dbmat->mat[j].nprop; k++)
      {
        if (Dbmat->mat[j].propu[k])
        {
          Dbmat->mat[j].ridx[k] = rindex;
          rindex++;
        }
      }
    }
  }
  for (j = 0; j < Dbcrs->numt; j++)
  {
    if (Dbcrs->crs[j].npropu)
    {
      for (k = 0; k < Dbcrs->crs[j].nprop; k++)
      {
        if (Dbcrs->crs[j].propu[k])
        {
          if (Dbcrs->crs[j].ridx[k] == 0)
          {
            Dbcrs->crs[j].ridx[k] = rindex;
            rindex++;
          }
        }
      }
    }
  }
  for (i = 0; i < Top->ne; i++)
  {
    for (j = 0; j < Tp->ntm; j++)
    {
      for (k = 0; k < Tp->ntm; k++)
      {
        l = j*Tp->ntm + k;
        nip = Tt->give_nip(i, j, k);
	if ((l >= nm[i]) || (nm[i] != Tp->ntm*Tp->ntm))
	{
	  fprintf(stderr, "\n\nError - bad number of materials\n");
	  fprintf(stderr, " element %ld has assigned %ld materials,\n", i+1, nm[i]);
	  fprintf(stderr, " but needs %ld, in function wr_intpoints\n", Tp->ntm*Tp->ntm);
	  fprintf(stderr, " (file %s, line %d)\n", __FILE__, __LINE__);
	  return(5);
	}
        for (ii = 0; ii < nip; ii++)
          fprintf(out, " %d %ld", me[i][l], Dbmat->mat[im[i][l]].ridx[mi[i][l]-1]); // writing element material type and index
        fprintf(out, "\n");
      }
    }
    fprintf(out, "\n");
  }
  fprintf(out, "\n");
  for (k = 0; k < Top->ne; k++)
  {
    delete [] me[k];
    delete [] mi[k];
    delete [] im[k];
  }
  delete [] me; delete [] mi; delete [] ce; delete [] nm;
  delete [] ci; delete [] im; delete [] ic;
  return(0);
}
*/


/**
 Function writes materials and thier parameters to the file given by the out.

  @param out - pointer to the opened text file, where the data will be written

  @retval 0 : on succes
*/
long wr_materials(FILE *out)
{
  long i, j, nmtu, wrt;
  for (i = 0, nmtu = 0; i < Dbmat->numt; i++)
  // countig number of used material types
  {
    if (Nmu[i])
      nmtu++;
  }
  fprintf(out, "%ld\n", nmtu);  // writing number of material types
  for (i = 0; i < Dbmat->numt; i++)
  {
    wrt = 1;
    for (j = 0; j < Dbmat->mat[i].nprop; j++)
    {
      if (Dbmat->mat[i].propu[j])  // material type index is used (<> 0)
      {
        if (wrt)  // for the first time write material type number and number of used indeces
        {
          fprintf(out, "%d %ld\n", Dbmat->mat[i].type, Dbmat->mat[i].npropu);
          wrt = 0;  // next time don't write
        }
        // write material type index and property string
        fprintf(out, "%ld %s\n", Dbmat->mat[i].ridx[j], Dbmat->mat[i].prop[j]);
      }
    }
  }
  fprintf(out, "\n");
  return(0);
}




/**
 Function writes cross-sections and thier parameters to the file given by the out.

  @param out - pointer to the opened text file, where the data will be written

  @retval 0 : on succes
*/
long wr_crsecs(FILE *out)
{
  long i, j, wrt, nctu;

//  fprintf(out, "%ld", Ecsn+Ncsn);
  for(i = 0, nctu = 0; i < Dbcrs->numt; i++)
  // countig number of used cross-section types
  {
    if (Ncu[i])
      nctu++;
  }
  fprintf(out, "%ld\n", nctu);    // writing number of cs types
  for (i = 0; i < Dbcrs->numt; i++)
  {
    wrt = 1;
    for (j = 0; j < Dbcrs->crs[i].nprop; j++)
    {
      if (Dbcrs->crs[i].propu[j])  // cs type index is used (<> 0)
      {
        if (wrt) // for the first time write cs type number and number of used indeces
        {
          fprintf(out, "%d %ld\n", Dbcrs->crs[i].type, Dbcrs->crs[i].npropu);
          wrt = 0;  // next time don't write
        }
        // write cs type index and property string
        fprintf(out, "%ld %s\n", Dbcrs->crs[i].ridx[j], Dbcrs->crs[i].prop[j]);
      }
    }
  }
  fprintf(out, "\n");
  return(0);
}



/**
 Function writes section with description of load to the text file given by out

  @param out - pointer to the opened text file, where the data will be written
  @param npropn - number of entries in the nodal section of property file
  @param nprope - number of entries in the element section of property file

  @retval  0 : on success
  @retval >0 : error numbers from function wr_sourcen or wr_bocon_el
*/
long wr_load(FILE *out, long npropn, long nprope)
{
  long ret = 0;
  fprintf(out, "%ld\n\n\n", Nlc);  // writing number of load cases
  wr_prescval(out, npropn); // writing prescribed values
  for (long i = 0; i < Nlc; i++)
  {
    if (i>0)  fprintf (out,"0\n");
    ret = wr_sourcen(out, npropn, i);    // writing nodal load
    if (ret)
      return ret;
    ret = wr_bocon_el(out, nprope, i);   // writing element load
    if (ret)
      return ret;
    
    fprintf(out, "0\n");
    //  toto je pocet klimatickych podminek
    fprintf(out, "0\n");
    //fprintf(out, "1 1 2\n");
    //Tf[i].print(out);
  }
  fprintf(out, "0\n");
  return(ret);
}





/**
 Function writes section with description of nodal prescribed
  values to the text file given by out.

  @param out   - pointer to the opened text file, where the data will be written
  @param nprop - number of entries in the input property file

  @retval 0 : on succes
*/
long wr_prescval(FILE *out, long nprop)
{
  long i, j, k, n;

  fprintf(out, "%ld\n", Nspv);
  if (Nspv == 0)
    return(0);
  for (n = 0; n < Nspv; n++)
  {
    for (i = 0; i < nprop; i++)
    // loop over all properties from input
    {
      if (Boc[i])
      // test if any boundary condition is prescribed
      {
        for (j = 0; j < Boc[i]->ndir; j++)
        // loop for writing bc
        // if direction has bc, write p.d. number and value
        {
          if (Boccn[i][j] != n+1)
            continue;

          if (Boc[i]->dir[j] >= 0)
          {
	    switch (Tp->tprob)
            {
	      case stationary_problem :
		break;
	      case nonstationary_problem :
	      case nonlinear_nonstationary_problem :
		fprintf(out, " %e", Boc[i]->iv[j]);
		break;
	      default :
		fprintf(stderr, "\n\n Unknown type of problem required in function bocon::read\n");
		fprintf(stderr, "in file %s, line %d\n", __FILE__, __LINE__);
	    }
            
            fprintf(out, " %d", (int)Boc[i]->tf[j]);
            switch (Boc[i]->tf[j])
            {
              case stat :
                fprintf(out, " %e\n", Boc[i]->f[j]);
                break;
              case pars :
                fprintf(out, " %s\n", Boc[i]->ft[j][0]);
                break;
              case tab  :
                fprintf(out, " %d %ld\n", (int)Boc[i]->tabf[j]->itype, Boc[i]->tabf[j]->asize);
                for (k = 0; k < Boc[i]->tabf[j]->asize; k++)
                  fprintf(out, "     %e %e\n", Boc[i]->tabf[j]->x[k], Boc[i]->tabf[j]->y[k]);
                break;
              case pars_set :
                for (k = 0; k < Boc[i]->neq[j]; k++)
                  fprintf(out, "    %e %s\n", Boc[i]->lv[j][k], Boc[i]->ft[j][k]);
                break;
              default :
                fprintf(stderr, "\n\n Unknown type of time function in file %s (line %d)\n\n", __FILE__, __LINE__);
            }
          }
        }
      }
    }
  }
  fprintf(out, "\n");
  return(0);
}



/**
 Function writes section with description of sources in nodes to the text file given by out

  @param out   - pointer to the opened text file, where the data will be written
  @param nprop - number of entries in the input property file
  @param nlc   - number of loading case which will be written

  @retval 0 : on succes
*/
long wr_sourcen(FILE *out, long nprop, long nlc)
{
  long i, j, k, nln;
  for (i = 0, nln = 0; i < Top->nn; i++)
  // countig overall number of nodal loads
  {
    for (j = 0; j < nprop; j++)
    {
      if (Top->nodes[i].prop[0] == Nnp[j])
      {
        if ((Sl[j]) && (Sl[j]->nlc == nlc+1))
          nln++;
      }
    }
  }
  fprintf(out, "%ld\n", nln);  // writing overall number of nodal loading entries
  for (i = 0; i < Top->nn; i++)
  {
    for (j = 0; j < nprop; j++)
    {
      if (Top->nodes[i].prop[0] == Nnp[j])
      {
        if ((Sl[j]) && (Sl[j]->nlc == nlc+1))
        {
          // writing node number
          fprintf(out, "%6ld", i+1);
          // writing load type
          if (Tp->tprob == nonstationary_problem){
	    fprintf(out, " %d", (int)Sl[j]->tf);
	  }
          switch ((int)Sl[j]->tf)
	    {
            case stat :
              fprintf(out, " %e", Sl[j]->f);
              break;
            case pars :
              fprintf(out, " %s", Sl[j]->ft[0]);
              break;
            case tab  :
              fprintf(out, " %d %ld", (int)Sl[j]->tabf->itype, Sl[j]->tabf->asize);
              for (k = 0; k < Sl[j]->tabf->asize; k++)
                fprintf(out, "\n    %e %e", Sl[j]->tabf->x[k], Sl[j]->tabf->y[k]);
              break;
            case pars_set :
              for (k = 0; k < Sl[j]->neq; k++)
                fprintf(out, "\n    %e %s", Sl[j]->lv[k], Sl[j]->ft[k]);
              break;
            default :
              fprintf(stderr, "\n\n Unknown type of time function in file %s (line %d)\n\n", __FILE__, __LINE__);
          }
          fprintf(out, "\n");
        }
      }
    }
  }
  fprintf(out, "\n");
  return(0);
}


/**
  Function writes section with description of boundary conditions on elements to the text file given by out

  @param out   - pointer to the opened text file, where the data will be written
  @param nprop - number of entries in the input property file
  @param nlc   - number of loading case which will be written

  @retval  0 : on succes
  @retval >0 : error numbers of function wr_loadeledg() or wr_boconelsurf()
*/
long wr_bocon_el(FILE *out, long nprop, long nlc)
{
  long ret = 0;
  long nle = 0;

  nle = search_loadedg_el(nprop, nlc);
  nle += search_boconsurf_el(nprop, nlc);

  fprintf(out, "%ld\n", nle);
  ret = wr_loadeledg(out, nprop, nlc);
  if (ret)
    return ret;
  ret = wr_boconelsurf(out, nprop, nlc);
  return ret;    
}

long search_loadedg_el(long nprop, long nlc)
{
  long i, j, k;
  long nl = 0;
  long *elel;
  elel = new long[Top->ne];
  memset(elel, 0, sizeof(*elel)*Top->ne);
  for (i = 0; i < nprop; i++)
  {
    if (Edgl[i])
    {
      if (Edgl[i]->nlc == nlc+1)
      {
        for (j = 0; j < Top->ne; j++)
        {
          for (k = 0; k < Top->elements[j].ned; k++)
          {
            if (Top->elements[j].propedg[k] == Nep[i])
            {
              if (elel[j] == 0)
                nl++;
              elel[j] = 1;
              break;
            }
          }
        }
      }
    }
  }
  delete [] elel;
  return (nl);
}



long wr_loadeledg(FILE *out, long nprop, long nlc)
{
  long i, j, k, l;
  long ned, nned;
  long *enod, *nodval;
  snode *tn;
  double f, fx, fy;
  char ftext[1025];
  
  
  for (i = 0; i < Top->ne; i++)
    {
      ned    = Tt->give_ned(i);
      nned   = Tt->give_nned(i);
      nodval=NULL;
      if (i==335)
	{
          i++;
          i = i-1;
	}
      for (j = 0; j < nprop; j++)
	{
	  if (Edgl[j])
	    {
	      if (Edgl[j]->nlc == nlc+1)
		{
		  for (k = 0; k < ned; k++)
		    {
		      if (Top->elements[i].propedg[k] == Nep[j])
			{
			  if (nodval == NULL)
			    {
			      nodval = new long[ned];
			      memset(nodval, 0, sizeof(*nodval)*ned);
			    }
			  nodval[k] = j+1;
			  //break;
			}
		    }
		}
	    }
	}
      if (nodval){
	fprintf(out, "%ld", i+1);
	fprintf (out, "\n");
	for (j = 0; j < ned; j++){
	  if (nodval[j]){
	    fprintf(out, " %d", (int)Edgl[nodval[j]-1]->tc);
	    if (Edgl[nodval[j]-1]->tc==4)
	      fprintf(out, " %ld",Edgl[nodval[j]-1]->ccid);
	  }
	  else
	    fprintf(out, " 0");
	}
	
	fprintf (out, "\n");
	for (j = 0; j < ned; j++){
	  if (nodval[j] == 0){
	    for (k = 0; k < nned; k++)
	      fprintf(out, " 0 0.0");
	    fprintf (out, "\n");
	    continue;
	  }
	  if (Edgl[nodval[j]-1]->tc==4){
	    for (k = 0; k < nned; k++)
	      fprintf(out, " 0 0.0");
	    fprintf (out, "\n");
	    continue;
	  }
	  if (Edgl[nodval[j]-1]->tc==2 || Edgl[nodval[j]-1]->tc>=30){
	    enod = new long[nned];
	    Tt->give_edge_nodes(i, j, enod);
	    for (k = 0; k < nned; k++){
	      tn = &Top->nodes[enod[k]-1];
	      fprintf(out, "\n %d", (int)Edgl[nodval[j]-1]->tf);
	      switch ((int)Edgl[nodval[j]-1]->tf){
	        case stat :
		  Edgl[nodval[j]-1]->getval(tn, Edgl[nodval[j]-1]->fa, Edgl[nodval[j]-1]->fb, f);
		  fprintf(out, " %e", f);
		  break;
	        case pars :
		  Edgl[nodval[j]-1]->getval(tn, 0, ftext);
		  fprintf(out, "\n     %s", ftext);
		  break;
	        case tab  :
		  fprintf(out, "\n     %d %ld", (int)Edgl[nodval[j]-1]->taba->itype, Edgl[nodval[j]-1]->taba->asize);
		  for (l = 0; l < Edgl[nodval[j]-1]->taba->asize; l++){
		    Edgl[nodval[j]-1]->getval(tn, Edgl[nodval[j]-1]->taba->x[l], Edgl[nodval[j]-1]->tabb->x[l], fx);
		    Edgl[nodval[j]-1]->getval(tn, Edgl[nodval[j]-1]->taba->y[l], Edgl[nodval[j]-1]->tabb->y[l], fy);
		    fprintf(out, "\n     %e %e", fx, fy);
		  }
		  break;
	        case pars_set :
		  for (l = 0; l < Edgl[nodval[j]-1]->neq; l++){
		    Edgl[nodval[j]-1]->getval(tn, Edgl[nodval[j]-1]->lva[l], Edgl[nodval[j]-1]->lvb[l], f);
		    Edgl[nodval[j]-1]->getval(tn, l, ftext);
		    fprintf(out, "\n    %e %s", f, ftext);
		  }
		  break;
	        default :
		  fprintf(stderr, "\n\n Unknown type of time function in file %s (line %d)\n\n", __FILE__, __LINE__);
	      }
	    }
	    fprintf(out, "\n");
	    delete [] enod;
	  }
	}
	fprintf(out, "   ");
	for (j = 0; j < ned; j++){
	  if (nodval[j] == 0){
	    for (k = 0; k < nned; k++)
	      fprintf(out, " 0.0");
	    fprintf (out, "  ");
            continue;
	  }
	  if (Edgl[nodval[j]-1]->tc==4){
	    for (k = 0; k < nned; k++)
	      fprintf(out, " 0.0");
	    fprintf (out, "  ");
	    continue;
	  }
	  if ((Edgl[nodval[j]-1]->tc >= 30) && (Top->elements[i].propedg[j] == Nep[nodval[j]-1])){
	    enod = new long[nned];
	    Tt->give_edge_nodes(i, j, enod);
	    for (k = 0; k < nned; k++){
	      tn = &Top->nodes[enod[k]-1];
	      Edgl[nodval[j]-1]->getval(tn, Edgl[nodval[j]-1]->trca, Edgl[nodval[j]-1]->trcb, f);
	      fprintf(out, " %e", f);
	    }
	    fprintf (out, "  ");
	  }
	  else {
	    for (k = 0; k < nned; k++)
	      fprintf(out, " 0.0");
	    fprintf (out, "  ");
	  }
	}
      }
      if (nodval)
      {
        fprintf(out, "\n");
        delete [] nodval;
        nodval = NULL;
      }
    }

  fprintf(out, "\n");
  return (0);
  
}


/**
  Function computes number of elements with boundary conditions on surfaces

  @param nprop - number of entries in the input property file
  @param nlc   - number of loading case which will be written

  @retval number of elements with a loaded surface
*/
long search_boconsurf_el(long nprop, long nlc)
{
  long i, j, k;
  long nl = 0;
  long *slel;
  slel = new long[Top->ne];
  memset(slel, 0, sizeof(*slel)*Top->ne);
  for (i = 0; i < nprop; i++)
  {
    if (Surfbc[i])
    {
      if (Surfbc[i]->lcid == nlc+1)
      {
        for (j = 0; j < Top->ne; j++)
        {
          for (k = 0; k < Tt->give_nsurf(j); k++)
          {
            if (Top->elements[j].propsurf[k] == Nep[i])
            {
              if (slel[j] == 0)
                nl++;
              slel[j] = 1;
              break;
            }
          }
        }
      }
    }
  }
  for (i = 0; i < nprop; i++)
  {
    if (Boce[i])
    {
      if (Boce[i]->lcid == nlc+1)
      {
        for (j = 0; j < Top->ne; j++)
        {
          if (Top->elements[j].prop == Nep[i])
            nl++;
        }
      }
    }
  }
  delete [] slel;
  return (nl);
}



/**
 Function writes element boundary conditions on surfaces to the text file given by out

  @param out   - pointer to the opened text file, where the data will be written
  @param nprop - number of entries in the input property file
  @param nlc   - number of loading case which will be written

  @retval 0 : on succes
*/
long wr_boconelsurf(FILE *out, long nprop, long nlc)
{
  long i, j, k, l;
  long wele, wels, nbc, nnbc, *surfprop;
  long dim;

  for (i = 0; i < Top->ne; i++)
  {
    dim = Tt->give_dimension(i);
    switch (dim)
    {
      case 1:
        fflush(stderr);
        fprintf(stderr, "\n\n Unknown type of boundary condition is required for dimesion 1\n");
        fprintf(stderr, " in function wr_boconelsurf, (file %s, line %d)", __FILE__, __LINE__);
        abort();
      case 2:
        nbc = Tt->give_ned(i);
        nnbc = Tt->give_nned(i);
        break;
      case 3:
        nbc = Tt->give_nsurf(i);
        nnbc = Tt->give_nnsurf(i);
        break;
      default :
        fflush(stderr);
        fprintf(stderr, "\n\n Unknown type of dimension is required\n");
        fprintf(stderr, " in function wr_boconelsurf, (file %s, line %d)", __FILE__, __LINE__);
        abort();
    }
    surfprop = new long[nbc];
    for (j = 0; j < nbc; j++)
      surfprop[j] = -1;
    wels = 0;
    for (j = 0; j < nprop; j++)
    {
      wele = 0;
      if (Surfbc[j])
      {
        if (Surfbc[j]->lcid == nlc+1)
        {
          for (k=0; k<nbc; k++)
          {
            if ((Top->elements[i].propsurf[k] == Nep[j]) && Surfbc[j])
            {
              wels += 1;
              if (surfprop[k] > -1)
              {
                fflush(stderr);
                fprintf(stderr, "\n\n Unsupported combination of surface boundary condition and element surface boundary condition\n");
                fprintf(stderr, " in function wr_boconelsurf (file : %s, line : %d)\n", __FILE__, __LINE__);
                abort();
              }     
              surfprop[k] = j;
            }
          }
	}
      }
      if (Boce[j])
      {
        if (Boce[j]->nbc != nbc)
	{
          fflush(stderr);
          fprintf(stderr, "\n\n Different number of boundary conditions\n");
          fprintf(stderr, " preprocessor file:%ld, element %ld: %ld.\n", Boce[j]->nbc, i+1, nbc);
          fprintf(stderr, " Function wr_boconelsurf, (file %s, line %d)", __FILE__, __LINE__);
          abort();
	}
        if (Boce[j]->lcid == nlc+1)
        {         
          for (k=0; k<nbc; k++)
          {
            if ((Top->elements[i].prop == Nep[j]) && (Boce[j]->bct[k]))
            {
              wele += 1;
              if (surfprop[k] > -1)
              {
                fflush(stderr);
                fprintf(stderr, "\n\n Unsupported combination of surface boundary condition and element surface boundary condition\n");
                fprintf(stderr, " in function wr_boconelsurf (file : %s, line : %d)\n", __FILE__, __LINE__);
                abort();
              }     
              surfprop[k] = j;
            }
          }
          if (wele) 
            break;
	}
      }
    }
    if ((wele == 0) && (wels == 0))
    {
      delete [] surfprop;
      continue;
    }
    // writing element id
    fprintf(out, "%ld", i+1);
    for (k=0; k<nbc; k++)
    {
      if ((surfprop[k] > 0) && wele){
        fprintf(out, " %ld", Boce[surfprop[k]]->bct[k]);
	if (Boce[surfprop[k]]->bct[k]==4)
	  fprintf (out," %ld",Boce[surfprop[k]]->ccid[k]);
      }
      else
      {
        if ((surfprop[k] > 0) && wels){
          fprintf(out, " %ld", Surfbc[surfprop[k]]->bct);
	  if (Surfbc[surfprop[k]]->bct==4)
	    fprintf (out," %ld",Surfbc[surfprop[k]]->ccid);
        }
        else
          fprintf(out, " 0");
      }
    }
    fprintf(out, "\n");
    for (k=0; k<nbc; k++)
    {
      if ((surfprop[k] >= 0) && wele)
      {
        for(l=0; l<nnbc;l++)
	{
          switch (Boce[surfprop[k]]->bct[k])
	  {
            case 0:
            case 1:
            case 2:
              fflush(stderr);
	      fprintf(stderr, "\n\n Zadany typ okrajove podminky neumime tisknout v dusledku laxniho pristupu\n");
	      fprintf(stderr, " nejmenovanych vyvojovych pracovniku k souboru loadelt.cpp\n");
              fprintf(stderr, " funkce wr_boconelsurf (file %s, line %d)\n", __FILE__, __LINE__);
              abort();
            case 4:
            case 30:
            case 90:
	    {
              switch (Tp->tprob)
	      {
                case stationary_problem:              
                  fprintf(out, " %le", Boce[surfprop[k]]->pv[k]);
                  break;
                case nonstationary_problem:
                case nonlinear_nonstationary_problem:
                  Boce[surfprop[k]]->tpv[k]->print(out);
                  break;
                default :
                  fflush(stderr);
                  fprintf(stderr, "\n\n Unknown type of problem required in function wr_boconelsurf\n");
                  fprintf(stderr, "in file %s, line %d\n", __FILE__, __LINE__);
                  abort();
	      }
              break;
            }
            default :
              fflush(stderr);
              fprintf(stderr, "\n\n Unknown type of boundary condition is required\n");
              fprintf(stderr, " in function wr_boconelsurf, (file %s, line %d)", __FILE__, __LINE__);
              abort();
	  }
	}
      }
      else
      {
        if ((surfprop[k] >= 0) && wels)
        {
          for(l=0; l<nnbc;l++)
	  {
            switch (Surfbc[surfprop[k]]->bct)
	    {
              case 0:
              case 1:
              case 2:
                fflush(stderr);
	        fprintf(stderr, "\n\n Zadany typ okrajove podminky neumime tisknout v dusledku laxniho pristupu\n");
	        fprintf(stderr, " nejmenovanych vyvojovych pracovniku k souboru loadelt.cpp\n");
                fprintf(stderr, " funkce wr_boconelsurf (file %s, line %d)\n", __FILE__, __LINE__);
                abort();
              case 4:
              case 30:
              case 90:
                {
                switch (Tp->tprob)
                {
                  case stationary_problem:              
                    fprintf(out, " %le", Surfbc[surfprop[k]]->pv);
                    break;
                  case nonstationary_problem:
                  case nonlinear_nonstationary_problem:
                    Surfbc[surfprop[k]]->tpv->print(out);
                    break;
                  default :
                    fflush(stderr);
                    fprintf(stderr, "\n\n Unknown type of problem required in function wr_boconelsurf\n");
                    fprintf(stderr, "in file %s, line %d\n", __FILE__, __LINE__);
                    abort();
	        }
                break;
              }
              default :
                fflush(stderr);
                fprintf(stderr, "\n\n Unknown type of boundary condition is required\n");
                fprintf(stderr, " in function wr_boconelsurf, (file %s, line %d)", __FILE__, __LINE__);
                abort();
	    }
	  }
        }
	else{
	  for (l = 0; l < nnbc; l++)
	    fprintf(out, " 0 0.0");
	}
      }
      fprintf(out, "\n");
    }    
    fprintf(out, "\n");
    for (k=0; k<nbc; k++)
    {
      if ((surfprop[k] >= 0) && wele)
      {
        for (l = 0; l < nnbc; l++)
	{
          switch (Boce[surfprop[k]]->bct[k])
	  {
            case 0:
            case 1:
            case 2:
              fflush(stderr);
	      fprintf(stderr, "\n\n Zadany typ okrajove podminky neumime tisknout v dusledku laxniho pristupu\n");
	      fprintf(stderr, " nejmenovanych vyvojovych pracovniku k souboru loadelt.cpp\n");
              fprintf(stderr, " funkce wr_boconelsurf (file %s, line %d)\n", __FILE__, __LINE__);
              abort();
            case 4:
            case 30:
              fprintf(out, " %le", Boce[surfprop[k]]->trc[k]);
              break;
            case 90:
              fprintf(out, " %le %le", Boce[surfprop[k]]->trc[k], Boce[surfprop[k]]->trr[k]);
              break;
            default :
              fflush(stderr);
              fprintf(stderr, "\n\n Unknown type of boundary condition is required\n");
              fprintf(stderr, " in function wr_boconelsurf, (file %s, line %d)", __FILE__, __LINE__);
              abort();
	  }
	}
      }
      else
      {
        if ((surfprop[k] >= 0) && wels)
        {
          for (l = 0; l < nnbc; l++)
	  {
            switch (Surfbc[surfprop[k]]->bct)
            {
              case 0:
              case 1:
              case 2:
                fflush(stderr);
	        fprintf(stderr, "\n\n Zadany typ okrajove podminky neumime tisknout v dusledku laxniho pristupu\n");
	        fprintf(stderr, " nejmenovanych vyvojovych pracovniku k souboru loadelt.cpp\n");
                fprintf(stderr, " funkce wr_boconelsurf (file %s, line %d)\n", __FILE__, __LINE__);
                abort();
              case 4:
              case 30:
                fprintf(out, " %le", Surfbc[surfprop[k]]->trc);
                break;
              case 90:
                fprintf(out, " %le %le", Surfbc[surfprop[k]]->trc, Surfbc[surfprop[k]]->trr);
                break;
              default :
                fflush(stderr);
                fprintf(stderr, "\n\n Unknown type of boundary condition is required\n");
                fprintf(stderr, " in function wr_boconelsurf, (file %s, line %d)", __FILE__, __LINE__);
                abort();
            }
          }
        }
        else
	{
          for (l = 0; l < nnbc; l++)
            fprintf(out, " 0.0");
	}
      }
      fprintf(out, "\n");
    }
    delete [] surfprop;
  }
  return (0);
}

/**
    Function writes section with initial conditions for each node for dynamic task.

    @param out   - pointer to the opened text file, where the data will be written
    @param nprop - number of entries in the input property file

    @retval 0 : on succes
*/
long wr_initcond(FILE *out, long nprop)
{
  long i, j, k;
  for (i = 0; i < Top->nn; i++)
  {
    for (j = 0; j < nprop; j++)
    {
      if (Top->nodes[i].prop[0] == Nnp[j])
      {
        if (Inic[j])
        {
          // writing node number
          fprintf(out, "%6ld ", i+1);
          for (k = 0; k < Ndofn; k++)
          // loop for writing values of initial conditions in single directions
            fprintf(out, "%e ", Inic[j]->con[k]);
          fprintf(out, "\n");
        }
      }
    }
  }
  fprintf(out, "\n");
  return(0);
}
