#include <string.h>
#include <math.h>
#include <stdlib.h>
#include "climatcond.h"
#include "globalt.h"
#include "aliast.h"
#include "kunzel.h"
#include "grunewaldmat.h"
#include "bazped.h"
#include "iotools.h"

kunmat *kun1;
grunewaldmat *grunw1;
climatcond::climatcond (void)
{
  T0 = 273.15;    //Reference temperature for enthalpy (0?C)
  Cwat = 4180;    //   Specific heat capacity of pure liquid water
  hvap = 2256000; //Specific enthalpy of water vapour at 0?C
  cvap = 1617;    // Specific heat capacity of water vapour

  mattyp =0;
  basicdirect = basicinclin = 0.0;

  KIND1 = KIND2 = KIND3 = KIND4 = KIND5 = KIND6 = WCorPH = RHorVP = 0;

  Smc = cfmax =Mhmc = Mhrh = WallDirect = WinFact = WinFactVD = coefAlf = AbsCoef = Albedo = Latitude = WallInclin = EmisCoeff = ExchCoeff =  ExchCoeffVD = RainCoeff = MinRainTemp = MinRainFlux = u = a = n = alfaK = Ax = Bx = Cx = 0.0;

  SI =  numberSI = d = 0;
  SIdata = NULL;
  data = NULL;
  
  aa=0.0;  b=0.0;
  eps=0.0;
  m=0.0;
  alpha=0.0;
  sig=5.67e-8;
  tn = 0;
  TepValue = NULL;
  RHValue = NULL;
  DifSValue = NULL;
  DirSValue = NULL;  
  WDValue = NULL;
  WSValue = NULL;
  VRValue = NULL;
  LWValue = NULL;
  SLWValue = NULL;
  NorRainValue = NULL;
  GasFluxValue = NULL;
  HeatFluxValue = NULL;
  ShWRadValue = NULL;
  LoWRadValue = NULL;
  //  radiation (JK + Tomaschko)
  RadValue = NULL;
  TempbValue = NULL;
  MatDataC = NULL; 


  files0 = new char[1000];
  files1 = new char[1000];
  files2 = new char[1000];
  files3 = new char[1000];
  files4 = new char[1000];
  files5 = new char[1000];
  files6 = new char[1000];
  files7 = new char[1000];
  files8 = new char[1000];
  files14 = new char[1000];
  files15 = new char[1000];

}

climatcond::~climatcond (void)
{
  long i, j;
  for(i=0; i<tn; i++)
  {
    if (TepValue != NULL)
      delete [] TepValue[i];
    if (RHValue != NULL)
      delete [] RHValue[i];
    if (DirSValue != NULL)  
      delete [] DirSValue[i];  
    if (DifSValue != NULL)  
      delete [] DifSValue[i];  
    if (WDValue != NULL)
      delete [] WDValue[i];
    if (WSValue != NULL)
      delete [] WSValue[i];
    if (VRValue != NULL)
      delete [] VRValue[i];
    if (LWValue != NULL)
      delete [] LWValue[i];
    if (SLWValue != NULL)
      delete [] SLWValue[i];
    if (NorRainValue != NULL)
      delete [] NorRainValue[i];
    if (GasFluxValue != NULL)
      delete [] GasFluxValue[i];
    if (HeatFluxValue != NULL)
      delete [] HeatFluxValue[i];
    if (ShWRadValue != NULL)
      delete [] ShWRadValue[i];
    if (LoWRadValue != NULL)
      delete [] LoWRadValue[i];
    //  radiation (JK + Tomaschko)
    if (RadValue != NULL)
      delete [] RadValue[i];
   //  Tb temperature (JK+Klier)
    if (TempbValue != NULL)
      delete [] TempbValue[i];
  }
  delete [] TepValue;
  delete [] RHValue;
  delete [] DifSValue;
  delete [] DirSValue;
  delete [] WDValue;
  delete [] WSValue;
  delete [] VRValue;
  delete [] LWValue;
  delete [] SLWValue;
  delete [] NorRainValue;
  delete [] GasFluxValue;
  delete [] HeatFluxValue;
  delete [] ShWRadValue;
  delete [] LoWRadValue;
  //  radiation (JK + Tomaschko)
  delete [] RadValue;
  //  Tb temperature (JK+Klier)
  delete [] TempbValue;

if (MatDataC != NULL)
  {
    for(i=0; i<2; i++)
    {
      for(j=0; j<numd; j++)
        delete [] MatDataC[i][j];
      delete [] MatDataC[i];
    }
    delete [] MatDataC;
  }

  if(SIdata != NULL)
    delete [] SIdata ;
  if(data != NULL)
    delete [] data ;



  delete [] files0;
  delete [] files1;
  delete [] files2;
  delete [] files3;
  delete [] files4;
  delete [] files5;
  delete [] files6;
  delete [] files7;
  delete [] files8;
  delete [] files14;
  delete [] files15;

}

//---------------------------------------------------------------------------
/**
   computes heat flux
   
   @param Tkind - type of flux computation
   @param Temperak - temperature of the structure (domain)
   @param t - time
   
   JM
*/
double climatcond::ConditHeatFlux(long Tkind, double Temperak, double t)
{
  double Tvalue, WinDir, WSvalue, Radvalue, Tempbvalue;
  double WallDir, betaWind,coefAlf0,tempr;
  FindValueinTimeT(t,TepValue,Tvalue);
  
  WallDir = basicdirect;
  switch (Tkind)
    {
    case 0:{
      //  classical heat transmission
      Jkq = -1.0 * coefAlf *((Tvalue+273.15) - Temperak);
      break;
    }
    case 1:{
      //  flux defined from climatic file
      //  not available at this time
      break;
    }
    case 2:{
		//  heat flux modified by wind speed and wind orientation
      
		FindValueinTimeT(t,WDValue,WinDir);
		FindValueinTimeT(t,WSValue,WSvalue);
		WinDir = WinDir*M_PI/180;
		//   WallDir = WallDirection(hranice);
		
		if ((fabs(WallDir -WinDir))<=M_PI)
		{
			betaWind = fabs(WallDir -WinDir);
		}
		else
		{
			betaWind = 2*M_PI - (fabs(WallDir -WinDir));
		}
		
		if (betaWind < (M_PI/2))
		{
			coefAlf0 = coefAlf + WinFact*sqrt(WSvalue);
		}
		else
		{
			coefAlf0 = coefAlf;
		}
		// printf ("\n\n Alf je %lf",coefAlf0);
		Jkq = -1.0 *(coefAlf0 *((Tvalue+273.15) - Temperak));
		break;
		   }
    case 3:{
      FindValueinTimeT(t,WSValue,WSvalue);
      FindValueinTimeT(t,RadValue,Radvalue);
      coefAlf0 = b + aa * WSvalue;
      tempr=Tvalue+273.15;
      Jkq = -1.0 *(coefAlf0 *(tempr - Temperak));
      Jkq += -1.0*eps*sig*(tempr*tempr*tempr*tempr - Temperak*Temperak*Temperak*Temperak);
      Jkq += -1.0*alpha*m*Radvalue;
      break;
    }
    case 4:{
      //FindValueinTimeT(t,WSValue,WSvalue);
      FindValueinTimeT(t,RadValue,Radvalue);
      FindValueinTimeT(t,TempbValue,Tempbvalue);

      tempr=Tvalue+273.15;

      Jkq = b *(Temperak - tempr);

      Jkq += aa * (Temperak - Tempbvalue);

      Jkq += eps*sig*(Temperak*Temperak*Temperak*Temperak - tempr*tempr*tempr*tempr);

      Jkq += Radvalue;

      break;
    }
      
    default:{
      fprintf (stderr,"\n\n unknown definition of ConditHeatFlux is required");
      fprintf (stderr,"\n in function read (file %s, line %d)\n",__FILE__,__LINE__);
    }
    }
  
  return (Jkq);
  
}

//---------------------------------------------------------------------------


void climatcond::ReadCllimaticFiles(char * Cfile, long cod)
{
  FILE *ffile;
  char *zahlavi;
  char *cdata;
  long years, days, hours, minits, sec;
  double Tep;
  long i = 0;
  
  if (strcmp(Cfile,"0") == 0)
    return;
     
  zahlavi = new char[1000];
    
  
  ffile = fopen(Cfile,"r");
  if (ffile==NULL){
    fprintf (stderr,"\n Input file for climatic condition is not found (file %s, line %d",__FILE__,__LINE__);
    fprintf (stderr,"\n Try it again!\n\n");
  }
  
  if((ffile = fopen(Cfile,"r")) == NULL) abort();
  
  do {
    fgets(zahlavi, 9990, ffile);
    cdata = "Data=";
  } while((strstr(zahlavi,cdata)) ==NULL);
  
  fscanf (ffile,"%ld %ld",&tn,&cc);	
  
  switch (cod){
  case 0: {
    TepValue = new double* [tn];
    for (i=0;i<tn;i++){
      TepValue[i] = new double [2];
    }
    break;
  }
  case 1: {
    RHValue = new double* [tn];
    for (i=0;i<tn;i++){
      RHValue[i] = new double [2];
    }
    break;
  }
  case 2: {
    DifSValue = new double* [tn];
    for (i=0;i<tn;i++){
      DifSValue[i] = new double [2];
    }
    break;
  }
  case 3: {
    DirSValue = new double* [tn];
    for (i=0;i<tn;i++){
      DirSValue[i] = new double [2];
    }
    break;
  }
  case 4: {
    WDValue = new double* [tn];
    for (i=0;i<tn;i++){
      WDValue[i] = new double [2];
    }
    break;
  }
  case 5: {
    WSValue = new double* [tn];
    for (i=0;i<tn;i++){
      WSValue[i] = new double [2];
    }
    break;
  }
  case 6: {
    VRValue = new double* [tn];
    for (i=0;i<tn;i++){
      VRValue[i] = new double [2];
    }
    break;
  }
  case 7: {
    LWValue = new double* [tn];
    for (i=0;i<tn;i++){
      LWValue[i] = new double [2];
    }
    break;
  }
  case 8: {
    SLWValue = new double* [tn];
    for (i=0;i<tn;i++){
      SLWValue[i] = new double [2];
    }
    break;
  }
  case 9: {
    NorRainValue = new double* [tn];
    for (i=0;i<tn;i++){
      NorRainValue[i] = new double [2];
    }
    break;
  }
  case 10: {
    GasFluxValue = new double* [tn];
    for (i=0;i<tn;i++){
      GasFluxValue[i] = new double [2];
    }
    break;
  }
  case 11: {
    HeatFluxValue = new double* [tn];
    for (i=0;i<tn;i++){
      HeatFluxValue[i] = new double [2];
    }
    break;
  }
  case 12: {
    ShWRadValue = new double* [tn];
    for (i=0;i<tn;i++){
      ShWRadValue[i] = new double [2];
    }
    break;
  }
  case 13: {
    LoWRadValue = new double* [tn];
    for (i=0;i<tn;i++){
      LoWRadValue[i] = new double [2];
    }
    break;
  }
  case 14: {
    //  radiation developed for JK + Tomaschko
    RadValue = new double* [tn];
    for (i=0;i<tn;i++){
      RadValue[i] = new double [2];
    }
    break;
  }
  case 15: {
    //  radiation developed for JK + Klier
    TempbValue = new double* [tn];
    for (i=0;i<tn;i++){
      TempbValue[i] = new double [2];
    }
    break;
  }

  default:{
    fprintf (stderr,"\n\n unknown definition of ReadCllimaticFiles is required");
    fprintf (stderr,"\n in function read (file %s, line %d)\n",__FILE__,__LINE__);
  }
  }
  
  
  i = 0;
  while ((fscanf (ffile,"%ld %ld %ld %ld %ld %lf", &years, &days, &hours, &minits, &sec,&Tep)) != EOF)
    {
      switch (cod)
	{
	case 0: TepValue[i][0] = (years * 31536000)+(days * 86400) + (hours * 3600) + (minits * 60) +sec;
	  TepValue[i][1] =Tep;
	  break;
	case 1: RHValue[i][0] = (years * 31536000)+(days * 86400) + (hours * 3600) + (minits * 60) +sec;
	  RHValue[i][1] =Tep;
	  break;
	case 2: DifSValue[i][0] = (years * 31536000)+(days * 86400) + (hours * 3600) + (minits * 60) +sec;
	  DifSValue[i][1] =Tep;
	  break;
	case 3: DirSValue[i][0] = (years * 31536000)+(days * 86400) + (hours * 3600) + (minits * 60) +sec;
	  DirSValue[i][1] =Tep;
	  break;
	case 4: WDValue[i][0] = (years * 31536000)+(days * 86400) + (hours * 3600) + (minits * 60) +sec;
	  WDValue[i][1] =Tep;
	  break;
	case 5: WSValue[i][0] = (years * 31536000)+(days * 86400) + (hours * 3600) + (minits * 60) +sec;
	  WSValue[i][1] =Tep;
	  break;
	case 6: VRValue[i][0] = (years * 31536000)+(days * 86400) + (hours * 3600) + (minits * 60) +sec;
	  VRValue[i][1] =Tep;
	  break;
	case 7: LWValue[i][0] = (years * 31536000)+(days * 86400) + (hours * 3600) + (minits * 60) +sec;
	  LWValue[i][1] =Tep;
	  break;
	case 8: SLWValue[i][0] = (years * 31536000)+(days * 86400) + (hours * 3600) + (minits * 60) +sec;
	  SLWValue[i][1] =Tep;
	  break;
	case 9: NorRainValue[i][0] = (years * 31536000)+(days * 86400) + (hours * 3600) + (minits * 60) +sec;
	  NorRainValue[i][1] =Tep;
	  break;
	case 10:GasFluxValue[i][0] = (years * 31536000)+(days * 86400) + (hours * 3600) + (minits * 60) +sec;
	  GasFluxValue[i][1] =Tep;
	  break;
	case 11: HeatFluxValue[i][0] = (years * 31536000)+(days * 86400) + (hours * 3600) + (minits * 60) +sec;
	  HeatFluxValue[i][1] =Tep;
	  break;
	case 12:ShWRadValue[i][0] = (years * 31536000)+(days * 86400) + (hours * 3600) + (minits * 60) +sec;
	  ShWRadValue[i][1] =Tep;
	  break;
	case 13:LoWRadValue[i][0] = (years * 31536000)+(days * 86400) + (hours * 3600) + (minits * 60) +sec;
	  LoWRadValue[i][1] =Tep;
	  break;
	case 14:RadValue[i][0] = (years * 31536000)+(days * 86400) + (hours * 3600) + (minits * 60) +sec;
	  RadValue[i][1] = Tep;
	  break;
	case 15:TempbValue[i][0] = (years * 31536000)+(days * 86400) + (hours * 3600) + (minits * 60) +sec;
	  TempbValue[i][1] = Tep;
	  break;
	default:{
	  fprintf (stderr,"\n\n unknown definition of ReadCllimaticFiles is required");
	  fprintf (stderr,"\n in function read (file %s, line %d)\n",__FILE__,__LINE__);
	}
	}
      
      i = i+1;
    }
  fclose (ffile);
  delete [] zahlavi;  
}

//---------------------------------------------------------------------------


void climatcond::FindValueinTimeT(double time, double **Matrix, double & value)
  
{
 // tady pro cas T nachazim promenou X

 long i = 0;

 
 switch (cc){
 case 0:{
   while (time >= 31536000)
     {
       time = time - (31536000);
     }
   
   while (Matrix[i][0] < time)
     {
       i= i+1;
     }
   
   
   if (time <= 3600)
     {
       value = Matrix[8759][1]+((Matrix[i][1] - Matrix[8759][1])/3600*(time));
     }
   else{
     value = Matrix[i-1][1] +((Matrix[i][1] - Matrix[i-1][1])/3600*(time - Matrix[i-1][0]));
     //	value = Matrix[i-1][1] +((Matrix[i][1] - Matrix[i-1][1])/(Matrix[i][0] - Matrix[i-1][0])*(time - Matrix[i-1][0]));
   }
   break;
 }
 case 1:{
   while (Matrix[i][0] < time)
     {
       i= i+1;
     }
   
   value = Matrix[i-1][1] +((Matrix[i][1] - Matrix[i-1][1])/(Matrix[i][0] - Matrix[i-1][0])*(time - Matrix[i-1][0]));
   break;
 }
 default:{
   fprintf (stderr,"\n\n unknown definition of FindValueinTimeT is required");
   fprintf (stderr,"\n in function read (file %s, line %d)\n",__FILE__,__LINE__);
 }
 }
 
}
//---------------------------------------------------------------------------

void climatcond::FindValuesinT(long TRH,double t, double &value)
{
 /*
  TRH =0 .... Temperature in time (t)
      =1 .... Relative humidity ni time (t)


 */

  switch (TRH)
    {
    case 0:FindValueinTimeT(t,TepValue,value);
    value = value +273.15;
    break;
    case 1: FindValueinTimeT(t,RHValue,value);
    break;
    }
}
//---------------------------------------------------------------------------


double climatcond::ConditShortWaveRadiatFlux(long SWkind, double Temperak, double t)
{
   double WallDir;
   double MAXSUNDECL = 0.4089306437;
   double HOURFREQU = 0.26179939;
   double YEARFREQU =0.017214206;
   double SPRINGBEGIN =80;
   double th, dt,day, sdt,cdt, hold, hnew, sold, snew, alf, r;
   double JkQdif, JkQdir, qkdir, qkdif;

    if (SWkind == 0)
    {
    return(AbsCoef *0);
    }

  
   WallDir = basicdirect;
   WallInclin =  basicinclin;
   FindValueinTimeT(t,DirSValue,qkdir);
   FindValueinTimeT(t,DifSValue,qkdif);

   th = t/3600;
   day = t/86400;
   dt = MAXSUNDECL *sin((YEARFREQU *(day-SPRINGBEGIN )));

   sdt = sin(Latitude )*sin(dt);
   cdt = cos(Latitude )*cos(dt);

   hold = asin(sdt - cdt*cos(HOURFREQU *(th - 2)));
   hnew = asin(sdt - cdt*cos(HOURFREQU *(th - 1)));

   sold = cos(dt)*(sin(HOURFREQU *(th-2)))/(cos(hold));
   snew = cos(dt)*(sin(HOURFREQU *(th-1)))/(cos(hnew));


   if((th>12) && (snew > sold))
    {
    alf = 2*M_PI + asin(snew);
    }
   if ((th <= 12) || (snew <= sold))
    {
    alf = M_PI - asin(snew);
    }

   r = cos(WallInclin ) + sin(WallInclin )*(cos(alf -WallDir ))/(tan(hnew));

   if (r<0)
    {
    JkQdir = 0;
    }
   else
    {
    if (r>0.5)
        {
        JkQdir = 0.5 *qkdir;
        }
    else
        {
        JkQdir = r * qkdir ;
        }
    }

   JkQdif  = qkdif *cos(WallInclin /2)*cos(WallInclin /2) + Albedo *(qkdif +qkdir )*sin(WallInclin /2)*sin(WallInclin /2);

   return (AbsCoef *(JkQdif +JkQdir ));

}

//---------------------------------------------------------------------------


double climatcond::ConditLongWaveRadiatFlux(long LWKind, double Temperak, double t)
{
  double qkemis, qksky;
  double flux = 0.0;
  switch (LWKind)
    {
    case 0:{ //ShowMessage("Jeste ve vystavbe");
      
      flux = 0.0;
      break;
    }
    case 1:{ 
      FindValueinTimeT(t,LWValue,qkemis);
      FindValueinTimeT(t,SLWValue,qksky);
      flux = (EmisCoeff *(qkemis +qksky ));
      break;
    }
    default:{
      fprintf (stderr,"\n\n unknown definition of ConditLongWaveRadiatFlux is required");
      fprintf (stderr,"\n in function read (file %s, line %d)\n",__FILE__,__LINE__);
    }
    }
 return (flux);
}

//---------------------------------------------------------------------------

double climatcond::Condit_Water_contact(long WCKind, double t)
{
  
  return(0.0);

}

//---------------------------------------------------------------------------


double climatcond::Condit_Rain_moistureFlux(long RainKind, double RHakt, double t,long ipp, long nid)
{
  long i;
  double NorRain, WallDir, VerRain, WinDir, WindVel, Temper, RelHum,MoistureAkt;
  double kwind, betawind, krainEff, jkNorRain, jkmaxWat;
  
  FindValueinTimeT(t,TepValue,Temper);
  FindValueinTimeT(t,RHValue,RelHum);
  FindValueinTimeT(t,VRValue,VerRain);
  FindValueinTimeT(t,WDValue,WinDir);
  FindValueinTimeT(t,WSValue,WindVel);
  
  RelHum = RelHum/100;
  VerRain = VerRain/3600;
  
  if (WCorPH == 1)abort();
  
  
  // jen pro testovani/-----------------------------
  
  if (RHakt <0) RHakt = 0;
  if (RHakt > 1) RHakt = 1;
  
  switch (mattyp)
    {
    case 150:{//pro bazped vstupuje u(kg/kg)=RHAkt, t(K)=TemperAkt
      MoistureAkt = RHakt; //moisture content u(kg/kg)
      i = Tm->ip[ipp].idm;//material type
      RHakt = Tm->bazped[i].inverse_sorption_isotherm(MoistureAkt);
    }
      break;
    case 155: //switch (MatCharC[0])
		MoistureAkt = Tt->nodes[nid].eqother[0];
		Smc = Tt->nodes[nid].eqother[2];
	
	break;
	case 158: 
			double MoistureAktp;
			MoistureAkt = Tt->nodes[nid].eqother[0];
			Smc = Tt->nodes[nid].eqother[3];
		break;
		
   	case 203:
	case 202:
	case 157:MoistureAkt = RHakt;
		break;
    case 159:{
		MoistureAkt = RHakt;
		RHakt = Tt->nodes[nid].eqother[1];
		Smc = Tt->nodes[nid].eqother[3];
			 }
		break;
	case 156: {
		MoistureAkt = RHakt;
		Smc = Tt->nodes[nid].eqother[2];
			  }
		break;
    default:{
      fprintf (stderr,"\n\n unknown definition of Condit_Rain_moistureFlux is required");
      fprintf (stderr,"\n in function read (file %s, line %d)\n",__FILE__,__LINE__);
    }
      
    }
  
  
  // --------------------------------------
  
  
  WallDir = basicdirect;
  WinDir = WinDir*M_PI/180;
  
  
  switch (RainKind){
  case 0: kwind = 1;
    NorRain = VerRain ;
    break;
  case 1:  if( (fabs(WallDir - WinDir )) <= M_PI)
      {
	betawind = fabs(WallDir - WinDir );
      }
    else
      {
	betawind = 2*M_PI - fabs(WallDir - WinDir );
      }
    if ((betawind <(M_PI/2)) && (WindVel >0) && (VerRain >0))
      {
	
		double c1, c2, c3, c4, c5;
		c1 = cos (betawind);
		c2 = 1141*sqrt(3600*VerRain/(WindVel *WindVel*WindVel*WindVel));
		c3 = sqrt (1 + c2);
		c5 = sqrt(3600*VerRain);
		c5 = sqrt(c5);
		c4 = exp(-12/(5*c5));
		kwind = c1/c3 * c4;
	
	
	
      }
    else
      {
	kwind = 0;
      }
    NorRain = VerRain ;
    
    break;
  default:{
    fprintf (stderr,"\n\n unknown definition of Condit_Rain_moistureFlux is required");
    fprintf (stderr,"\n in function read (file %s, line %d)\n",__FILE__,__LINE__);
  }
  }
  
  if(NorRain <= MinRainFlux )
    {
      krainEff = 1;
    }
  else
    {
      krainEff = RainCoeff ;
    }
  if ((Temper+273.15) <=MinRainTemp )
    {
      jkNorRain = 0;
    }
  else
    {
      jkNorRain = kwind *krainEff *NorRain ;
    }
  jkmaxWat = ExchCoeff *(Smc -MoistureAkt );
  //  printf ("\n\n rozdil je %lf",(Smc -MoistureAkt ));
  if (jkNorRain <jkmaxWat )
    {
      return (jkNorRain );
    }
  else
    {
      return (jkmaxWat );
    }
 
}

//---------------------------------------------------------------------------


double climatcond::Condit_Rain_heatFlux(long RainKind, double RHakt, double t,long ipp, long nid)
{

   double Temper, RelHum, RainMoistureFlux, TemperRain;
   double uwT;  // Specific internal energy of the liquid water uw(T)

   if (RHakt <0) RHakt = 0;
   if (RHakt > 1) RHakt = 1;

   RainMoistureFlux =-1*Condit_Rain_moistureFlux(RainKind, RHakt, t,ipp, nid);
    FindValueinTimeT(t,TepValue,Temper);
    FindValueinTimeT(t,RHValue,RelHum);
    RelHum = RelHum/100;

     TemperRain = pow(RelHum,0.1247)*(109.8+(Temper+273.15))- 109.8;

    uwT = Cwat *(TemperRain -T0);

   return (uwT*RainMoistureFlux );
  
}

//---------------------------------------------------------------------------


double climatcond::Condit_Vapour_Diffusion_moisture_flux(long VDKind,double t, double RHAkt, double TemperAkt,long ipp, long nid)
{
  long i;
  double Temper, RelHum, VapPres, WindFactVD,VapPresF, WallDir, WindDir, WindVel,MoistureAkt;
  double betaWind, psat, betaExch, pv, smc;
  
  if (RHAkt < 0) RHAkt = 0;
  if (RHAkt > 1) RHAkt = 1;
  
  FindValueinTimeT(t,TepValue,Temper);
  FindValueinTimeT(t,RHValue,RelHum);
  RelHum = RelHum/100;
  
  WallDir = basicdirect;
  
  //pro kunzel je w(m3/m3); pro bazped je u(kg/kg)
  smc = MatConstC[1];
  
  switch (mattyp)
    {
    case 150:{//pro bazped vstupuje u(kg/kg)=RHAkt, t(K)=TemperAkt
		MoistureAkt = RHAkt; //moisture content u(kg/kg)
		i = Tm->ip[ipp].idm;//material type
		RHAkt = Tm->bazped[i].inverse_sorption_isotherm(MoistureAkt);
			 }
		break;
    case 155:{
		MoistureAkt = Tt->nodes[nid].eqother[0];
		smc = Tt->nodes[nid].eqother[2];
			 }
		break;
	case 158:{
		smc = Tt->nodes[nid].eqother[3];
		MoistureAkt = Tt->nodes[nid].eqother[0];
			 }
		break;
	case 159:{
		MoistureAkt = RHAkt;
		RHAkt = Tt->nodes[nid].eqother[1];
		smc = Tt->nodes[nid].eqother[3];
			 }
		break;
	case 203:
		{
		MoistureAkt = RHAkt;
		RHAkt = Tt->nodes[nid].eqother[8];
		}
		break;
    case 202:
    case 201:
    case 200: TemperAkt = 273.15; // temperature specimentu
    case 157:
    case 156:
			{
				MoistureAkt = RHAkt;
				RHAkt = Tt->nodes[nid].eqother[0];
				smc = Tt->nodes[nid].eqother[2];

			}
		break;
    default:{
      fprintf (stderr,"\n\n unknown definition of Condit_Vapour_Diffusion_moisture_flux is required");
      fprintf (stderr,"\n in function read (file %s, line %d)\n",__FILE__,__LINE__);
    }
    }
  
  // ----------------------------------------------------------------------------------
  // nutne doplnit zavislost pro model bazped, kde je neznama moisture content u(kg/kg)
  switch (VDKind )
    {
    case 0: betaExch = ExchCoeffVD ;
      break;
    case 1:  FindValueinTimeT(t,WDValue,WindDir);
      FindValueinTimeT(t,WSValue,WindVel);
      WindDir =(M_PI/180)*WindDir;
      if (fabs (WallDir -WindDir) <= M_PI)
	{
	  betaWind = fabs (WallDir -WindDir);
	}
      else
	{
	  betaWind = 2*M_PI - fabs(WallDir -WindDir);
	}
      
      if (betaWind < (M_PI/2))
	{
	  betaExch = ExchCoeffVD +WindFactVD * sqrt(WindVel );
	}
      else
	{
	  betaExch = ExchCoeffVD ;
	}
      break;
    case 2: betaExch = ExchCoeffVD*MoistureAkt + alfaK;
      break;
    case 3: betaExch = ExchCoeffVD*exp(alfaK*MoistureAkt);
      break;
    case 4: betaExch = Ax*MoistureAkt*MoistureAkt + Bx*MoistureAkt + Cx;
      break;
    case 5: find_data(MoistureAkt,betaExch);
      break;
    default:{
      fprintf (stderr,"\n\n unknown definition of Condit_Vapour_Diffusion_moisture_flux is required");
      fprintf (stderr,"\n in function read (file %s, line %d)\n",__FILE__,__LINE__);
    }
    }
  
  
  // musim doplnit i moznost TRY tlaku misto RH
  
  //external vapor pressure
  psat = exp(23.5771 - 4042.9/((Temper+273.15)  - 37.58));
  if (RHorVP == 0)
    {
      VapPres = psat * RelHum ;
    }
  else
    {
      VapPres = VapPresF;
    }
  
  //interni vapor pressure
  psat = exp(23.5771 - 4042.9/(TemperAkt   - 37.58));
  pv = psat * RHAkt ;
  double tokp;
  
  if ((VapPres  - pv) <0)
    {
      // return (betaExch *(VapPres -pv ));

      tokp =  (betaExch *(pv - VapPres));
    }
  else
    {
      // return(((Smc -MoistureAkt)/Smc)*betaExch *(VapPres -pv ));
      tokp = (((smc-MoistureAkt)/smc)*betaExch *(pv -VapPres));
      
    }
 // fprintf(Outt2, "%ld  %lf %lf  %lf %lf %lf\n",nid,  t, smc, RHAkt,MoistureAkt, RelHum);// kapa a decko
  return(tokp);
}

//---------------------------------------------------------------------------

double climatcond::Condit_Vapour_Diffusion_Heat_Flux(long VDKind,double t, double RHAkt, double TemperAkt,long ipp, long nid)
{
  double JkmvDiff, Tvap, Temper, hv;
  
  if (RHAkt < 0) RHAkt = 0;
  if (RHAkt > 1) RHAkt = 1;
  
  
  JkmvDiff = Condit_Vapour_Diffusion_moisture_flux(VDKind, t,RHAkt,TemperAkt,ipp, nid);
  FindValueinTimeT(t,TepValue,Temper);
  
  Temper = Temper + 273.15;
  
  Tvap = (Temper + TemperAkt )/2;
  
  hv = cvap *(Tvap -T0)+hvap ;
  
  return (hv * JkmvDiff );
  
}

//---------------------------------------------------------------------------


double climatcond::Get_Heat_Flux(double nvalvrh,double nvalvT,long ipp, long nid)
{
  double JH1, JH2, JH3, JH7, JH8;
  double time;

  mednamest mn;

    mn = Tp->mednam;

  //T = Tm->ip[ipp].av[1];
  time = Tp->time;
	
  if (nvalvrh < 0.0) nvalvrh = 0.0;
  if (nvalvrh > 1.0) nvalvrh = 1.0;
  
  switch (KIND1)
    {
    case 0: JH1 = 0;
      break;
    case 11: JH1 = ConditHeatFlux(1,nvalvT,time);
      break;
    case 12: JH1 = ConditHeatFlux(0,nvalvT,time);
      break;
    case 13: JH1 = ConditHeatFlux(2,nvalvT,time);
      break;
    case 14: JH1 = ConditHeatFlux(3,nvalvT,time);
      break;
    case 15: JH1 = ConditHeatFlux(4,nvalvT,time);
      break;
    default:{
      fprintf (stderr,"\n\n unknown definition of Get_Heat_Flux-KIND1 is required");
      fprintf (stderr,"\n in function read (file %s, line %d)\n",__FILE__,__LINE__);
    }
    }
  
  switch (KIND2)
    {
    case 0: JH2 = 0;
      break;
    case 21: JH2 =-1* ConditShortWaveRadiatFlux(0,nvalvT,time);
      break;
    case 22: JH2 =-1* ConditShortWaveRadiatFlux(1,nvalvT,time);
      break;
    default:{
      fprintf (stderr,"\n\n unknown definition of Get_Heat_Flux-KIND2 is required");
      fprintf (stderr,"\n in function read (file %s, line %d)\n",__FILE__,__LINE__);
    }
    }
  
  switch(KIND3)
    {
    case 0: JH3 = 0;
      break;
    case 31: JH3 =-1* ConditLongWaveRadiatFlux(0,nvalvT,time);
      break;
    case 32: JH3 =-1* ConditLongWaveRadiatFlux(1,nvalvT,time);
      break;
    default:{
      fprintf (stderr,"\n\n unknown definition of Get_Heat_Flux-KIND3 is required");
      fprintf (stderr,"\n in function read (file %s, line %d)\n",__FILE__,__LINE__);
    }
    }
  
   switch(mn){
   case 30: 
   case 10:{
     switch (KIND5){
     case 0: JH7 = 0;
       break;
     case 51: JH7 = Condit_Rain_heatFlux(0,nvalvrh,time,ipp, nid) ;
       break;
     case 52: JH7 = Condit_Rain_heatFlux(1,nvalvrh,time,ipp, nid) ;
       break;
     default:{
       fprintf (stderr,"\n\n unknown definition of Get_Heat_Flux-KIND5 is required");
       fprintf (stderr,"\n in function read (file %s, line %d)\n",__FILE__,__LINE__);
     }
     }
     break;
   }
   default: JH7 = 0;
     break;
   }
   
   switch(mn){
   case 30:
   case 10:{
     switch (KIND6){
     case 0: JH8 =0;
       break;
     case 61: JH8 = Condit_Vapour_Diffusion_Heat_Flux(0,time,nvalvrh, nvalvT,ipp, nid) ;
       break;
     case 62: JH8 = Condit_Vapour_Diffusion_Heat_Flux(1,time,nvalvrh, nvalvT,ipp, nid) ;
       break;
     case 63: JH8 = Condit_Vapour_Diffusion_Heat_Flux(2,time,nvalvrh, nvalvT,ipp, nid) ;
       break;
     case 64: JH8 = Condit_Vapour_Diffusion_Heat_Flux(3,time,nvalvrh, nvalvT,ipp, nid) ;
       break;
     case 65: JH8 = Condit_Vapour_Diffusion_Heat_Flux(4,time,nvalvrh, nvalvT,ipp, nid) ;
       break; 
     case 66: JH8 = Condit_Vapour_Diffusion_Heat_Flux(4,time,nvalvrh, nvalvT,ipp, nid) ;
       break;
     default:{
       fprintf (stderr,"\n\n unknown definition of Get_Heat_Flux-KIND6 is required");
       fprintf (stderr,"\n in function read (file %s, line %d)\n",__FILE__,__LINE__);
     }
     }
     break;
   }
   default: JH8 = 0;
     break;
   }
   
   
   return(JH1 +JH2 +JH3 + JH7 + JH8);
}

//---------------------------------------------------------------------------


double climatcond::WallDirection(long hranice)
{
  switch (hranice)
    {
    case 1: return (WallDirect+ M_PI);
      break;
    case 2: return (WallDirect+ M_PI+ M_PI/2 );
      break;
    case 3: return (WallDirect);
      break;
    case 4: return (WallDirect + M_PI);
      break;
    }
   return (basicdirect);
  
}
//---------------------------------------------------------------------------

void climatcond::read(XFILE *in)
{
  int h,k,i,j;
  long klier=0;
  xfscanf (in,"%d",(int*)&mattyp);
  xfscanf (in,"%lf",&basicdirect);
  xfscanf (in,"%lf",&basicinclin);
  
  //-------- volba typu sorpcni izotermy....
  if ( mattyp != 155 && mattyp != 158&& mattyp != 156 && mattyp != 159)
  {
	  switch (mattyp)
		{
	  case 158:{
			   }
		  break;
	  case 150:{
		  xfscanf (in,"%d %d",&MatCharC[0],&MatCharC[1]);
		  h = 2;
		  break;
			   }
	  case 203:
	  case 202:{
		  xfscanf (in,"%d %d %d",&MatCharC[0],&MatCharC[1],&MatCharC[2]);
		  h = 3;
		  break;
			   }
	  default:{
		  xfscanf (in,"%d %d",&MatCharC[0],&MatCharC[1]);
		  h = 2;
			  }
	  }
	  for (i = 0; i <h; i++)
	  {
		  switch (MatCharC[i]){
		  case 1: {
			  xfscanf (in,"%lf",&MatConstC[i] );
			  break;
				  }
		  case 30:{
			  xfscanf (in,"%lf %lf",&MatFunceC[i][0],&MatFunceC[i][1] );
			  break;
				  }
		  case 31:{
			  xfscanf (in,"%lf %lf %lf",&MatFunceC[i][0],&MatFunceC[i][1],&MatFunceC[i][2] ); // to je pro Hansena
			  break;
				  }
		  case 0:
		  case 2:{
			  xfscanf (in,"%d",&numd);
			  double x,y;
			  MatDataC = new double**[2];
			  for (j = 0; j<2; j++)
			  {
				  MatDataC[j] = new double*[numd];
				  for(k = 0; k<numd; k++)
				  {
					  MatDataC[j][k] = new double[2];
				  }
			  }
			  for (j=0; j<numd;j++)
			  {
				  xfscanf (in,"%lf %lf",&x, &y);
				  MatDataC[i][j][0] = x;
				  MatDataC[i][j][1] = y;
			  }
			  break;
				 }
		  default:{
			  fprintf (stderr,"\n\n unknown definition of Read climatic. mat. data is required");
			  fprintf (stderr,"\n in function read (file %s, line %d)\n",__FILE__,__LINE__);
				  }
		  }
	  }
  }
  //pro kunzel je w(m3/m3); pro bazped je u(kg/kg)
  //Smc = MatConstC[1];//tady doplneno!!!
  
  xfscanf (in,"%ld",&KIND1 );
  
  switch (KIND1){
  case 0:{     //KIND1 = 0;
    break;
  }
  case 11:{    xfscanf (in,"%lf",&coefAlf );
      //KIND1 = 11;
      break;
  }
  case 12:{    xfscanf (in,"%lf",&coefAlf );
      //KIND1 = 12;
      break;
  }
  case 13:{    xfscanf (in,"%lf %lf",&coefAlf, &WinFact);
      break;
  }
  case 14:{
    //  JK+Tomaschko
    //  aa - coefficient at velocity
    //  b - absolute coefficient
    //  eps - coefficient of emissivity
    //  alpha - absorption coefficient
    //  m - parameter of cloudiness
    xfscanf (in,"%lf %lf   %lf %lf %lf",&aa,&b,&eps,&alpha,&m);
    break;
  }
case 15:{
    //  JK+Klier
    //  b - transmission coefficient for wind influence
    //  aa -
    //  eps - radiation coefficient
    xfscanf (in,"%lf %lf %lf",&b,&aa,&eps);
    
    klier=1;
    break;
  }

  default:{
    fprintf (stderr,"\n\n unknown definition of Read clim.data-KIND1 is required");
    fprintf (stderr,"\n in function read (file %s, line %d)\n",__FILE__,__LINE__);
  }
  }
  
  xfscanf (in,"%ld",&KIND2 );
  
  switch (KIND2)
    {
    case 0:     //KIND1 = 0;
      break;
    case 21:    xfscanf (in,"%lf",&AbsCoef  );
      //KIND1 = 11;
      break;
    case 22:    xfscanf (in,"%lf %lf %lf",&AbsCoef , &Albedo , &Latitude ); // Latitude se zadava v RADianech
      break;
    default:{
      fprintf (stderr,"\n\n unknown definition of Read clim.data-KIND2 is required");
      fprintf (stderr,"\n in function read (file %s, line %d)\n",__FILE__,__LINE__);
    }
    }
  
  xfscanf (in,"%ld",&KIND3 );
  
  switch (KIND3)
    {
    case 0:     //KIND1 = 0;
      break;
    case 31:    xfscanf (in,"%lf",&EmisCoeff );
      
      break;
    case 32:    xfscanf (in,"%lf",&EmisCoeff );
      break;
    default:{
      fprintf (stderr,"\n\n unknown definition of Read clim.data-KIND3 is required");
      fprintf (stderr,"\n in function read (file %s, line %d)\n",__FILE__,__LINE__);
    }
    }
  
  xfscanf (in,"%ld",&KIND4 );
  
  switch (KIND4)
    {
    case 0:     //KIND1 = 0;
      break;
    case 41:    xfscanf (in,"%ld",&WCorPH );
      
      break;
    default:{
      fprintf (stderr,"\n\n unknown definition of Read clim.data-KIND4 is required");
      fprintf (stderr,"\n in function read (file %s, line %d)\n",__FILE__,__LINE__);
    }  
    }
  
  xfscanf (in,"%ld",&KIND5 );
  
  switch (KIND5)
    {
    case 0:     //KIND1 = 0;
      break;
    case 51:    xfscanf (in,"%lf %lf %lf %lf",&ExchCoeff , &RainCoeff , &MinRainTemp, &MinRainFlux );
      //KIND1 = 11;
      break;
    case 52:    xfscanf (in,"%lf %lf %lf %lf",&ExchCoeff , &RainCoeff , &MinRainTemp, &MinRainFlux );
      
      
      break;
    default:{
      fprintf (stderr,"\n\n unknown definition of Read clim.data-KIND5 is required");
      fprintf (stderr,"\n in function read (file %s, line %d)\n",__FILE__,__LINE__);
    }
    }
  
  xfscanf (in,"%ld",&KIND6 );
  
  switch (KIND6)
    {
    case 0:     //KIND1 = 0;
      break;
    case 61:    xfscanf (in,"%lf %ld",&ExchCoeffVD, &RHorVP );
      //KIND1 = 11;
      break;
    case 62:    xfscanf (in,"%lf %ld %lf", &ExchCoeffVD, &RHorVP, &WinFactVD);
      
      break;
    case 63:    xfscanf (in,"%lf %ld %lf", &ExchCoeffVD, &RHorVP, &alfaK);
      
      break;
    case 64:    xfscanf (in,"%lf %ld %lf", &ExchCoeffVD, &RHorVP, &alfaK);
      
      break;
    case 65:    xfscanf (in,"%lf %ld %lf %lf", &Ax, &RHorVP, &Bx, &Cx);
      
      break;
    case 66:    xfscanf (in,"%ld %d", &RHorVP, &d);
      double x, y;
      data = new double* [2];
      for (i=0;i<2;i++){
	data[i] = new double [d];
      }
      for (i = 0; i<d;i++)
	{
	  xfscanf (in,"%lf %lf", &x, &y);
	  data[0][i] = x;
	  data [1][i] = y;
	}
      break;
    default:{
      fprintf (stderr,"\n\n unknown definition of Read clim.data-KIND6 is required");
      fprintf (stderr,"\n in function read (file %s, line %d)\n",__FILE__,__LINE__);
    }
    }
  
  xfscanf(in, "%a", files0);
  ReadCllimaticFiles(files0,0);
  
  xfscanf(in, "%a", files1);
  ReadCllimaticFiles(files1,1);
  
  xfscanf(in, "%a", files2);
  ReadCllimaticFiles(files2,2);
  
  xfscanf(in, "%a", files3);
  ReadCllimaticFiles(files3,3);
  
  xfscanf(in, "%a", files4);
  ReadCllimaticFiles(files4,4);
  
  xfscanf(in, "%a", files5);
  ReadCllimaticFiles(files5,5);
  
  xfscanf(in, "%a", files6);
  ReadCllimaticFiles(files6,6);
  
  xfscanf(in, "%a", files7);
  ReadCllimaticFiles(files7,7);
  
  xfscanf(in, "%a", files8);
  ReadCllimaticFiles(files8,8); 
  
  //  file with radiation (JK + Tomaschko)
  xfscanf(in, "%a", files14);
  ReadCllimaticFiles(files14,14); 
  
 if (klier==1){
    //  file with radiation (JK + Klier)
    xfscanf(in, "%a", files15);
    ReadCllimaticFiles(files15,15); 
  }
}
//---------------------------------------------------------------------------


void climatcond::print(FILE *out)
{
  //fprintf (out,"\n climcond \n");
  int h,i,j;
  
  fprintf (out,"\n %ld ",mattyp );
  fprintf (out," %e ",basicdirect );
  fprintf (out," %e ",basicinclin );
  
  //-------- volba typu sorpcni izotermy....
  switch (mattyp)
    {
    case 150:{
      fprintf (out,"\n  %d  %d ",MatCharC[0],MatCharC[1]);
      h = 2;
      break;
    }
    case 203:
    case 202:{
      fprintf (out,"\n  %d  %d  %d ",MatCharC[0],MatCharC[1],MatCharC[2]);
      h = 3;
      break;
    }
    default:{
      fprintf (out,"\n  %d  %d ",MatCharC[0],MatCharC[1]);
      h = 2;
    }
    }

  for (i = 0; i <h; i++)
    {
      switch (MatCharC[i]){
      case 1: {
	fprintf (out,"\n %e ",MatConstC[i] );
	break;
      }
      case 30:{
	fprintf (out,"\n %e %e ",MatFunceC[i][0],MatFunceC[i][1] );
	break;
      }
      case 31:{
	fprintf (out,"\n %e %e %e ",MatFunceC[i][0],MatFunceC[i][1],MatFunceC[i][2] ); // to je pro Hansena
	break;
      }
      case 0:
      case 2:{
	fprintf (out,"\n %d ",numd);
	for (j=0; j<numd;j++)
	  {
	    fprintf (out,"\n %e %e ", MatDataC[i][j][0], MatDataC[i][j][1]);
	  }
	break;
      }
      default:{
	fprintf (out,"\n");
      }
      }
    }
  
  
  fprintf (out,"\n %ld ",KIND1 );
  
  switch (KIND1){
  case 0:{     //KIND1 = 0;
    break;
  }
  case 11:{    fprintf (out,"\n %e ",coefAlf );
      //KIND1 = 11;
      break;
  }
  case 12:{    fprintf (out,"\n %e ",coefAlf );
      //KIND1 = 12;
      break;
  }
  case 13:{    fprintf (out,"\n %e %e ",coefAlf, WinFact);
      break;
  }
  case 14:{
    //  aa - coefficient at velocity
    //  b - absolute coefficient
    //  eps - coefficient of emissivity
    //  alpha - absorption coefficient
    //  m - parameter of cloudiness
    fprintf (out,"\n %e %e  %e %e %e ",aa,b,eps,alpha,m);
  break;
  }
  case 15:{
    //  JK+Klier
    //  b - transmission coefficient for wind influence
    //  aa -
    //  eps - radiation coefficient
    fprintf (out,"%d %le %le",d,aa,eps);
    break;
  }
  default:{
    fprintf (out,"\n");
  }
  }
  
  fprintf (out,"\n %ld ",KIND2 );
  
  switch (KIND2)
    {
    case 0:     //KIND1 = 0;
      break;
    case 21:    fprintf (out,"\n %e ",AbsCoef  );
      //KIND1 = 11;
      break;
    case 22:    fprintf (out,"\n %e %e %e ",AbsCoef , Albedo , Latitude );
      break;
    default:{
      fprintf (stderr,"\n\n unknown definition of Read clim.data-KIND2 is required");
      fprintf (stderr,"\n in function read (file %s, line %d)\n",__FILE__,__LINE__);
    }
    }
  
  fprintf (out,"\n %ld ",KIND3 );
  
  switch (KIND3)
    {
    case 0:     //KIND1 = 0;
      break;
    case 31:    fprintf (out,"\n %e ",EmisCoeff );
      
      break;
    case 32:    fprintf (out,"\n %e ",EmisCoeff );
      break;
    default:{
      fprintf (out,"\n");
    }
    }
  
  fprintf (out,"\n %ld ",KIND4 );
  
  switch (KIND4)
    {
    case 0:     //KIND1 = 0;
      break;
    case 41:    fprintf (out,"\n %ld ",WCorPH );
      
      break;
    default:{
      fprintf (stderr,"\n\n unknown definition of Read clim.data-KIND4 is required");
      fprintf (stderr,"\n in function read (file %s, line %d)\n",__FILE__,__LINE__);
    }
      
    }
  
  fprintf (out,"\n %ld ",KIND5 );
  
  switch (KIND5)
    {
    case 0:     //KIND1 = 0;
      break;
    case 51:    fprintf (out,"\n %e %e %e %e ",ExchCoeff , RainCoeff , MinRainTemp, MinRainFlux );
      //KIND1 = 11;
      break;
    case 52:    fprintf (out,"\n %e %e %e %e ",ExchCoeff , RainCoeff , MinRainTemp, MinRainFlux );
      
      
      break;
    default:{
      fprintf (out,"\n");
    }
    }
  
  fprintf (out,"\n %ld ",KIND6 );
  
  switch (KIND6)
    {
    case 0:     //KIND1 = 0;
      break;
    case 61:    fprintf (out,"\n %e %ld ",ExchCoeffVD, RHorVP );
      //KIND1 = 11;
      break;
    case 62:    fprintf (out,"\n %e %ld %e ", ExchCoeffVD, RHorVP, WinFactVD);
      
      break;
    case 63:    fprintf (out,"\n %e %ld %e ", ExchCoeffVD, RHorVP, alfaK);
      
      break;
    case 64:    fprintf (out,"\n %e %ld %e ", ExchCoeffVD, RHorVP, alfaK);
      
      break;
    case 65:    fprintf (out,"\n %e %ld %e %e ", Ax, RHorVP, Bx, Cx);
      
      break;
    case 66:    fprintf (out,"\n %ld %d ", RHorVP, d);
      for (i = 0; i<d;i++)
	{
	  fprintf (out,"\n %e %e ",data[0][i],data [1][i]);
	}
      break;
    default:{
      fprintf (out,"\n");
    }
    }
  
  
  fprintf (out,"\n%s",files0); 
  fprintf (out,"\n%s",files1); 
  fprintf (out,"\n%s",files2); 
  fprintf (out,"\n%s",files3); 
  fprintf (out,"\n%s",files4); 
  fprintf (out,"\n%s",files5); 
  fprintf (out,"\n%s",files6); 
  fprintf (out,"\n%s",files7); 
  fprintf (out,"\n%s",files8); 
  fprintf (out,"\n%s",files14); 
  fprintf (out,"\n%s",files15); 
  
  fprintf (out,"\n");
}


double climatcond::give_flux(long icid, double * nval,long ipp,long nid)
{
  double flux;
  mednamest mn;
  
  
  mn = Tp->mednam;
  //if(nval[0] <=0.0)
  //{
//	fprintf(Outt, "%le  %le %ld \n",  Tp->time, nval[0], ipp);// kapa a decko
 // }


  switch (mn)
    {
    case 1: flux = Get_Heat_Flux(0,nval[0],ipp, nid);
      break;
    case 2:  flux = get_moisture_flux(nval[0], -1000,ipp, nid);
      break;
    case 10:{
		switch (icid)
		{
		case 0: flux = get_moisture_flux(nval[0], nval[1],ipp, nid);
			break;
		case 1: flux = Get_Heat_Flux(nval[0],nval[1],ipp, nid);
			break;
		default:{
			fprintf (stderr,"\n\n unknown definition of Clim. give flux is required");
			fprintf (stderr,"\n in function read (file %s, line %d)\n",__FILE__,__LINE__);
				}
		}
			}
		break;
	case 25: switch (icid)
	{
	case 0: flux = get_moisture_flux(nval[0], 293.15,ipp, nid);
	  break;
	case 1: flux = get_salt_flux(nval[0],nval[1], 293.15, nid);
	  break;
	default:{
	  fprintf (stderr,"\n\n unknown definition of Clim. give flux is required");
	  fprintf (stderr,"\n in function read (file %s, line %d)\n",__FILE__,__LINE__);
	}
	}
      break;
    case 30:switch (icid)
	{
	case 0: flux = get_moisture_flux(nval[0], nval[3],ipp, nid);
	  break;
	case 1: flux = get_salt_flux(nval[0],nval[1], nval[3], nid);
	  break;
	case 3: flux = Get_Heat_Flux(nval[0], nval[3],ipp, nid);
	  break;
	default:{
	  fprintf (stderr,"\n\n unknown definition of Clim. give flux is required");
	  fprintf (stderr,"\n in function read (file %s, line %d)\n",__FILE__,__LINE__);
	}
	}
      break;
    default:{
      fprintf (stderr,"\n\n unknown definition of Clim. give flux is required");
      fprintf (stderr,"\n in function read (file %s, line %d)\n",__FILE__,__LINE__);
    }
    }

  //fprintf(Outt2, "%ld %le %le %le %le\n",nid,  Tp->time, flux, nval[0], nval[1]);// kapa a decko
//  fprintf(Outt, "%d  %le \n", icid, flux);
  return (flux);
}

//---------------------------------------------------------------------------


double climatcond::get_moisture_flux(double nvalrh, double nvalT,long ipp, long nid)
{
  /*
   nvalrh - rh for Kunzel, moisture for Salt2
  */
   double jh4, jh5, jh6;
   double time;

  //T = Tm->ip[ipp].av[1];
  time = Tp->time;

  if (nvalrh < 0.0) nvalrh = 0.0;
  if (nvalrh > 1.0) nvalrh = 1.0;
  
  switch (KIND4)
    {
    case 0: jh4 = 0;
      break;
    case 41: jh4 = Condit_Water_contact(0,time);
      break;
	default:{
			fprintf (stderr,"\n\n unknown definition of Get_moisture_flux-KIND4 is required");
			fprintf (stderr,"\n in function read (file %s, line %d)\n",__FILE__,__LINE__);
			}

    }

  switch (KIND5)
    {
    case 0: jh5 = 0;
      break;
    case 51: jh5 = -1* Condit_Rain_moistureFlux(0,nvalrh,time,ipp, nid);
    break;
    case 52: jh5 = -1* Condit_Rain_moistureFlux(1,nvalrh,time,ipp, nid);
      break;
		default:{
			fprintf (stderr,"\n\n unknown definition of Get_moisture_flux-KIND5 is required");
			fprintf (stderr,"\n in function read (file %s, line %d)\n",__FILE__,__LINE__);
			}
    }

 if (nvalT != -1000)
    {
    switch(KIND6)
        {
        case 0: jh6 = 0;
        break;
        case 61: jh6 = Condit_Vapour_Diffusion_moisture_flux(0,time,nvalrh,nvalT,ipp, nid);
        break;
        case 62: jh6 = Condit_Vapour_Diffusion_moisture_flux(1,time,nvalrh,nvalT,ipp, nid);
         break;
         case 63: jh6 = Condit_Vapour_Diffusion_moisture_flux(2,time,nvalrh,nvalT,ipp, nid);
        break;
         case 64: jh6 = Condit_Vapour_Diffusion_moisture_flux(3,time,nvalrh,nvalT,ipp, nid);
        break;
        case 65: jh6 = Condit_Vapour_Diffusion_moisture_flux(4,time,nvalrh,nvalT,ipp, nid);
        break;
         case 66: jh6 = Condit_Vapour_Diffusion_moisture_flux(5,time,nvalrh,nvalT,ipp, nid);
        break;
		default:{
			fprintf (stderr,"\n\n unknown definition of Get_moisture_flux-KIND6 is required");
			fprintf (stderr,"\n in function read (file %s, line %d)\n",__FILE__,__LINE__);
			}
        }
    }

//  printf ("\n\n jh5 je %lf",jh5);
//  printf ("\n\n jh6 je %lf",jh6);
 //fprintf(Outt2, "%ld %le %le %le %le\n",nid,  Tp->time, jh6, nvalrh, nvalT);// kapa a decko
  return(jh4 +jh5 +jh6); 
}
//---------------------------------------------------------------------------

void climatcond::Sorption_izoterm_data(double Fi, double &moist)
{
  int i = 0;

switch (mattyp)
     {
	 case 158:
     case 155: {
		 if (Fi <=0)
                    {
                    Fi = 0;
                    moist = MatDataC[0][0][1];
                    }
               else
                    {
                    if (Fi < 1)
                         {
                         while (Fi > MatDataC[0][i][0])
                              {
                              i++;
                              }
                              moist = MatDataC[0][i-1][1] + ((Fi - MatDataC[0][i-1][0])*(MatDataC[0][i][1] - MatDataC[0][i-1][1]))/(MatDataC[0][i][0] - MatDataC[0][i-1][0]);
                         }
                    else {
                         moist = MatConstC[1];
                         }
                    }
               }
     break;
     case 200:
	 case 203:
     case 201:
     case 202:
     case 157:
     case 156:{
               if (Fi <=0)
                    {
                    Fi = 0;
                    moist = MatDataC[0][0][0];
                    }
               else
                    {
                    if (Fi < 1)
                         {
                         while (Fi > MatDataC[0][i][1])
                              {
                              i++;
                              }
                              moist = MatDataC[0][i-1][0] + ((Fi - MatDataC[0][i-1][1])*(MatDataC[0][i][0] - MatDataC[0][i-1][0]))/(MatDataC[0][i][1] - MatDataC[0][i-1][1]);
                         }
                    else {
                         moist = 1;
                         }
                    }
               }
     break;
	 default:{
			fprintf (stderr,"\n\n unknown definition of Sorption_izoterm_data_CLIM is required");
			fprintf (stderr,"\n in function read (file %s, line %d)\n",__FILE__,__LINE__);
			}
     }
  
}

void climatcond::find_data(double x, double & y)
{
 int i = 0;

if (x <=0)
     {
     x = 0;
     y = data[1][0];
     }
else
     {
     if (x < 1)
          {
          while (x > data[0][i])
                {
                i++;
                }
          y = data[1][i-1]+ ((x - data[0][i-1])*(data[1][i] - data[1][i-1]))/(data[0][i] - data[0][i-1]);
          }
     else {
         // y = donst[1];
          }
     }    
}

double climatcond::get_salt_flux(double w, double cf, double temp, long nid)
{
  double jh6salt;
  double time;

  //T = Tm->ip[ipp].av[1];
  time = Tp->time;

  if (temp != -1000)
    {
    switch(KIND6)
        {
        case 0: jh6salt = 0;
        break;
        case 61: jh6salt = Condit_Vapour_Diffusion_moisture_flux_salt(0,time,w,cf,temp, nid);
        break;
        case 62: jh6salt = Condit_Vapour_Diffusion_moisture_flux_salt(1,time,w,cf,temp, nid);
         break;
         case 63: jh6salt = Condit_Vapour_Diffusion_moisture_flux_salt(2,time,w,cf,temp, nid);
        break;
         case 64: jh6salt = Condit_Vapour_Diffusion_moisture_flux_salt(3,time,w,cf,temp, nid);
        break;
        case 65: jh6salt = Condit_Vapour_Diffusion_moisture_flux_salt(4,time,w,cf,temp, nid);
        break;
         case 66: jh6salt = Condit_Vapour_Diffusion_moisture_flux_salt(5,time,w,cf,temp, nid);
        break;
		default:{
			fprintf (stderr,"\n\n unknown definition of Get_salt_flux-KIND6 is required");
			fprintf (stderr,"\n in function read (file %s, line %d)\n",__FILE__,__LINE__);
			}
        }
    }
 /* if (nid == 0)
  {
	fprintf(Outt1, "%le  %le  %le  %le  %d %lf\n", time, w, cf, jh6salt*60, nid,Tt->nodes[nid].eqother[10]);// eflorescence
  }
  if (nid == 25)	
  {
	fprintf(Outt2, "%le  %le  %le  %le  %d %lf\n", time, w, cf,jh6salt*60, nid,Tt->nodes[nid].eqother[10]);// eflorescence
  } */
//   printf ("\n\n jh5 je %lf",jh5);
//  printf ("\n\n jh6 je %lf",jh6);
  return(jh6salt); 
}


double climatcond::Condit_Vapour_Diffusion_moisture_flux_salt(long VDKind,double t, double w, double cf, double temp, long nid)
{
    double Temper, RelHum, VapPres, WindFactVD,VapPresF, WallDir, WindDir, WindVel;
    double rh,betaWind, psat, betaExch, pv, smc, hvezda;

    FindValueinTimeT(t,TepValue,Temper);
    FindValueinTimeT(t,RHValue,RelHum);
    RelHum = RelHum/100;
   
    WallDir = basicdirect;
    //WindDir =(M_PI/180)*WindDir;

    // jen pro testovani/-----------------------------

    smc = MatConstC[1];
    cfmax = MatConstC[2];
 switch (mattyp)
     {
     case 203: 
		 {
			 rh = Tt->nodes[nid].eqother[8];
		 }
		 break;
     case 202:
     case 201:
     case 200: temp = 283.15; // temperature specimentu
               switch (MatCharC[0])
                         {
                         case 30:  hvezda = (MatFunceC[0][0])/(1-sqrt(1-(MatFunceC[0][1])));
                                   grunw1->give_data_si_root_fi(w,temp,0,MatFunceC[0][0],MatConstC[1],MatFunceC[0][1],hvezda,rh );
                         break;
                         //case 31:  MoistureAkt = (MatFunceC[0][0]*pow((1-(log(RHakt))/MatFunceC[0][1]),(-1/MatFunceC[0][2])));
                         case 2:   Sorption_izoterm_data(w,rh);
                         break;
                         }
     break;
	 default:{
			fprintf (stderr,"\n\n unknown definition of Condit_Vapour_Diffusion_moisture_flux_salt is required");
			fprintf (stderr,"\n in function read (file %s, line %d)\n",__FILE__,__LINE__);
			}
     }

// --------------------------------------

    switch (VDKind )
        {
        case 0: betaExch = ExchCoeffVD ;
        break;
        case 1:  FindValueinTimeT(t,WDValue,WindDir);
                 FindValueinTimeT(t,WSValue,WindVel);
                 WindDir =(M_PI/180)*WindDir;
                if (fabs (WallDir -WindDir) <= M_PI)
                    {
                    betaWind = fabs (WallDir -WindDir);
                    }
                else
                    {
                    betaWind = 2*M_PI - fabs(WallDir -WindDir);
                    }

                if (betaWind < (M_PI/2))
                    {
                    betaExch = ExchCoeffVD +WindFactVD * sqrt(WindVel );
                    }
                else
                    {
                    betaExch = ExchCoeffVD ;
                    }
        break;
        case 2: betaExch = ExchCoeffVD*w + alfaK;
        break;
        case 3: betaExch = ExchCoeffVD*exp(alfaK*w);
        break;
        case 4: betaExch = Ax*w*w + Bx*w + Cx;
        break;
        case 5: find_data(w,betaExch);
		break;
		default:{
			fprintf (stderr,"\n\n unknown definition of Condit_Vapour_Diffusion_moisture_flux_salt-VDKind is required");
			fprintf (stderr,"\n in function read (file %s, line %d)\n",__FILE__,__LINE__);
			}
        }


  // musim doplnit i moznost TRY tlaku misto RH

    psat = exp(23.5771 - 4042.9/((Temper+273.15)  - 37.58));

    if (RHorVP == 0)
        {
        VapPres = psat * RelHum ;
        }
    else
        {
        VapPres = VapPresF;
        }

    psat = exp(23.5771 - 4042.9/(temp   - 37.58));
    pv = psat * rh ;

   /* if(cf >= cfmax)
          {
          
		
		  if ((VapPres  - pv) <0)
               {
              // return (betaExch *(VapPres -pv )*cf/1000);
               return (betaExch *(pv - VapPres)*cf/1000);

               }
          else
               {
               // return(((Smc -MoistureAkt)/Smc)*betaExch *(VapPres -pv ));
               return(((smc -w)/smc)*betaExch *(pv -VapPres)*cf/1000);

               }
		return (betaExch *(pv - VapPres)*cf/1000);

          }
    else  {
          return(0);
          }
*/
	double tok;
	    if (cf >= cfmax )
		{
		/*	if (Tt->nodes[nid].eqother[10] == 0.0)
			{
				Tt->nodes[nid].eqother[12] = Tt->nodes[nid].eqother[12] + Tt->nodes[nid].eqother[13];
				Tt->nodes[nid].eqother[10] = 1.0;
			}*/
			
			if ((VapPres  - pv) <0)
			{
					if(Tt->nodes[nid].eqother[11] == 1)
						{
						tok = betaExch *(pv - VapPres)*cf/1000;
						Tt->nodes[nid].eqother[11]= Tt->nodes[nid].eqother[11] + 1;
						Tt->nodes[nid].eqother[12]= Tt->nodes[nid].eqother[12] + tok;
						Tt->nodes[nid].eqother[13]= tok;
						return(tok);
					}
					else
					{
						Tt->nodes[nid].eqother[11]= Tt->nodes[nid].eqother[11]+1;
						if(Tt->nodes[nid].eqother[11] ==5)
							Tt->nodes[nid].eqother[11] = 1;
						return(Tt->nodes[nid].eqother[13]);
					}
			}
			else
			{
					if(Tt->nodes[nid].eqother[11] == 1)
						{
						tok = ((smc -w)/smc)*betaExch *(pv -VapPres)*cf/1000;
						Tt->nodes[nid].eqother[11]= Tt->nodes[nid].eqother[11]+1;
						Tt->nodes[nid].eqother[12]= Tt->nodes[nid].eqother[12]+ tok;
						Tt->nodes[nid].eqother[13]= tok;
						return(tok);
					}
					else
					{
						Tt->nodes[nid].eqother[11]= Tt->nodes[nid].eqother[11]+1;
						if(Tt->nodes[nid].eqother[11] ==5)
							Tt->nodes[nid].eqother[11] = 1;
						return(Tt->nodes[nid].eqother[13]);
					}

			}
		}
		else
		{
			if( Tt->nodes[nid].eqother[12] > 1.0e-20)
			{
				if ((VapPres  - pv) <0)
				{
					if(Tt->nodes[nid].eqother[11] == 1)
					{
						tok =betaExch *(pv - VapPres)*cf/1000;
						if (fabs(tok) > Tt->nodes[nid].eqother[12])
							tok = Tt->nodes[nid].eqother[12];
						Tt->nodes[nid].eqother[11]= Tt->nodes[nid].eqother[11]+1;
						Tt->nodes[nid].eqother[12]= Tt->nodes[nid].eqother[12]-tok;
						Tt->nodes[nid].eqother[13]= tok;
						return(tok);
					}
					else
					{
						Tt->nodes[nid].eqother[11]= Tt->nodes[nid].eqother[11]+1;
						if(Tt->nodes[nid].eqother[11] ==5)
							Tt->nodes[nid].eqother[11] = 1;
						return(Tt->nodes[nid].eqother[13]);
					}
				}
				else
				{
					if(Tt->nodes[nid].eqother[11] == 1)
					{
						tok = ((smc -w)/smc)*betaExch *(pv -VapPres)*cf/1000;
						if (fabs(tok) > Tt->nodes[nid].eqother[12])
							tok = Tt->nodes[nid].eqother[12];
						Tt->nodes[nid].eqother[11]= Tt->nodes[nid].eqother[11]+1;
						Tt->nodes[nid].eqother[12]= Tt->nodes[nid].eqother[12]-tok;
						Tt->nodes[nid].eqother[13]= tok;
						return(tok);
					}
					else
					{
						Tt->nodes[nid].eqother[11]= Tt->nodes[nid].eqother[11]+1;
						if(Tt->nodes[nid].eqother[11] ==5)
							Tt->nodes[nid].eqother[11] = 1;
						return(Tt->nodes[nid].eqother[13]);
					}
				}
			}
			else{

				Tt->nodes[nid].eqother[11] = 1;
				Tt->nodes[nid].eqother[12] = 0.0;
				Tt->nodes[nid].eqother[13] = 0.0;
				return(0);
			}
		}
}

//rh = Tt->nodes[nid].eqother[3];
