#ifndef NODET_H
#define NODET_H

#include <stdio.h>
#include "iotools.h"
#include "aliast.h"
#include "vector.h"

/**
   class nodet defines node for transport problems
   it contains informations about nodes which are not stored in gnode
*/

class nodet
{
 public:
  nodet (void);
  ~nodet (void);
  void read (XFILE *in,long ndof);
  void print (FILE *out);

  void alloc_grad (long ncomp);
  void alloc_flux (long ncomp);
  void alloc_other (long ncomp0);
  void alloc_eqother (long ncomp0);
  void alloc_nodval ();
  void alloc_nodvali ();
  void alloc_nodvalp ();
  void alloc_nodvalt ();

  void nullgrad ();
  void nullflux ();
  void nullother ();
  void nulleqother ();
  void nullnodval ();
  void nullnodvalp ();
  void nullnodvali ();
  void nullnodvalt ();

  void storegrad (long fi,long ncomp,vector &gradv);
  void storeflux (long fi,long ncomp,vector &fluxv);
  void storeother (long ncomp,vector &otherv);
  void storeeqother (long ncomp,vector &eqotherv);

  void grad_averageval ();
  void flux_averageval ();
  void other_averageval  ();
  void eqother_averageval  ();

  void save_nodval (double *nv);
  void give_values (double *in, double *inp, double *ineq );
  void save_values (double *out);
  void actual_previous_change ();


  ///  number of DOFs on node
  long ndofn;
  ///  number of components of gradients array
  long ncompgrad;
  ///  number of components of other array
  long ncompother;
  ///  number of components of eq_other array
  long ncompeqother;

  ///  type of cross section
  crsectypet crst;
  ///  number of appropriate cross section type
  long idcs;

  ///  number of contributions to the gradient array
  long ncontr_grad;
  ///  number of contributions to the flux array
  long ncontr_flux;
  ///  number of contributions to the other array
  long ncontr_other;
  ///  number of contributions to the eqother array
  long ncontr_eqother;

  ///  array containing gradients
  double *gradient;
  ///  array containing fluxes
  double *flux;
  ///  array containing other values
  double *other;
  ///  array containing eqother values
  double *eqother;

  ///  array containing actual nodal values
  double *nodval;
  ///  array containing nodal values from the previous step
  double *nodvalp;
  ///  array containing initial nodal values
  double *nodvali;
  ///  array containing time derivative of nodal values
  double *nodvalt;
  
};

#endif
