#include "probdesct.h"
#include "globalt.h"
#include "aggregator.h"
#include <string.h>
#include "intools.h"
#include "iotools.h"
#include "gmatrix.h"

probdesct::probdesct (void)
{
  kwdsw = nokwd;

  ntm=0;
  tmatt=nomedium;
  mednam=heat;

  //  smaller number of integration points
  savemode=1;

  //  type of transport solver
  trsolv=fullnewtont;
  //trsolv=modnewtont;

  //  type of residuum computation
  trestype=lrhst;
  //trestype=fluxest;

  //  convergence control for fully newton-raphson
  convergcontrolt=no;
  
  tstorkm = (storagetype) 0; 
  tstorcm = (storagetype) 0; 
  
  limit=0.0;  zero=1.0e-20;
  
  time=0.0;  alpha=1.0;
  err=0.0;  nii=0;
  errarr = NULL;
  threshrhs = NULL;

  stochasticcalc=0;
  diagcap=0;

  gdim=0;  
  scale1 = 1.0;
  scale2 = 1.0;
  scale3 = 1.0;
  scale4 = 1.0;

  //  gradients are not computed and stored
  gradcomp = 0;
  //  fluxes are not computed and stored
  fluxcomp = 0;
  //  components of array other are not computed and stored
  othercomp = 0;
  //  components of array eqother are computed and stored
  eqothercomp = 1;
  
  //  gradients are not averaged
  gradaver = 0;
  //  fluxes are not averaged
  fluxaver = 0;
  //  array other is not averaged
  otheraver = 0;
  //  array eqother is not averaged
  eqotheraver = 0;
  
  //  actual nodal values are not stored on node
  nvs=0;
  //  previous nodal values are not stored on node
  pnvs=0;
  //  initial nodal values are not stored on node
  invs=0;
  //  time derivatives of actual nodal values are not stored on node
  tdnvs=0;
  
  
  memset (name,0,sizeof(*name)*1001);
  path = filename = suffix = NULL;

  //  type of time printing
  tprt = dayst;
  //tprt = secondst;
  
  ssle = new slesolv();

  // backup on harddisk
  hdbcont.hdbtype = nohdb;  // no backup
  hdbcont.hdbfmtr = text;
  hdbcont.hdbfmts = text;
  hdbcont.hdbnames[0] = 0;
  hdbcont.hdbnamer[0] = 0;
  hdbcont.prec=15;
  hdbcont.selother_r  = NULL;
  hdbcont.selother_s  = NULL;
  hdbcont.selother_id = NULL;
}

probdesct::~probdesct (void)
{
  delete [] errarr;
  delete [] threshrhs;

  delete [] path; 
  delete [] filename; 
  delete [] suffix;
  delete ssle;
}


/**  function reads basic data about solved problem
     JK, 25.9.2001
     
     @param in - input file
*/
void probdesct::read (XFILE *in)
{
  long i;

  //  problem name
  xfscanf (in,"%a",name);
  
  //  detail messages
  Mesprt=0;
  xfscanf (in,"%k%ld","mesprt",&Mesprt);
  if (Mesprt==1)  fprintf (stdout,"\n detail information will be printed\n");
  else fprintf (stdout,"\n only important messages will be printed\n");
  
  //  problem type
  xfscanf (in,"%k%m","problemtype",&problemtypet_kwdset,(int*)&tprob);
  
  //  transported matter
  xfscanf (in,"%k%m","transmatter",&transmattert_kwdset,(int*)&tmatt);
  switch (tmatt){
  case onemedium:{        ntm=1;  break; }
  case twomediacoup:{     ntm=2;  break; }
  case threemediacoup:{   ntm=3;  break; }
  case fourmediacoup:{    ntm=4;  break; }
  default:{
    print_err("unknown type of transported matter is required",__FILE__,__LINE__,__func__);
  }
  }
  
  //  names of transported media
  xfscanf (in,"%k%m","mednames",&mednamest_kwdset,(int*)&mednam);
  
  // scale of transported media
  switch (tmatt){
  case onemedium:{        xfscanf (in,"%k%lf","scale1",&scale1); break; }
  case twomediacoup:{     xfscanf (in,"%k%lf %k%lf","scale1",&scale1,"scale2",&scale2); break; }
  case threemediacoup:{   xfscanf (in,"%k%lf %k%lf %k%lf","scale1",&scale1,"scale2",&scale2,"scale3",&scale3); break; }
  case fourmediacoup:{    xfscanf (in,"%k%lf %k%lf %k%lf %k%lf","scale1",&scale1,"scale2",&scale2,"scale3",&scale3,"scale4",&scale4); break; }
  default:{
    print_err("unknown type of transported matter is required",__FILE__,__LINE__,__func__);
  }
  }
  
  //  gravity acceleration is taken into account
  xfscanf (in,"%k%m","gravityacceleration",&gravityaccelerationt_kwdset,(int*)&tgravity);
  
  if(tgravity == gr_yes){
    xfscanf (in,"%lf %lf %lf",&gr1,&gr2,&gr3);
  }


  /*************************************/
  //  computation of gradients and fluxes 17.07.2005 TKr.
  /*   fscanf (in,"%ld",&gradcomp);
       if (gradcomp!=0 && gradcomp!=1)
       fprintf (stderr,"\n\n wrong identification of gradient computation in function probdesc::read (file %s, line %d).\n",
       __FILE__,__LINE__);
       if (gradcomp==1)  fscanf (in,"%ld",&gradaver);
       if (gradaver!=0 && gradaver!=1)
       fprintf (stderr,"\n\n wrong identification of gradient averaging in function probdesc::read (file %s, line %d).\n",
       __FILE__,__LINE__);
       
       fscanf (in,"%ld",&fluxcomp);
       if (fluxcomp!=0 && fluxcomp!=1)
       fprintf (stderr,"\n\n wrong identification of flux computation in function probdesc::read (file %s, line %d).\n",
       __FILE__,__LINE__);
       if (fluxcomp==1)  fscanf (in,"%ld",&fluxaver);
       if (fluxaver!=0 && fluxaver!=1)
       fprintf (stderr,"\n\n wrong identification of flux averaging in function probdesc::read (file %s, line %d).\n",
       __FILE__,__LINE__);
       
       if (Mesprt==1){
       if (gradcomp==0)  fprintf (stdout,"\n gradients will not be computed and stored");
       else{
       fprintf (stdout,"\n grads will be computed and stored");
       if (gradaver==0)  fprintf (stdout,"\n gradients will not be averaged");
       if (gradaver==1)  fprintf (stdout,"\n gradients will be averaged at nodes");
       }
       
       if (fluxcomp==0)  fprintf (stdout,"\n fluxes will not be computed and stored");
       else{
       fprintf (stdout,"\n fluxes will be computed and stored");
       if (fluxaver==0)  fprintf (stdout,"\n fluxes will not be averaged");
       if (fluxaver==1)  fprintf (stdout,"\n fluxes will be averaged at nodes");
       }
       }
  */
  /*************************************/



  // **********************************
  //  stochastic or fuzzy computation
  // **********************************
  //  stochasticcalc=0 - deterministic computation
  //  stochasticcalc=1 - stochastic/fuzzy computation, data are read all at once
  //  stochasticcalc=2 - stochastic/fuzzy computation, data are read sequentially
  //  stochasticcalc=3 - stochastic/fuzzy computation, data are generated in the code
  xfscanf (in,"%k%ld","stochasticcalc",&stochasticcalc);
  
  // ***************************
  //  renumbering of the nodes
  // ***************************
  xfscanf (in,"%k%m","noderenumber",&noderenumb_kwdset,(int*)&Gtt->nodren);
  if (Mesprt==1){
    switch (Gtt->nodren){
    case no_renumbering:{
      fprintf (stdout,"\n nodes will not be renumbered");
      break;
    }
    case rev_cuthill_mckee:{
      fprintf (stdout,"\n nodes will be renumbered by reverse Cuthill-McKee algorithm");
      break;
    }
    default:{
      print_err("unknown type of node reordering is required",__FILE__,__LINE__,__func__);
    }
    }
  }
  
  
  switch (tprob){
    // ****************************
    //  LINEAR STATIONARY PROBLEM
    // ****************************
  case stationary_problem:{
    if (Mesprt==1)  fprintf (stdout,"\n linear stationary problem");
    
    //  tstorsm - type of storage of conductivity matrix
    xfscanf (in,"%k%m","conductmatstor",&storagetype_kwdset,(int*)&tstorkm);
    
    //  data about solver of system of linear equations
    ssle->read (Gtt,in,Mesprt);
    
    break;
  }

    // ****************************
    //  NONLINEAR STATIONARY PROBLEM
    // ****************************
  case nonlinear_stationary_problem:{
    if (Mesprt==1)  fprintf (stdout,"\n nonlinear stationary problem");
    
    //  data about nonlinear solver
    nlman.read (in,Mesprt);

    //  tstorsm - type of storage of conductivity matrix
    xfscanf (in,"%k%m","conductmatstor",&storagetype_kwdset,(int*)&tstorkm);
    
    //  data about solver of system of linear equations
    ssle->read (Gtt,in,Mesprt);
    
    break;
  }

    // *******************************
    //  LINEAR NONSTATIONARY PROBLEM
    // *******************************
  case nonstationary_problem:{
    if (Mesprt==1)  fprintf (stdout,"\n linear nonstationary problem");
    
    //  time controller
    timecont.read (in);

    // backup controler
    hdbcont.read(in);
    if ((Mesprt==1) && hdbcont.hdbtype)
      fprintf(stdout, "\n backup will be performed %d", hdbcont.hdbtype);

    xfscanf (in,"%k%lf","alpha_integration", &alpha);
    
    xfscanf (in,"%k%m%k%m","conductmatstor",&storagetype_kwdset,(int*)&tstorkm,"capacmatstor",&storagetype_kwdset,(int*)&tstorcm);
    
    //  data about solver of system of linear equations
    ssle->read (Gtt,in,Mesprt);
    
    // diagonalization of capacity matrix
    xfscanf (in,"%k%ld","diagonalization",&diagcap);
  
    break;
  }
    
    // ***************************************
    //  LINEAR NONSTATIONARY GROWING PROBLEM
    // ***************************************
  case growing_np_problem:{
    if (Mesprt==1)  fprintf (stdout,"\n nonstationary problem with growing number of elements");
    
    //  time controller
    timecont.read (in);

    xfscanf (in,"%k%lf","alpha_integration", &alpha);

    xfscanf (in,"%k%m%k%m","conductmatstor",&storagetype_kwdset,(int*)&tstorkm,"capacmatstor",&storagetype_kwdset,(int*)&tstorcm);
    
    //  data about solver of system of linear equations
    ssle->read (Gtt,in,Mesprt);
  
    // diagonalization of capacity matrix
    xfscanf (in,"%k%ld","diagonalization",&diagcap);

    //  degrees of freedom are controlled by a time function
    Gtt->dofcontr=1;
    
    //  this problem requires storage of actual nodal values on nodes
    nvs=1;
    //  this problem requires storage of initial nodal values on nodes
    invs=1;
    //  this problem requires storage of time derivatives of nodal values on nodes
    tdnvs=1;
    
    break;
  }
    
    // **********************************
    //  NONLINEAR NONSTATIONARY PROBLEM
    // **********************************
  case nonlinear_nonstationary_problem:{
    if (Mesprt==1)  fprintf (stdout,"\n nonlinear nonstationary problem");
    
    //  time controller
    timecont.read (in);
    
    // backup controler
    hdbcont.read(in);
    if ((Mesprt==1) && hdbcont.hdbtype)
      fprintf(stdout, "\n backup will be performed %d", hdbcont.hdbtype);

    errarr = new double [ntm];
    threshrhs = new double [ntm];

    //xfscanf (in,"%k%lf %lf %lf %ld","alpha_integration", &alpha,&err,&threshrhs,&nii);
    xfscanf (in,"%k%lf %ld","alpha_integration", &alpha,&nii);

    for (i=0;i<ntm;i++){
      xfscanf (in,"%lf %lf",errarr+i,threshrhs+i);
    }

    
    //  type of transport solver
    xfscanf (in,"%k%m","transportsolvertype",&nlsolvertype_kwdset,(int*)&trsolv);
    
    //  type of residuum computation
    xfscanf (in,"%k%m","residuumcomputationtype",&transpresiduumtype_kwdset,(int*)&trestype);
    
    //  convergence control for fully newton-raphson
    xfscanf (in,"%k%m","convergencecontrol",&answertype_kwdset,(int*)&convergcontrolt);


    xfscanf (in,"%k%m%k%m","conductmatstor",&storagetype_kwdset,(int*)&tstorkm,"capacmatstor",&storagetype_kwdset,(int*)&tstorcm);
    
    //  data about solver of system of linear equations
    ssle->read (Gtt,in,Mesprt);
    
    // diagonalization of capacity matrix
    xfscanf (in,"%k%ld","diagonalization",&diagcap);
    
    
    break;
  }
    
    // ***************************************
    //  NONLINEAR NONSTATIONARY GROWING PROBLEM
    // ***************************************
  case growing_np_problem_nonlin:{
    if (Mesprt==1)  fprintf (stdout,"\n nonlinear nonstationary problem with growing number of elements");
    
    //  time controller
    timecont.read (in);

    xfscanf (in,"%k%lf %lf %lf %ld","alpha_integration",&alpha,&err,&threshrhs,&nii);


    //  type of transport solver
    xfscanf (in,"%k%m","transportsolvertype",&transpsolver_kwdset,(int*)&trsolv);
    
    //  type of residuum computation
    xfscanf (in,"%k%m","residuumcomputationtype",&transpresiduumtype_kwdset,(int*)&trestype);
    
    //  convergence control for fully newton-raphson
    xfscanf (in,"%k%m","convergencecontrol",&answertype_kwdset,(int*)&convergcontrolt);

    xfscanf (in,"%k%m%k%m","conductmatstor",&storagetype_kwdset,(int*)&tstorkm,"capacmatstor",&storagetype_kwdset,(int*)&tstorcm);
    
    //  data about solver of system of linear equations
    ssle->read (Gtt,in,Mesprt);
  
    // diagonalization of capacity matrix
    xfscanf (in,"%k%ld","diagonalization",&diagcap);
  
    //  degrees of freedom are controlled by a time function
    Gtt->dofcontr=1;
    
    //  this problem requires storage of actual nodal values on nodes
    nvs=1;
    //  this problem requires storage of initial nodal values on nodes
    invs=1;
    //  this problem requires storage of time derivatives of nodal values on nodes
    tdnvs=1;
 
    break;
  }
    
    // *******************************************
    //  NONSTATIONARY PROBLEM WITH DISCONTINUITY
    // *******************************************
  case discont_nonstat_problem:
  case discont_nonlin_nonstat_problem:{
    if (Mesprt==1)  fprintf (stdout,"\n discontinuous nonlinear_nonstationary problem");
    
    //  time controller
    timecont.read (in);
 
    // backup controler
    hdbcont.read(in);
    if ((Mesprt==1) && hdbcont.hdbtype)
      fprintf(stdout, "\n backup will be performed %d", hdbcont.hdbtype);

    errarr = new double [ntm];
    threshrhs = new double [ntm];
    
    xfscanf (in,"%k%lf %ld","alpha_integration",&alpha,&nii);
    
    for (i=0;i<ntm;i++){
      xfscanf (in,"%lf %lf",errarr+i,threshrhs+i);
    }
    
    xfscanf (in,"%k%m%k%m","conductmatstor",&storagetype_kwdset,(int*)&tstorkm,"capacmatstor",&storagetype_kwdset,(int*)&tstorcm);
    
    //  data about solver of system of linear equations
    ssle->read (Gtt,in,Mesprt);
  
    // diagonalization of capacity matrix
    xfscanf (in,"%k%ld","diagonalization",&diagcap);
    
    break;
  }
    
    // ***************************************
    //  provisional connection with Hermes2D
    // ***************************************
  case hermes:{
    
    break;
  }

  default:{
    print_err("unknown problem type is required",__FILE__,__LINE__,__func__);
  }
  }
  
  // **********************************
  //  special requirements of solvers
  // **********************************
  if (ssle->tlinsol==sfeti){
    //  sequential topology will be read in function transtop::read
    Gtt->rst=1;
    
    if (ssle->feti.prec==feti_dirichlet){
      //  code numbers will be generated with respect to the Schur complement method
      //  internal DOFs are ordered first, interface DOFs are ordered afterwards
      Gtt->cngen=2;
    }
  }
  if (ssle->tlinsol==sfetidef){
    //  sequential topology will be read in function transtop::read
    Gtt->rst=2;
  }
  if (ssle->prec.pt==boss){
    if (ssle->prec.agg->impl==2){
      //  sequential topology will be read in function transtop::read
      Gtt->rst=1;
    }
  }

  
}

/**  function prints basic data about solved problem
     JK, 25.9.2001
     
     @param in - output file
*/
void probdesct::print (FILE *out)
{
  long i;

  //  problem name
  fprintf (out, "%s\n", name);

  //  detail messages
  fprintf (out,"%ld\n", Mesprt);

  //  problem type
  fprintf (out,"%d", tprob);

  //  transported matter
  fprintf (out," %d", tmatt);
  
  //  names of transported media
  fprintf (out, " %d", mednam);
  
  // scale of transperted media
  switch (tmatt){
  case onemedium:{        fprintf (out," %f\n",scale1); break; }
  case twomediacoup:{     fprintf (out," %f %f\n",scale1,scale2); break; }
  case threemediacoup:{   fprintf (out," %f %f %f\n",scale1,scale2,scale3); break; }
  case fourmediacoup:{    fprintf (out," %f %f %f %f\n",scale1,scale2,scale3,scale4); break; }
  default:{
    print_err("unknown type of transported matter is required",__FILE__,__LINE__,__func__);
  }
  }
  
  //  gravity acceleration
  fprintf (out,"%d\n",tgravity);
  if(tgravity == gr_yes){
    fprintf (out,"%lf %lf %lf\n",gr1,gr2,gr3);
  }
  
  
  // stochastic calculation
  fprintf (out,"%ld\n",stochasticcalc);

  // ***************************
  //  renumbering of the nodes
  // ***************************
  fprintf (out,"%d\n",Gtt->nodren);



  switch (tprob){
    // ****************************
    //  LINEAR STATIONARY PROBLEM
    // ****************************
  case stationary_problem:{
    fprintf (out,"%d\n", tstorkm);
    
    ssle->print (out);

    break;
  }
    
    // ****************************
    //  NONLINEAR STATIONARY PROBLEM
    // ****************************
  case nonlinear_stationary_problem:{
    
    nlman.print (out);
    
    fprintf (out,"%d\n", tstorkm);
    
    ssle->print (out);

    break;
  }
    
    // *******************************
    //  LINEAR NONSTATIONARY PROBLEM
    // *******************************
  case nonstationary_problem:{
    
    timecont.print (out);

    fprintf (out, "%lf\n",alpha);
    
    fprintf (out,"%d %d ",tstorkm,tstorcm);
    
    ssle->print (out);

    fprintf (out,"%ld\n",diagcap);

    fprintf (out,"\n");    
    
    break;
  }

    // ***************************************
    //  LINEAR NONSTATIONARY GROWING PROBLEM
    // ***************************************
  case growing_np_problem:{
    
    timecont.print (out);

    fprintf (out, "%lf\n",alpha);
    
    fprintf (out,"%d %d ",tstorkm,tstorcm);
    
    ssle->print (out);

    fprintf (out,"%ld\n",diagcap);

    fprintf (out,"\n");    
    
    break;
  }

    // **********************************
    //  NONLINEAR NONSTATIONARY PROBLEM
    // **********************************
  case nonlinear_nonstationary_problem:{
    
    timecont.print (out);

    fprintf (out, "%lf %ld\n", alpha,nii);
    
    for (i=0;i<ntm;i++){
      fprintf (out, "%lf %lf\n", errarr[i], threshrhs[i]);
    }
    
    fprintf (out,"%d %d %d\n",trsolv,trestype,convergcontrolt);

    fprintf (out,"%d %d \n",tstorkm,tstorcm);

    ssle->print (out);

    fprintf (out,"%ld\n",diagcap);

    fprintf (out,"\n");    

    break;
  }

    // ***************************************
    //  NONLINEAR NONSTATIONARY GROWING PROBLEM
    // ***************************************
  case growing_np_problem_nonlin:{
    
    timecont.print (out);

    fprintf (out, "%lf %ld\n",alpha,nii);
    for (i=0;i<ntm;i++){
      fprintf (out, "%lf %lf\n",errarr[i],threshrhs[i]);
    }
    
    fprintf (out,"%d %d %d\n",trsolv,trestype,convergcontrolt);
    
    fprintf (out,"%d %d ",tstorkm,tstorcm);
    
    ssle->print (out);

    fprintf (out,"%ld\n",diagcap);

    fprintf (out,"\n");    
    
    break;
  }
    
    // *******************************************
    //  NONSTATIONARY PROBLEM WITH DISCONTINUITY
    // *******************************************
  case discont_nonstat_problem:
  case discont_nonlin_nonstat_problem:{
    
    //  time controller
    timecont.print (out);

    
    fprintf (out,"%lf %ld\n",alpha,nii);
    for (i=0;i<ntm;i++){
      fprintf (out,"%lf %lf\n",errarr[i],threshrhs[i]);
    }

    fprintf (out,"%d %d\n",tstorkm,tstorcm);
    
    //  data about solver of system of linear equations
    ssle->print (out);
  
    // diagonalization of capacity matrix
    fprintf (out,"%ld\n",diagcap);
    
    break;
  }

  default:{
    print_err("unknown problem type is required",__FILE__,__LINE__,__func__);
  }
  }  
}


/**
   Function decomposes name of file(with path) to path,filename and suffix.
   ppp/ppp/fff.sss.sss -> path = "ppp/ppp" , name = "fff" , suffix = ".sss.sss"
   
   @param file - decomposed name of file
   @param path,name,suffix - components of name of file
   
   created  1.4.2003, Ladislav Svoboda, termit@cml.fsv.cvut.cz
 */
void probdesct::filename_decompositiont (const char *file,char *&patht,char *&namet,char *&suffixt)
{
  char *fix;
  patht = new char[strlen(file)+1];
  namet = new char[strlen(file)+1];
  suffixt = new char[strlen(file)+1];
  
  strcpy (patht,file);
  fix = strrchr(patht,'/');
  if (fix==NULL){
    strcpy (namet,patht);
    patht[0] = '\0';
  }
  else{
    strcpy (namet,fix+1);
    fix[1] = '\0';
  }
  
  fix = strrchr(namet,'.');
  if (fix==NULL)
    suffixt[0] = '\0';
  else{
    strcpy (suffixt,fix);
    fix[0] = '\0';
  }
}
