/*
  File:     quadlineart.cpp
  Author:   Jaroslav Kruis, 31.3.2001
  Purpose:  twodimensional quadrilateral element with linear approximation functions
*/

#include "globalt.h"
#include "quadlineart.h"
#include "genfile.h"
#include "globmatt.h"

quadlineart::quadlineart (void)
{
  long i;
  
  //  number of nodes on element
  nne=4;
  //  number of edges
  ned=4;
  //  number of nodes on one edge
  nned=2;
  //  geometrical dimension
  ncomp=2;
  
  //  number of transported variables
  ntm=Tp->ntm;
  
  
  dofe = new long* [ntm];
  nip = new long* [ntm];
  intordkm = new long* [ntm];
  intordcm = new long* [ntm];
  ordering = new long* [ntm];
  for (i=0;i<ntm;i++){
    dofe[i] = new long [ntm];
    nip[i] = new long [ntm];
    intordkm[i] = new long [ntm];
    intordcm[i] = new long [ntm];
    ordering[i] = new long [nne];
  }
  
  
  switch (Tp->tmatt){
  case nomedium:{  break; }
  case onemedium:{
    ordering[0][0]=1;  ordering[0][1]=2;  ordering[0][2]=3;  ordering[0][3]=4;
    dofe[0][0]=4;  intordkm[0][0]=2;  intordcm[0][0]=2;
    nip[0][0]=8;
    ndofe=4;  napfun=1;
    break;
  }
  case twomediacoup:{
    ordering[0][0]=1;  ordering[0][1]=3;  ordering[0][2]=5;  ordering[0][3]=7;
    ordering[1][0]=2;  ordering[1][1]=4;  ordering[1][2]=6;  ordering[1][3]=8;
    
    intordkm[0][0]=2;  intordkm[0][1]=2;  intordkm[1][0]=2;  intordkm[1][1]=2;
    intordcm[0][0]=2;  intordcm[0][1]=2;  intordcm[1][0]=2;  intordcm[1][1]=2;
    
    if (Tp->savemode==0){
      nip[0][0]=8;       nip[0][1]=8;       nip[1][0]=8;       nip[1][1]=8;
    }
    if (Tp->savemode==1){
      nip[0][0]=8;       nip[0][1]=0;       nip[1][0]=0;       nip[1][1]=0;
    }
    
    
    dofe[0][0]=4;  dofe[0][1]=4;  dofe[1][0]=4;  dofe[1][1]=4;
    ndofe=8;  napfun=2;
    break;
  }
  case threemediacoup:{
    ordering[0][0]=1;   ordering[0][1] =4;  ordering[0][2] =7;  ordering[0][3] =10;
    ordering[1][0]=2;   ordering[1][1] =5;  ordering[1][2] =8;  ordering[1][3] =11;
    ordering[2][0]=3;   ordering[2][1] =6;  ordering[2][2] =9;  ordering[2][3] =12;

    intordkm[0][0]=2;  intordkm[0][1]=2;  intordkm[0][2]=2;
    intordkm[1][0]=2;  intordkm[1][1]=2;  intordkm[1][2]=2;
    intordkm[2][0]=2;  intordkm[2][1]=2;  intordkm[2][2]=2;

    intordcm[0][0]=2;  intordcm[0][1]=2;  intordcm[0][2]=2;
    intordcm[1][0]=2;  intordcm[1][1]=2;  intordcm[1][2]=2;
    intordcm[2][0]=2;  intordcm[2][1]=2;  intordcm[2][2]=2;
    
    if (Tp->savemode==0){
      nip[0][0]=8;  nip[0][1]=8;  nip[0][2]=8;
      nip[1][0]=8;  nip[1][1]=8;  nip[1][2]=8;
      nip[2][0]=8;  nip[2][1]=8;  nip[2][2]=8;
    }
    if (Tp->savemode==1){
      nip[0][0]=8;  nip[0][1]=0;  nip[0][2]=0;
      nip[1][0]=0;  nip[1][1]=0;  nip[1][2]=0;
      nip[2][0]=0;  nip[2][1]=0;  nip[2][2]=0;
    }
    
    dofe[0][0]=4;  dofe[0][1]=4;  dofe[0][2]=4;
    dofe[1][0]=4;  dofe[1][1]=4;  dofe[1][2]=4;
    dofe[2][0]=4;  dofe[2][1]=4;  dofe[2][2]=4;


    ndofe=12;  napfun=3;
    break;
  }
  case fourmediacoup:{
    ordering[0][0]=1;   ordering[0][1] =5;  ordering[0][2] =9;   ordering[0][3] =13;
    ordering[1][0]=2;   ordering[1][1] =6;  ordering[1][2] =10;  ordering[1][3] =14;
    ordering[2][0]=3;   ordering[2][1] =7;  ordering[2][2] =11;  ordering[2][3] =15;
    ordering[3][0]=4;   ordering[3][1] =8;  ordering[3][2] =12;  ordering[3][3] =16;

    intordkm[0][0]=2;  intordkm[0][1]=2;  intordkm[0][2]=2;  intordkm[0][3]=2;
    intordkm[1][0]=2;  intordkm[1][1]=2;  intordkm[1][2]=2;  intordkm[1][3]=2;
    intordkm[2][0]=2;  intordkm[2][1]=2;  intordkm[2][2]=2;  intordkm[2][3]=2;
    intordkm[3][0]=2;  intordkm[3][1]=2;  intordkm[3][2]=2;  intordkm[3][3]=2;

    intordcm[0][0]=2;  intordcm[0][1]=2;  intordcm[0][2]=2;  intordcm[0][3]=2;
    intordcm[1][0]=2;  intordcm[1][1]=2;  intordcm[1][2]=2;  intordcm[1][3]=2;
    intordcm[2][0]=2;  intordcm[2][1]=2;  intordcm[2][2]=2;  intordcm[2][3]=2;
    intordcm[3][0]=2;  intordcm[3][1]=2;  intordcm[3][2]=2;  intordcm[3][3]=2;
    
    if (Tp->savemode==0){
      nip[0][0]=8;  nip[0][1]=8;  nip[0][2]=8;  nip[0][3]=8;
      nip[1][0]=8;  nip[1][1]=8;  nip[1][2]=8;  nip[1][3]=8;
      nip[2][0]=8;  nip[2][1]=8;  nip[2][2]=8;  nip[2][3]=8;
      nip[3][0]=8;  nip[3][1]=8;  nip[3][2]=8;  nip[3][3]=8;
    }
    if (Tp->savemode==1){
      nip[0][0]=8;  nip[0][1]=0;  nip[0][2]=0;  nip[0][3]=0;
      nip[1][0]=0;  nip[1][1]=0;  nip[1][2]=0;  nip[1][3]=0;
      nip[2][0]=0;  nip[2][1]=0;  nip[2][2]=0;  nip[2][3]=0;
      nip[3][0]=0;  nip[3][1]=0;  nip[3][2]=0;  nip[3][3]=0;
    }
    
    dofe[0][0]=4;  dofe[0][1]=4;  dofe[0][2]=4;  dofe[0][3]=4;
    dofe[1][0]=4;  dofe[1][1]=4;  dofe[1][2]=4;  dofe[1][3]=4;
    dofe[2][0]=4;  dofe[2][1]=4;  dofe[2][2]=4;  dofe[2][3]=4;
    dofe[3][0]=4;  dofe[3][1]=4;  dofe[3][2]=4;  dofe[3][3]=4;

    ndofe=16;  napfun=4;
    break;
  }
  default:{
    print_err("unknown number of transported matters is required",__FILE__,__LINE__,__func__);
  }
  }
}

quadlineart::~quadlineart (void)
{
  long i;

  for (i=0;i<ntm;i++){
    delete [] dofe[i];
    delete [] nip[i];
    delete [] intordkm[i];
    delete [] intordcm[i];
    delete [] ordering[i];
  }
  delete [] dofe;
  delete [] nip;
  delete [] intordkm;
  delete [] intordcm;
  delete [] ordering;
}

/**
   function assembles element code numbers
   they are used for localization between all element unknowns and unknowns related to one matter
   
   @param cn - code numbers
   @param ri - number of matter (usually row index)
*/
void quadlineart::codnum (long *cn,long ri)
{
  long i;
  for (i=0;i<nne;i++){
    cn[i]=ordering[ri][i];
  }
}

/**
   function approximates function defined by nodal values
   
   @param xi,eta - coordinates on element
   @param nodval - %vector of nodal values

   JK, 8.5.2002
*/
double quadlineart::approx (double xi,double eta,vector &nodval)
{
  double f;
  vector bf(nne);
  
  bf_lin_4_2d (bf.a,xi,eta);
  
  scprd (bf,nodval,f);

  return f;
}


/**
   function assembles coordinates of rquired integration point
   
   @param eid - element id
   @param ipp - integration point id
   @param ri - row index
   @param ci - column index
   @param ipcoord - array containing coordinates of required integration point
   
   JK, 8.5.2002 + TKr 14.11.2003
*/
void quadlineart::ipcoord (long eid,long ipp,long ri,long ci,vector &coord)
{
  long i,j,ii;
  double xi,eta;
  vector x(nne),y(nne),w(intordkm[ri][ci]),gp(intordkm[ri][ci]);
  
  gauss_points (gp.a,w.a,intordkm[ri][ci]);
  Tt->give_node_coord2d (x,y,eid);
  
  if (Tp->savemode==0)
    ii=Tt->elements[eid].ipp[ri][ci];
  if (Tp->savemode==1)
    ii=Tt->elements[eid].ipp[0][0];
  

  for (i=0;i<intordkm[ri][ci];i++){
    xi=gp[i];
    for (j=0;j<intordkm[ri][ci];j++){
      eta=gp[j];
      if (ii==ipp){
	coord[0]=approx (xi,eta,x);
	coord[1]=approx (xi,eta,y);
	coord[2]=0.0;
      }
      ii++;
    }
  }
}

/**
   function assembles coordinates of integration points in block [ri][ci]
   
   @param eid - element id
   @param ri - row index
   @param ci - column index
   @param ipcoord - array containing coordinates of integration points
   
   JK, 8.5.2002 + TKr 14.11.2003
*/
void quadlineart::ipcoordblock (long eid,long ri,long ci,double **coord)
{
  long i,j,k;
  double xi,eta;
  vector x(nne),y(nne),w(intordkm[ri][ci]),gp(intordkm[ri][ci]);
  
  gauss_points (gp.a,w.a,intordkm[ri][ci]);
  Tt->give_node_coord2d (x,y,eid);
  
  k=0;
  for (i=0;i<intordkm[ri][ci];i++){
    xi=gp[i];
    for (j=0;j<intordkm[ri][ci];j++){
      eta=gp[j];
      
      coord[k][0]=approx (xi,eta,x);
      coord[k][1]=approx (xi,eta,y);
      coord[k++][2]=0.0;
    }
  }
}

/**
   function computes values in integration points from nodal values
   (function interpolates the nodal values to integration points)

   eid - element id

   JK, 25.9.2001
*/
void quadlineart::intpointval (long eid)
{
  long i,j,k,ii,jj,ipp;
  double xi,eta,val;
  ivector cn(ndofe);
  vector r(ndofe),t(nne),gp,w;
  
  Tt->give_code_numbers (eid,cn.a);
  //  nodal values on element
  elemvalues (0,eid,r.a,cn.a,ndofe);
  
  for (k=0;k<Tp->ntm;k++){
    
    for (i=0;i<dofe[k][k];i++){
      t[i]=r[ordering[k][i]-1];
    }
    
    for (ii=0;ii<Tp->ntm;ii++){
      for (jj=0;jj<Tp->ntm;jj++){
	
	//  interpolation to integration points of conductivity matrix
	allocv (intordkm[ii][jj],gp);
	allocv (intordkm[ii][jj],w);
	gauss_points (gp.a,w.a,intordkm[ii][jj]);
	
	ipp=Tt->elements[eid].ipp[ii][jj];
	for (i=0;i<intordkm[ii][jj];i++){
	  xi=gp[i];
	  for (j=0;j<intordkm[ii][jj];j++){
	    eta=gp[j];
	    
	    val = approx (xi,eta,t);
	    Tm->ip[ipp].av[k]=val;
	    ipp++;
	  }
	}
	
	destrv (gp);  destrv (w);
	
	//  interpolation to integration points of capacity matrix
	allocv (intordcm[ii][jj],gp);
	allocv (intordcm[ii][jj],w);
	gauss_points (gp.a,w.a,intordcm[ii][jj]);
	
	for (i=0;i<intordcm[ii][jj];i++){
	  xi=gp[i];
	  for (j=0;j<intordcm[ii][jj];j++){
	    eta=gp[j];
	    
	    val = approx (xi,eta,t);
	    Tm->ip[ipp].av[k]=val;
	    ipp++;
	  }
	}
	
	destrv (gp);  destrv (w);

	if (Tp->savemode==1)  break;
      }
      if (Tp->savemode==1)  break;
    }
  }

}


/**
   function computes gradients in integration points from nodal values
   
   eid - element id
   
   JK, 25.9.2001   
*/
void quadlineart::intpointgrad (long eid)
{
  long i,j,ii,jj,k,ipp;
  double xi,eta,jac;
  ivector cn(ndofe);
  vector x(nne),y(nne),r(ndofe),t(nne),gp,w,grad(ncomp);
  matrix gm(ncomp,nne);
  
  //  nodal coordinates
  Tt->give_node_coord2d (x,y,eid);
  //  code numbers
  Tt->give_code_numbers (eid,cn.a);
  //  nodal values on element
  elemvalues (0,eid,r.a,cn.a,ndofe);
  
  for (k=0;k<Tp->ntm;k++){
    
    for (i=0;i<dofe[k][k];i++){
      t[i]=r[ordering[k][i]-1];
    }
    
    for (ii=0;ii<Tp->ntm;ii++){
      for (jj=0;jj<Tp->ntm;jj++){
	
	//  computation of gradients in integration points of conductivity matrix
	allocv (intordkm[ii][jj],gp);
	allocv (intordkm[ii][jj],w);
	gauss_points (gp.a,w.a,intordkm[ii][jj]);
	
	ipp=Tt->elements[eid].ipp[ii][jj];
	for (i=0;i<intordkm[ii][jj];i++){
	  xi=gp[i];
	  for (j=0;j<intordkm[ii][jj];j++){
	    eta=gp[j];
	    
	    //  matrix of gradients
	    grad_matrix (gm,x,y,xi,eta,jac);
	    mxv (gm,t,grad);
	    Tm->storegrad (k,ipp,0,grad);
	    ipp++;
	  }
	}
	destrv (gp);  destrv (w);
	
	//  computation of gradients in integration points of capacity matrix
	allocv (intordcm[ii][jj],gp);
	allocv (intordcm[ii][jj],w);
	gauss_points (gp.a,w.a,intordcm[ii][jj]);
	
	for (i=0;i<intordcm[ii][jj];i++){
	  xi=gp[i];
	  for (j=0;j<intordcm[ii][jj];j++){
	    eta=gp[j];
	    
	    //  matrix of gradients
	    grad_matrix (gm,x,y,xi,eta,jac);
	    mxv (gm,t,grad);
	    Tm->storegrad (k,ipp,0,grad);
	    ipp++;
	  }
	}
	
	destrv (gp);  destrv (w);
	
	if (Tp->savemode==1)  break;
      }
      if (Tp->savemode==1)  break;
    }
  }
  
}

/**
   function interpolates nodal values of array other to integration points
   
   @param eid - element id
   
   JK, 17.9.2005
*/
void quadlineart::intpointother (long eid)
{
  long i,j,k,ii,jj,ipp,ncompo;
  double xi,eta,val,*r;
  ivector nodes(nne);
  vector t(nne),gp,w;
  
  //  nodes of required element
  Tt->give_elemnodes (eid,nodes);
  
  //  number of components in array other
  ncompo=Tt->nodes[i].ncompother;
  r = new double [ncompo*nne];
  
  //  nodal values of array other
  nodalotherval (r,nodes);
  
  for (k=0;k<ncompo;k++){
    
    for (i=0;i<nne;i++){
      t[i]=r[i*ncompo+k];
    }
    
    for (ii=0;ii<Tp->ntm;ii++){
      for (jj=0;jj<Tp->ntm;jj++){
	
	//  interpolation to integration points of conductivity matrix
	allocv (intordkm[ii][jj],gp);
	allocv (intordkm[ii][jj],w);
	gauss_points (gp.a,w.a,intordkm[ii][jj]);
	
	ipp=Tt->elements[eid].ipp[ii][jj];
	for (i=0;i<intordkm[ii][jj];i++){
	  xi=gp[i];
	  for (j=0;j<intordkm[ii][jj];j++){
	    eta=gp[j];
	    
	    val = approx (xi,eta,t);
	    Tm->ip[ipp].other[k]=val;
	    ipp++;
	  }
	}
	
	destrv (gp);  destrv (w);
	
	//  interpolation to integration points of capacity matrix
	allocv (intordcm[ii][jj],gp);
	allocv (intordcm[ii][jj],w);
	gauss_points (gp.a,w.a,intordcm[ii][jj]);
	
	for (i=0;i<intordcm[ii][jj];i++){
	  xi=gp[i];
	  for (j=0;j<intordcm[ii][jj];j++){
	    eta=gp[j];
	    
	    val = approx (xi,eta,t);
	    Tm->ip[ipp].other[k]=val;
	    ipp++;
	  }
	}
	
	destrv (gp);  destrv (w);

	if (Tp->savemode==1)  break;
      }
      if (Tp->savemode==1)  break;
    }
  }
  
  delete [] r;
}

/**
   function assembles %matrix of base functions

   @param n - %matrix of base functions
   @param xi,eta - natural coordinates

   JK, 25.9.2001
*/
void quadlineart::bf_matrix (matrix &n,double xi,double eta)
{
  fillm (0.0,n);
  bf_lin_4_2d (n.a,xi,eta);
}

/**
   function returns one approximation function evaluated in required point

   @param f - value of approximation function
   @param xi,eta - natural coordinates
   @param i - number of approximation function

   JK, 1.2.2003
*/
/*
void quadlineart::give_approx_fun (double &f,double xi,double eta,long i)
{
  double n[4];
  bf_lin_4_2d (n,xi,eta);
  f=n[i];
}
*/

/**
   function assembles gradient of %matrix of base functions

   @param gm - gradient %matrix
   @param x,y - array containing node coordinates
   @param xi,eta - natural coordinates
   @param jac - Jacobian

   JK, 25.9.2001
*/
void quadlineart::grad_matrix (matrix &gm,vector &x,vector &y,double xi,double eta,double &jac)
{
  long i;
  vector dx(nne),dy(nne);

  dx_bf_lin_4_2d (dx.a,eta);
  dy_bf_lin_4_2d (dy.a,xi);

  derivatives_2d (dx,dy,jac,x,y,xi,eta);

  fillm (0.0,gm);

  for (i=0;i<nne;i++){
    gm[0][i]=dx[i];
    gm[1][i]=dy[i];
  }

}



/**
   function computes conductivity %matrix of 2D problems for one transported matter
   finite element with bilinear approximation functions

   @param lcid - number of load case
   @param eid - element id
   @param ri,ci - row and column indices of the computed block in the resulting %matrix
   @param km - conductivity %matrix

   JK, 25.9.2001
*/
void quadlineart::conductivity_matrix (long lcid,long eid,long ri,long ci,matrix &km)
{
  long i,j,ii;
  double thick,xi,eta,ww1,ww2,jac;
  ivector nodes(nne);
  vector x(nne),y(nne),w(intordkm[ri][ci]),gp(intordkm[ri][ci]),t(nne);
  matrix gm(ncomp,dofe[ri][ci]),d;

  matrix n(1,dofe[ri][ci]);
  
  //  nodes on required elements
  Tt->give_elemnodes (eid,nodes);
  //  thickness
  Tc->give_thickness (eid,nodes,t);
  //  node coordinates
  Tt->give_node_coord2d (x,y,eid);
  //  coordinates and weights of integration points
  gauss_points (gp.a,w.a,intordkm[ri][ci]);
  
  fillm (0.0,km);
  
  if (Tp->savemode==0)
    ii=Tt->elements[eid].ipp[ri][ci];
  if (Tp->savemode==1)
    ii=Tt->elements[eid].ipp[0][0];
  
  for (i=0;i<intordkm[ri][ci];i++){
    xi=gp[i];  ww1=w[i];
    for (j=0;j<intordkm[ri][ci];j++){
      eta=gp[j];  ww2=w[j];
      
      //  matrix of gradients
      grad_matrix (gm,x,y,xi,eta,jac);
      
      //  matrix of conductivity of the material
      allocm(ncomp,ncomp,d);
      Tm->matcond (d,ii,ri,ci);
      
      //  thickness
      thick = approx (xi,eta,t);
      
      jac=fabs(jac);
      
      jac*=thick*ww1*ww2;
      
      //  contribution to the conductivity matrix of the element
      bdbj (km.a,gm.a,d.a,jac,gm.m,gm.n);
      destrm(d);
      
      //convective terms
      if (Tp->tmatt == threemediacoup || Tp->tmatt == twomediacoup)
	{
	  allocm(1,ncomp,d);
	  
	  Tm->matcond2(d,ii,ri,ci);
	  bf_matrix (n, xi, eta);
	  bdbjac(km, n, d, gm, jac);	
	  destrm(d);
	}
      
      ii++;  
    }
  }
  
  //coupled b.c.//changed 17.7.2006
  if ((Tt->elements[eid].transi[lcid]==3) || (Tt->elements[eid].transi[lcid]==4)){
    transmission_matrix (lcid,eid,ri,ci,km);
  }
}



/**
   function computes capacity %matrix of 2D problems for one transported matter
   finite element with bilinear approximation functions

   @param eid - element id
   @param ri,ci - row and column indices of the computed block in the resulting %matrix
   @param cm - capacity %matrix

   JK, 4.10.2001
*/
void quadlineart::capacity_matrix (long eid,long ri,long ci,matrix &cm)
{
  long i,j,ii;
  double jac,xi,eta,w1,w2,thick,rho,c;
  ivector nodes(nne);
  vector x(nne),y(nne),w(intordcm[ri][ci]),gp(intordcm[ri][ci]),t(nne),dens(nne);
  matrix n(1,dofe[ri][ci]);

  Tt->give_elemnodes (eid,nodes);
  Tc->give_thickness (eid,nodes,t);
  Tc->give_density (eid,nodes,dens);

  Tt->give_node_coord2d (x,y,eid);
  gauss_points (gp.a,w.a,intordcm[ri][ci]);

  fillm (0.0,cm);
  
  if (Tp->savemode==0)
    ii=Tt->elements[eid].ipp[ri][ci]+intordkm[ri][ci]*intordkm[ri][ci];
  if (Tp->savemode==1)
    ii=Tt->elements[eid].ipp[0][0]+intordkm[0][0]*intordkm[0][0];
  
  for (i=0;i<intordcm[ri][ci];i++){
    xi=gp[i];  w1=w[i];
    for (j=0;j<intordcm[ri][ci];j++){
      eta=gp[j];  w2=w[j];
      jac_2d (jac,x,y,xi,eta);
      bf_matrix (n,xi,eta);

      c=Tm->capcoeff (ii,ri,ci);
      thick = approx (xi,eta,t);
      rho = approx (xi,eta,dens);
      jac*=w1*w2*thick*rho*c;

      jac=fabs(jac);

      nnj (cm.a,n.a,jac,n.m,n.n);
      ii++;
    }
  }

}


/**
   function computes source %vector of one matter on one element

   \int_{Omega} N^T N d Omega . s

   @param sv - source %vector of one matter
   @param lcid - load case id
   @param eid - element id
   @param ri,ci - row and column indices of the block (ri must be equal to ci)

   JK, 4.10.2001
*/
void quadlineart::quantity_source_vector (vector &sv,vector &nodval,long eid,long ri,long ci)
{
  long i,j;
  double jac,xi,eta,w1,w2,thick;
  ivector nodes(nne);
  vector x(nne),y(nne),w(intordcm[ri][ci]),gp(intordcm[ri][ci]),t(nne),dens(nne),v(dofe[ri][ci]);
  matrix n(1,dofe[ri][ci]),nm(dofe[ri][ci],dofe[ri][ci]);
  
  Tt->give_elemnodes (eid,nodes);
  
  Tc->give_thickness (eid,nodes,t);
  
  Tt->give_node_coord2d (x,y,eid);
  gauss_points (gp.a,w.a,intordcm[ri][ci]);
  
  fillm (0.0,nm);
  
  for (i=0;i<intordcm[ri][ci];i++){
    xi=gp[i];  w1=w[i];
    for (j=0;j<intordcm[ri][ci];j++){
      eta=gp[j];  w2=w[j];
      jac_2d (jac,x,y,xi,eta);
      bf_matrix (n,xi,eta);

      thick = approx (xi,eta,t);
      jac*=w1*w2*thick;
      
      nnj (nm.a,n.a,jac,n.m,n.n);
    }
  }
  mxv (nm,nodval,v);  addv (sv,v,sv);

}


/**
   function computes internal fluxes of 1D problems for one transported matter
   finite element with linear approximation functions
   
   @param lcid - number of load case
   @param eid - number of element
   @param ifl - %vector of internal fluxes
   
   JK, 31.3.2002
*/
void quadlineart::internal_fluxes (long lcid,long eid,vector &ifl)
{
  long i,j,ipp;
  double xi,eta,jac,thick;
  ivector nodes(nne);
  vector x(nne),y(nne),w,gp,t(nne),fl(ncomp),contr(dofe[lcid][lcid]);
  matrix gm(ncomp,dofe[lcid][lcid]);
  

  Tt->give_elemnodes (eid,nodes);
  Tc->give_thickness (eid,nodes,t);
  Tt->give_node_coord2d (x,y,eid);
  
  allocv (intordkm[lcid][lcid],gp);
  allocv (intordkm[lcid][lcid],w);
  
  gauss_points (gp.a,w.a,intordkm[lcid][lcid]);
  
  if (Tp->savemode==0)
    ipp=Tt->elements[eid].ipp[lcid][lcid];
  if (Tp->savemode==1)
    ipp=Tt->elements[eid].ipp[0][0];
  
  
  for (i=0;i<intordkm[lcid][lcid];i++){
    xi=gp[i];
    for (j=0;j<intordkm[lcid][lcid];j++){
      eta=gp[j];
      thick = approx (xi,eta,t);
      
      Tm->computenlfluxes (lcid,ipp);
      
      Tm->givefluxes (lcid,ipp,0,fl);

      grad_matrix (gm,x,y,xi,eta,jac);

      mtxv (gm,fl,contr);

      cmulv (jac*w[i]*w[j]*thick,contr);
      
      addv (contr,ifl,ifl);
      
      ipp++;
    }
  }
  
  destrv (w);  destrv (gp);
}


/**
   function assembles resulting element conductivity %matrix

   @param eid - element id
   @param lcid - load case id
   @param km - resulting conductivity %matrix of one element

   JK, 6.1.2002
*/
void quadlineart::res_conductivity_matrix (long eid,long lcid,matrix &km)
{
  long i,j,*rcn,*ccn;
  matrix lkm;

  for (i=0;i<ntm;i++){
    for (j=0;j<ntm;j++){
      rcn = new long [dofe[i][j]];
      ccn = new long [dofe[i][j]];
      allocm (dofe[i][j],dofe[i][j],lkm);
      conductivity_matrix (i,eid,i,j,lkm);
      codnum (rcn,i);
      codnum (ccn,j);
      mat_localize (km,lkm,rcn,ccn);
      destrm (lkm);
      delete [] rcn;  delete [] ccn;
    }
  }
}



/**
   function assembles resulting element capacity %matrix
   
   @param eid - element id
   @param cm - resulting capacity %matrix of one element

   JK, 6.1.2002
*/
void quadlineart::res_capacity_matrix (long eid,matrix &cm)
{
  long i,j,*rcn,*ccn,k,l;
  double s;
  matrix lcm;

  for (i=0;i<ntm;i++){
    for (j=0;j<ntm;j++){
      rcn = new long [dofe[i][j]];
      ccn = new long [dofe[i][j]];
      allocm (dofe[i][j],dofe[i][j],lcm);
      //  capacity matrix for one matter
      capacity_matrix (eid,i,j,lcm);
      
      // diagonalization of capacity matrix on one element
      if (Tp->diagcap == 1){
	for (k=0;k<dofe[i][j];k++){
	  s=0.0;
	  for (l=0;l<dofe[i][j];l++){
	    s+=lcm[k][l];
	    lcm[k][l]=0.0;
	  }
	  lcm[k][k]=s;
	}
      }
      
      codnum (rcn,i);
      codnum (ccn,j);
      //  localization to element capacity matrix
      mat_localize (cm,lcm,rcn,ccn);
      destrm (lcm);
      delete [] rcn;  delete [] ccn;
    }
  }
}

/**
   function assembles resulting element convection %vector

   @param f - resulting convection %vector of one element
   @param lcid - load case id
   @param eid - element id
   @param leid - id of element with boundary condition
   
   JK, 6.1.2002
*/
void quadlineart::res_convection_vector (vector &f,long lcid,long eid,long leid)
{
  long *cn;
  vector lf;
  
  //  array for code numbers
  cn = new long [dofe[lcid][lcid]];
  //  array for nodal values of one matter
  allocv (dofe[lcid][lcid],lf);
  
  convection_vector (lf,lcid,eid,leid,lcid,lcid);
  
  //  code numbers
  codnum (cn,lcid);
  //  localization to element vector
  locglob (f.a,lf.a,cn,dofe[lcid][lcid]);
  
  destrv (lf);
  delete [] cn;
  
  cmulv (-1.0,f,f);

}

/**
   function assembles resulting element transmission %vector

   @param f - resulting transmission %vector of one element
   @param lcid - load case id
   @param eid - element id
   @param leid - id of element with boundary condition
   
   JK, 6.1.2002
*/
void quadlineart::res_transmission_vector (vector &f,long lcid,long eid,long leid)
{
  long *cn,i;
  vector lf;
  
  cn = new long [dofe[lcid][lcid]];
  codnum (cn,lcid);
  
  //coupled b.c.
  for (i=0;i<ntm;i++){    
    allocv (dofe[lcid][lcid],lf);
    
    if ((Tt->elements[eid].transi[i]==3) || (Tt->elements[eid].transi[i]==4)){
      transmission_vector (lf,i,eid,leid,lcid,i);
    }
    
    locglob (f.a,lf.a,cn,dofe[lcid][lcid]);
    destrv (lf);
    
  }
  delete [] cn;
}

/**
   function assembles resulting element source %vector

   @param sv - resulting source %vector of one element
   @param nodval - array of nodal values
   @param lcid - load case id
   @param eid - element id
   
   JK, 6.1.2002
*/
void quadlineart::res_quantity_source_vector (vector &sv,vector &nodval,long lcid,long eid)
{
  long *cn;
  vector lsv;

  cn = new long [dofe[lcid][lcid]];
  allocv (dofe[lcid][lcid],lsv);
  quantity_source_vector (lsv,nodval,eid,lcid,lcid);
  codnum (cn,lcid);
  locglob (sv.a,lsv.a,cn,dofe[lcid][lcid]);
  destrv (lsv);
  delete [] cn;
}

/**
   function assembles resulting element internal fluxes %vector

   @param eid - element id
   @param elemif - resulting internal fluxes %vector of one element
   
   JK, 6.1.2002
*/
void quadlineart::res_internal_fluxes (long eid,vector &elemif)
{
  long i,*cn;
  vector lif,tdnv;
  matrix cm;

  for (i=0;i<ntm;i++){
    cn = new long [dofe[i][i]];
    allocv (dofe[i][i],lif);
    internal_fluxes (i,eid,lif);
    codnum (cn,i);
    locglob (elemif.a,lif.a,cn,dofe[i][i]);

    destrv (lif);
    delete [] cn;
  }

  cn = new long [ndofe];
  Gtt->give_code_numbers (eid,cn);
  allocv (ndofe,tdnv);
  allocv (ndofe,lif);
  allocm (ndofe,ndofe,cm);
  nodalderivatives (tdnv.a,cn,ndofe);
  res_capacity_matrix (eid,cm);
  mxv (cm,tdnv,lif);

  subv (elemif,lif,elemif);
  
  delete [] cn;
  destrm (cm);
  destrv (lif);  destrv (tdnv);
}

/**
   function computes element quantity integral
   
   @param eid - element id
   @param nodval - %vector of quantity nodal values

   @retval f - element quantity integral

   TKr, 30.1.2004
*/
/*
double quadlineart::total_integral(long eid,vector &nodval)
{
  long i,j;
  double thick,xi,eta,ww1,ww2,jac,value,f;
  ivector nodes(nne);
  vector x(nne),y(nne),w(2),gp(2),t(nne);

  Tt->give_elemnodes (eid,nodes);
  Tc->give_thickness (eid,nodes,t);

  Tt->give_node_coord2d (x,y,eid);
  gauss_points (gp.a,w.a,2);

  f = 0.0;

  for (i=0;i<2;i++){
    xi=gp[i];  ww1=w[i];
    for (j=0;j<2;j++){
      eta=gp[j];  ww2=w[j];

      jac_2d (jac,x,y,xi,eta);
      thick = approx (xi,eta,t);
      value = approx (xi,eta,nodval);

      jac*=thick*ww1*ww2*value;
      f = f + jac;
    }
  }
  return(f);
}
*/


/**
   function assembles resulting element boundary flux %vector

   @param f - resulting boundary flux %vector of one element
   @param lcid - load case id
   @param eid - element id
   @param leid - element id
   
   TKr, 28.2.2004
*/
void quadlineart::res_boundary_flux (vector &f,long lcid,long eid,long leid)
{
  long *cn,i;
  vector lf;

  cn = new long [dofe[lcid][lcid]];
  codnum (cn,lcid);
  
  //coupled b.c.
  for (i=0;i<ntm;i++){    
    allocv (dofe[lcid][lcid],lf);
    
    if ((Tt->elements[eid].transi[i]==3) || (Tt->elements[eid].transi[i]==4)){
      boundary_flux (lf,i,eid,leid,lcid,i);
    }
    
    locglob (f.a,lf.a,cn,dofe[lcid][lcid]);
    destrv (lf);
    
  }
  
  delete [] cn;
}


/**
   function computes other values in nodes of element

   @param lcid - load case id
   @param eid - element id
   
   10.5.2002
*/
void quadlineart::nod_others (long lcid,long eid,long ri,long ci)
{
  long i,j,k,ipp,ncompother,ii;
  vector other,h,r(ndofe);
  ivector nod(nne),cn(ndofe);
  
  Tt->give_elemnodes (eid,nod);  
  Tt->give_code_numbers (eid,cn.a);
  elemvalues (lcid,eid,r.a,cn.a,ndofe);
  ncompother = Tm->givencompother();
  allocv (ncompother,other);
  allocv (ntm,h);

  ii=0;
  
  if (Tp->savemode==0)
    ipp=Tt->elements[eid].ipp[ri][ci];
  if (Tp->savemode==1)
    ipp=Tt->elements[eid].ipp[0][0];
  
  
  for (i=0;i<nne;i++){
    
    for (j=0;j<ntm;j++){
      h[j] = r[ii];
      ii++;
    }
    
    for(k=0;k<ncompother;k++)
      other[k] = Tm->givecompother(k,ipp,h.a);
    
    Tt->nodes[nod[i]].storeother(ncompother,other);
  }

  destrv (other);  destrv (cn);  destrv (nod);  destrv (r);
  destrv (h);
}

/**
   function computes gradients in nodes
   
   eid - element id

   TKr, 25.9.2001   
*/
void quadlineart::nod_grads (long lcid,long eid,long ri,long ci)//not completed
{
  long i,j,ii,jj,k,ipp;
  double xi,eta,jac;
  ivector nod(nne),cn(ndofe);
  vector x(nne),y(nne),r(ndofe),t(nne),grad(ncomp);
  matrix gm(ncomp,nne);
  
  Tt->give_node_coord2d (x,y,eid);
  Tt->give_elemnodes (eid,nod);
  Tt->give_code_numbers (eid,cn.a);
  elemvalues (0,eid,r.a,cn.a,ndofe);
  
  ii=0;
  jj=0;
  
  if (Tp->savemode==0)
    ipp=Tt->elements[eid].ipp[ri][ci];
  if (Tp->savemode==1)
    ipp=Tt->elements[eid].ipp[0][0];


  for (i=0;i<nne;i++){
    for (k=0;k<Tp->ntm;k++){
      
      for (j=0;j<dofe[k][k];j++){
	t[j]=r[ordering[k][j]-1];
      }
      //  matrix of gradients
      if(i == 0) xi = 1.0; eta = 1.0;
      if(i == 1) xi = -1.0; eta = 1.0;
      if(i == 2) xi = -1.0; eta = -1.0;
      if(i == 3) xi = 1.0; eta = -1.0;      
      grad_matrix (gm,x,y,xi,eta,jac);

      mxv (gm,t,grad);
      
      Tt->nodes[nod[i]].storegrad(jj,ntm,grad);
    }
    jj++;
  }
}











/**
   function computes nodal fluxes caused by prescribed fluxes
   (Neumann boundary condition)
   
   @param v - array of nodal fluxes
   @param lcid - load case id
   @param eid - element id
   @param leid - id of loaded element
   @param ri,ci - row and column indices
   
   JK, 19.8.2004
*/
void quadlineart::convection_vector (vector &v,long lcid,long eid,long leid,long ri,long ci)
{
  long i,ipp,*bc;
  ivector nodes(nne);
  vector x(nne),y(nne),w(intordkm[ri][ci]),gp(intordkm[ri][ci]),t(nne);
  vector list(nned*ned),trc(nned*ned),nodval(nne),av(nne),coef(nne);
  matrix km(nne,nne);
  
  fillv (0.0,v);
  
  //  element nodes
  Tt->give_elemnodes (eid,nodes);
  //  thicknesses at nodes
  Tc->give_thickness (eid,nodes,t);
  //  node coordinates on element
  Tt->give_node_coord2d (x,y,eid);
  
  //  coordinates and weights of integration points
  gauss_points (gp.a,w.a,intordkm[ri][ci]);
  
  //  indicators of boundary conditions
  bc = new long [ned];
  Tb->lc[lcid].elemload[leid].give_bc (bc);
  //  nodal values of exterior variable
  Tb->lc[lcid].elemload[leid].give_nodval (lcid,list);
  
  //  auxiliary coefficients, necessary for function edge_integral
  for (i=0;i<nned*ned;i++){
    trc[i]=1.0;
  }
  
  //  integration point number
  if (Tp->savemode==0)
    ipp=Tt->elements[eid].ipp[ri][ci];
  if (Tp->savemode==1)
    ipp=Tt->elements[eid].ipp[0][0];
  
  //  loop over edges
  for (i=0;i<ned;i++){
    
    if (bc[i]==2 || bc[i]==3 || bc[i]==4){
      //  nodal values on actual edge
      edgenodeval (i,nodval,list);
      //  nodal values of auxiliary coefficients
      edgenodeval (i,coef,trc);
      
      fillm (0.0,km);
      
      //  matrix obtained from integration over edge
      edge_integral (i,x,y,intordkm[ri][ci],gp,w,t,coef,km);
      
      mxv (km,nodval,av);
      
      addv (v,av,v);
    }
  }
  
  delete [] bc;
}

/**
   function computes transmission complement to the conductivity %matrix for one matter
   
   \int_{Gamma_3} N^T c_{tr} N dGamma
   
   @param lcid - number of load case
   @param eid - element id
   @param ri,ci - row and column indices
   @param km - part of the conductivity %matrix
   
   JK, 19.8.2004
*/
void quadlineart::transmission_matrix (long lcid,long eid,long ri,long ci,matrix &km)
{
  long i,j,ipp,leid,*bc;
  ivector nodes(nne);
  vector x(nne),y(nne),w(intordkm[ri][ci]),gp(intordkm[ri][ci]);
  vector trc(nned*ned),coeff(nne),t(nne);
  
  //  element nodes
  Tt->give_elemnodes (eid,nodes);
  //  thicknesses at nodes
  Tc->give_thickness (eid,nodes,t);
  //  node coordinates on element
  Tt->give_node_coord2d (x,y,eid);
  
  //  coordinates and weights of integration points
  gauss_points (gp.a,w.a,intordkm[ri][ci]);

  //  correspondence between element ordering and boundary element ordering
  for (i=0;i<Tb->lc[lcid].neb;i++){
    if (Tb->lc[lcid].elemload[i].eid==eid){
      leid=i;

      //break;
      //}
      //}
      
      //  indicators of boundary conditions
      bc = new long [ned];
      Tb->lc[lcid].elemload[leid].give_bc (bc);
      //  transmission coefficients
      Tb->lc[lcid].elemload[leid].give_trc (lcid,trc);
      
      //  integration point number
      if (Tp->savemode==0)
	ipp=Tt->elements[eid].ipp[ri][ci];
      if (Tp->savemode==1)
	ipp=Tt->elements[eid].ipp[0][0];
      
      //  loop over edges
      for (j=0;j<ned;j++){
	
	if (bc[j]==4 || bc[j]>10){
	  //  transformation of coefficients
	  transf_coeff (j,coeff,trc,eid,ri,ci,ipp,bc);
	  //  matrix obtained from integration over edges
	  edge_integral (j,x,y,intordkm[ri][ci],gp,w,t,coeff,km);
	}
      }

    }
  }
  
  delete [] bc;
}

/**
   function computes nodal values on element caused by
   prescribed transmission (Newton boundary condition)
   
   function computes \int \kappa T_{ext}
   
   @param v - %vector of nodal values
   @param lcid - load case id
   @param eid - element id
   @param leid - id of loaded element
   @param ri,ci - row and column indices
   
*/
void quadlineart::transmission_vector (vector &v,long lcid,long eid,long leid,long ri,long ci)
{
  long i,ipp,*bc;
  ivector nodes(nne);
  vector x(nne),y(nne),z(nne),w(intordkm[ri][ci]),gp(intordkm[ri][ci]),t(nne);
  vector list(nned*ned),trc(nned*ned),trr(nned*ned),nodval(nne),av(nne),coef(nne);
  matrix km(nne,nne);
  
  fillv (0.0,v);
  
  //  element nodes
  Tt->give_elemnodes (eid,nodes);
  //  thicknesses at nodes
  Tc->give_thickness (eid,nodes,t);
  //  node coordinates on element
  Tt->give_node_coord2d (x,y,eid);
  
  //  coordinates and weights of integration points
  gauss_points (gp.a,w.a,intordkm[ri][ci]);
  
  //  indicators of boundary conditions
  bc = new long [ned];
  Tb->lc[lcid].elemload[leid].give_bc (bc);
  //  nodal values of exterior variable
  Tb->lc[lcid].elemload[leid].give_nodval (lcid,list);
  //  transmission coefficients
  Tb->lc[lcid].elemload[leid].give_trc (lcid,trc);
  //  transmission/radiation coefficients
  Tb->lc[lcid].elemload[leid].give_trr (lcid,trr);
  
  //  integration point number
  if (Tp->savemode==0)
    ipp=Tt->elements[eid].ipp[ri][ci];
  if (Tp->savemode==1)
    ipp=Tt->elements[eid].ipp[0][0];
  
  for (i=0;i<ned;i++){
    
    if (bc[i]==4 || bc[i]>10){
      //  transformation of prescribed nodal values
      transf_val (i,nodval,list,trc,trr,eid,ri,ci,ipp,bc);
      //  determination of transmission coefficients on edge
      transf_coeff (i,coef,trc,eid,ri,ci,ipp,bc);
      
      fillm (0.0,km);
      //  matrix obtained from integration over edges
      edge_integral (i,x,y,intordkm[ri][ci],gp,w,t,coef,km);
      //  matrix-vector multiplication
      mxv (km,nodval,av);
      //  contribution added to element vector
      addv (v,av,v);
    }
  }
  
  delete [] bc;
}


/**
   function computes contributions to the boundary flux from transmission vector
   
   \int_{Gamma_3} N^T c_{tr} N dGamma * nodal_external_value

   @param tmv - %vector of boundary fluxes
   @param lcid - load case id
   @param eid - element id
   @param leid - element id
   @param ri,ci - row and column indices of the computed block in the resulting matrix
   
   TKr, 28.2.2004
*/
void quadlineart::boundary_flux (vector &v,long lcid,long eid,long leid,long ri,long ci)
{
  long i,ipp,*bc;
  ivector nodes(nne);
  vector x(nne),y(nne),w(intordkm[ri][ci]),gp(intordkm[ri][ci]),t(nne);
  vector list(nned*ned),trc(nned*ned),trr(nned*ned),nodval(nne),av(nne),coef(nne);
  matrix km(nne,nne);
  
  fillv (0.0,v);
  
  //  element nodes
  Tt->give_elemnodes (eid,nodes);
  //  thicknesses at nodes
  Tc->give_thickness (eid,nodes,t);
  //  node coordinates on element
  Tt->give_node_coord2d (x,y,eid);
  
  //  coordinates and weights of integration points
  gauss_points (gp.a,w.a,intordkm[ri][ci]);
  
  //  indicators of boundary conditions
  bc = new long [ned];
  Tb->lc[lcid].elemload[leid].give_bc (bc);
  //  nodal values of exterior variable
  Tb->lc[lcid].elemload[leid].give_nodval (lcid,list);
  //  transmission coefficients
  Tb->lc[lcid].elemload[leid].give_trc (lcid,trc);
  //  transmission/radiation coefficients
  Tb->lc[lcid].elemload[leid].give_trr (lcid,trr);
  
  //  integration point number
  if (Tp->savemode==0)
    ipp=Tt->elements[eid].ipp[ri][ci];
  if (Tp->savemode==1)
    ipp=Tt->elements[eid].ipp[0][0];
  
  for (i=0;i<ned;i++){
    
    if (bc[i]==4 || bc[i]>10){
      //  transformation of nodal values
      transf_flux (i,nodval,list,trc,trr,eid,ri,ci,ipp,bc);
      
      edgenodeval (i,coef,trc);
  
      fillm (0.0,km);
      //
      edge_integral (i,x,y,intordkm[ri][ci],gp,w,t,coef,km);
      
      mxv (km,nodval,av);
      
      addv (v,av,v);
    }
  }
  
  delete [] bc;
}

/**
   function integrates N^T c N over edges
   
   @param edg - edge id (number of edge)
   @param x, y - coordinates of element nodes
   @param intord - order of numerical integration
   @param gp, w  - coordinates and weights of integration points
   @param t - nodal thicknesses
   @param coef - array of nodal values of coefficient
   @param km - output %matrix
   
   JK
*/
void quadlineart::edge_integral (long edg,vector &x,vector &y,long intord,vector &gp,vector &w,
				 vector &t,vector &coef,matrix &km)
{
  long i;
  double xi,eta,jac,ipval,thick;
  matrix n(1,nne);
  
  if (edg==0){
    eta=1.0;
    for (i=0;i<intord;i++){
      xi=gp[i];
      
      bf_matrix (n,xi,eta);
      jac1d_2d (jac,x,y,xi,edg);
      
      ipval=approx (xi,eta,coef);
      thick=approx (xi,eta,t);
      
      jac*=w[i]*ipval*thick;
      nnj (km.a,n.a,jac,n.m,n.n);
    }
  }

  if (edg==1){
    xi=-1.0;
    for (i=0;i<intord;i++){
      eta=gp[i];
      
      bf_matrix (n,xi,eta);
      jac1d_2d (jac,x,y,eta,edg);
      
      ipval=approx (xi,eta,coef);
      thick=approx (xi,eta,t);
      
      jac*=w[i]*ipval*thick;
      nnj (km.a,n.a,jac,n.m,n.n);
    }
  }

  if (edg==2){
    eta=-1.0;
    for (i=0;i<intord;i++){
      xi=gp[i];
      
      bf_matrix (n,xi,eta);
      jac1d_2d (jac,x,y,xi,edg);
      
      ipval=approx (xi,eta,coef);
      thick=approx (xi,eta,t);
      
      jac*=w[i]*ipval*thick;
      nnj (km.a,n.a,jac,n.m,n.n);
    }
  }

  if (edg==3){
    xi=1.0;
    for (i=0;i<intord;i++){
      eta=gp[i];
      
      bf_matrix (n,xi,eta);
      jac1d_2d (jac,x,y,eta,edg);
      
      ipval=approx (xi,eta,coef);
      thick=approx (xi,eta,t);
      
      jac*=w[i]*ipval*thick;
      nnj (km.a,n.a,jac,n.m,n.n);
    }
  }

}

/**
   function
   
   @param edg - edge id
   @param coeff - array of 
   @param list - array of nodal values prescribed on boundary
   @param trc - array of transmission coefficients
   @param trr - array of transmission/radiation coefficients
   @param eid - element id
   @param ri,ci - row and column indices
   @param ipp - integration point id
   @param bc - array describing type of boundary condition
*/
void quadlineart::transf_flux (long edg,vector &coeff,vector &list,vector &trc,vector &trr,long eid,long ri,long ci,long ipp,long *bc)
{
  long i,j,k;
  double tr,new_flux;
  ivector nodes(nne),edgenod(nned);

  //  zeroing of array of coefficients
  fillv (0.0,coeff);
  
  //  global numbers of nodes of element
  Tt->give_elemnodes (eid,nodes);

  //  local numbers of nodes on required edge
  linquadrilat_edgnod (edgenod.a,edg);

  //  actual position in the array list
  j=edg*nned;

  for (i=0;i<nned;i++){
    if (bc[edg]==90){
      tr = trr[j+i]/trc[j+i];
    }

    //  node number
    k=edgenod[i];
    
    Tm->transmission_flux(new_flux,list[j+i],tr,ri,ci,nodes[k],bc[edg],ipp);
    
    coeff[k]=new_flux;
  }
  
}


/**
   function
   
   @param edg - edge id
   @param coeff - array of 
   @param list - array of nodal values prescribed on boundary
   @param eid - element id
   @param ri,ci - row and column indices
   @param ipp - integration point id
   @param bc - array describing type of boundary condition
*/
void quadlineart::transf_coeff (long edg,vector &coeff,vector &list,long eid,long ri,long ci,long ipp,long *bc)
{
  long i,j,k;
  double new_coeff;
  ivector nodes(nne),edgenod(nned);

  //  zeroing of array of coefficients
  fillv (0.0,coeff);
  
  //  global numbers of nodes of element
  Tt->give_elemnodes (eid,nodes);

  //  local numbers of nodes on required edge
  linquadrilat_edgnod (edgenod.a,edg);
  
  //  actual position in the array list
  j=edg*nned;

  for (i=0;i<nned;i++){
    //  node number
    k=edgenod[i];
    
    Tm->transmission_transcoeff(new_coeff,list[j+i],ri,ci,nodes[k],bc[edg],ipp);
    
    coeff[k]=new_coeff;
  }
  
}


/**
   function prepares nodal values of transported quantity, transmission
   coefficents and transmission/radiation coefficients
   
   @param edg - number of required edge (edge id)
   @param nodval - array of transformed nodal values
   @param list - array of nodal values defined on all edges
   @param trc - array of transmission coefficients
   @param trr - array of transmission/radiation coefficients
   @param ri,ci - row and column indices
   @param ipp - integration point number
   @param bc - array defining boundary conditions
   
   JK, 19.8.2004
*/
void quadlineart::transf_val (long edg,vector &nodval,vector &list,vector &trc,vector &trr,long eid,long ri,long ci,long ipp,long *bc)
{
  long i,j,k;
  double tr,new_nodval;
  ivector nodes(nne),edgenod(nned);
  
  //  zeroing of array of nodal values
  fillv (0.0,nodval);
  
  //  global numbers of nodes of element
  Tt->give_elemnodes (eid,nodes);

  //  local numbers of nodes on required edge
  linquadrilat_edgnod (edgenod.a,edg);
  
  //  actual position in the array list
  j=edg*nned;
  
  //  loop over number of nodes on one edge
  for (i=0;i<nned;i++){
    if (bc[edg]==90){
      tr = trr[j+i]/trc[j+i];
    }
    
    //  node number
    k=edgenod[i];
    
    Tm->transmission_nodval(new_nodval,list[j+i],tr,ri,ci,nodes[k],bc[edg],ipp);
    
    //  transformed nodal values
    nodval[k]=new_nodval;
  }
  
}

/**
   function picks up nodal values on required edges
   
   @param edg - number of required edge
   @param nodval - array of nodal values
   @param list - array of nodal values defined on all edges
   
   JK, 19.8.2004
*/
void quadlineart::edgenodeval (long edg,vector &nodval,vector &list)
{
  long i,j;
  ivector edgenod(nned);
  
  fillv (0.0,nodval);
  linquadrilat_edgnod (edgenod.a,edg);
  
  for (i=0;i<nned;i++){
    j=edgenod[i];
    nodval[j]=list[edg*nned+i];
  }
}


