#include "transtop.h"
#include "globalt.h"
#include "globmatt.h"
#include <string.h>
#include <limits.h>

transtop::transtop ()
{
  //  number of nodes
  nn=0;
  //  number of elements
  ne=0;
  
  //  array of nodes
  nodes=NULL;
  //  array of elements
  elements=NULL;
  //  array of end nodes
  enodes=NULL;
  //  array of edges
  edges=NULL;
  
  //nnj = 0;
  
  nedis=0;
  lnnd=NULL;  lnd=NULL;

  //  number of cycles on nodes
  ncycl = NULL;
  
  //JM
  //jnmap = NULL;
  //prevval = NULL;

}

transtop::~transtop ()
{
  long i;

  delete [] nodes;  
  delete [] elements;
  delete [] enodes;
  delete [] edges;
  delete [] lnnd;

  for (i=0;i<nedis;i++){
    delete [] lnd[i];
  }
  /*
  for (i = 0;i<nnj;i++)
  {
    //if (jnmap != NULL)
    //delete [] jnmap[i];
    if (prevval != NULL)
      delete [] prevval[i];
  }
  delete [] prevval;
  //delete [] jnmap;
  */
   
  delete [] lnd;
  
  delete [] ncycl;
}

/**
   function reads basic data about topology
   
   @param in - input stream
*/
void transtop::read (XFILE *in)
{
  long i,j,k,ndofn,nne,dim,order;
  
  // ************
  //  node data
  // ************
  xfscanf (in,"%k%ld","number_of_nodes",&nn);
  Gtt->alloc_nodes (nn);
  
  if (Mesprt==1)  fprintf (stdout,"\n number of nodes  %ld",nn);
  nodes = new nodet [nn];
  for (i=0;i<nn;i++){
    j=nn+1;
    xfscanf (in,"%ld",&j);
    if (j<1)
      print_err("node number in function read is less than 1",__FILE__,__LINE__,__func__);
    if (j>nn)
      print_err("node number in function read is greater than number of nodes",__FILE__,__LINE__,__func__);
    Gtt->gnodes[j-1].read (in);
    ndofn = Gtt->gnodes[j-1].ndofn;
    nodes[j-1].read (in,ndofn);
  }
  
  // ********************
  //  constrained nodes
  // ********************
  xfscanf (in,"%k%ld","number_of_constraints",&ncn);
  if (Mesprt==1)  fprintf (stdout,"\n number of constrained nodes  %ld",ncn);
  for (i=0;i<ncn;i++){
    k=nn+1;
    xfscanf (in,"%ld",&k);
    if (k<1)
      print_err("number of constrained node in function read is less than 1",__FILE__,__LINE__,__func__);
    if (k>nn){
      print_err("number of constrained node in function read is greater than number of all nodes",__FILE__,__LINE__,__func__);
    }
    Gtt->gnodes[k-1].constr (Gtt->dofcontr,in);
  }
  
  // ***************
  //  element data
  // ***************
  xfscanf (in,"%k%ld","number_of_elements",&ne);
  Gtt->alloc_elements (ne);
  if (Mesprt==1)  fprintf (stdout,"\n number of elements  %ld",ne);
  elements = new elementt [ne];
  
  
  /*
  if (Tp->tprob == discont_nonstat_problem || Tp->tprob == discont_nonlin_nonstat_problem){
    xfscanf (in,"%ld",&nedis);
    lnd = new long* [nedis];
    lnnd = new long [nedis];
    for (i=0;i<nedis;i++){
      xfscanf (in,"%ld %ld",&j,&lnnd[i]);
      elements[j-1].discont=i;
      lnd[i] = new long [lnnd[i]];
      for (j=0;j<lnnd[i];j++){
	xfscanf (in,"%ld",&lnd[i][j]);
	lnd[i][j]--;
      }
    }
  }
  */



  for (i=0;i<ne;i++){
    j=ne+1;
    xfscanf (in,"%ld",&j);
    if (j<1)
      print_err("element number in function read is less than 1",__FILE__,__LINE__,__func__);
    if (j>ne){
      print_err("element number in function read is greater than total number of elements",__FILE__,__LINE__,__func__);
    }
    elements[j-1].read (in,j-1);
  }
  
  //  initialization of auxinf variable
  for (i=0;i<ne;i++){
    nne = give_nne (i);
    order = give_degree (i);
    dim = give_dimension (i);
    
    Gtt->gelements[i].auxinf = nne*100+order*10+dim;
  }
  
  //  allocation and initiation of arrays lnso and leso
  //  lnso - list of nodes switched on
  //  leso - list of elements switched on
  Gtt->lneso_init ();
  
  
  if (Tp->tprob == growing_np_problem){
    Gtt->auxinf_init ();
  }
  if (Tp->tprob == growing_np_problem_nonlin){
    Gtt->auxinf_init ();
  }
  
  
  if (Gtt->rst==1){
    //  sequential topology is read
    //  it is used in the case of sequential version of
    //  any domain decomposition method
    Gtt->read_seq_top (in);
  }
  if (Gtt->rst==2){
    //  sequential topology is read in the form
    //  of Boolean matrices
    Tp->ssle->feti.read_booldata (in);
  }

  
  /*
  //  JK, 15.11.2007
  //  reading additional informations about mesh due to domain decomposition method
  //  problem will be solved by sequential FETI method
  if (Tp->ssle->tlinsol == sfeti){
    //Gtt->read_seq_top (Tp->ssle->feti.ns,in);
    Gtt->read_seq_top (in);
  }
  //  JK, 15.11.2007
  
  //  old version of discontinuity
  if (Tp->tprob == discont_nonstat_problem){
    //Gtt->read_ltg1 (in);
    Gtt->cngen=2;
    xfscanf (in,"%ld",&nnj);

    jnmap = new long* [nnj];
    for (i=0;i<nnj;i++){
      jnmap[i] = new long [2];
      xfscanf (in,"%ld %ld",&jnmap[i][0],&jnmap[i][1]);
      jnmap[i][0]--;
      jnmap[i][1]--;
    }
    
    prevval = new double* [nnj];
    for (i=0;i<nnj;i++){
      ndofn = give_ndofn (jnmap[i][0]);
      prevval[i] = new double [ndofn+2];
      for (j=0;j<ndofn+2;j++){
	prevval[i][j]=0.0;
      }
    }
  }
  */
  
  
  //  new version of discontinuity
  //  13.12.2007, JK
  /*
  if (Tp->tprob == discont_nonstat_problem){
    xfscanf (in,"%ld",&ns);
    Gtt->read_feti (ns,in);
    Gtt->cngen=3;
  }
  */
  //  13.12.2007, JK

  /*
  if (Tp->ssle->tlinsol == saddle_point){
    //Gtt->read_seq_top (Tp->ssle->sp->ns,in);
    Gtt->read_seq_top (in);
  }
  */
}


/**
   function returns element type
   
   @param eid - element id
*/
elemtypet transtop::give_elem_type (long eid)
{
  return (elements[eid].te);
}

/**
   function returns appropriate DOF of node
   
   @param nid - node id
   @param n - number of required DOF
*/
long transtop::give_dof (long nid,long n)
{
  return Gtt->give_dof (nid,n);
}

/**
   function returns numbers of nodes defining apropriate element

   eid - element id
   nodes - array containing nodes

   22.7.2001
*/
void transtop::give_elemnodes (long eid,ivector &nodes)
{
  Gtt->give_nodes (eid,nodes);
}

/**
   function assembles code numbers of actual element
   
   @param eid - element id
   @param cn - array containing code numbers on element
   
   25.6.2001
*/
void transtop::give_code_numbers (long eid,long *cn)
{
  Gtt->give_code_numbers (eid,cn);
}

/**
   function extracts code numbers of actual node
   
   @param nid - node id
   @param cn - array containing code numbers on node
   
   7.12.2002
*/
void transtop::give_node_code_numbers (long nid,long *cn)
{
  Gtt->give_node_code_numbers (nid,cn);
}

/**
   function returns node coordinates of the appropriate element

   19.7.2001
*/
void transtop::give_node_coord1d (vector &x,long eid)
{
  Gtt->give_node_coord1d (x,eid);
}

/**
   function returns node coordinates of the appropriate element
   
   @param x,y - vectors containing node coordinates
   @param eid - element id

   19.7.2001
*/
void transtop::give_node_coord2d (vector &x,vector &y,long eid)
{
  Gtt->give_node_coord2d (x,y,eid);
}

/**
   function returns node coordinates of the appropriate element
   
   @param x,y,z - vectors containing coordinates
   @param eid - element id
   
   19.7.2001
*/
void transtop::give_node_coord3d (vector &x,vector &y,vector &z,long eid)
{
  Gtt->give_node_coord3d (x,y,z,eid);
}

/**
   function returns number of DOFs of element
   
   @param eid - element id
   
   JK, 22.2.2002
*/
long transtop::give_ndofe (long eid)
{
  long ndofe;
  elemtypet te;
  
  te = give_elem_type (eid);

  switch (te){
  case barlint:{         ndofe=Lbt->ndofe;     break; }
  case barlintax:{       ndofe=Lbat->ndofe;    break; }
  case barquadt:{        ndofe=Qbt->ndofe;     break; }
  case barquadtax:{      ndofe=Qbat->ndofe;    break; }
  case trlint:{          ndofe=Ltt->ndofe;     break; }
  case trlaxisym:{       ndofe=Ltat->ndofe;    break; }
  case quadlint:{        ndofe=Lqt->ndofe;     break; }
  case quadquadt:{       ndofe=Qqt->ndofe;     break; }
  case quadquadtax:{     ndofe=Qqat->ndofe;    break; }
  case quadlaxisym:{     ndofe=Lqat->ndofe;    break; }
  case lineartett:{      ndofe=Ltett->ndofe;   break; }
  case linearhext:{      ndofe=Lht->ndofe;     break; }
  case quadratichext:{   ndofe=Qht->ndofe;     break; }
  default:{
    print_err("unknown element type is required",__FILE__,__LINE__,__func__);
  }
  }
  return ndofe;
  
}


/**
   function returns number of DOFs of node
   
   @param nid - node id
*/
long transtop::give_ndofn (long nid)
{
  return Gtt->give_ndofn (nid);
}

/**
   function returns total number of integration points on element
   
   @param eid - element id
   
   30.12.2001
*/
long transtop::give_tnip (long eid)
{
  if (eid < Tt->ne-1)
    return elements[eid+1].ipp[0][0] - elements[eid].ipp[0][0];

  return Tm->tnip-elements[eid].ipp[0][0];
}



/**
   function returns number of components
   
   @param eid - element id
   
   17.1.2003 - Tomas Krejci
*/
long transtop::give_ncomp (long eid)
{
  long ncomp;
  elemtypet te;
  
  te = give_elem_type (eid);
  
  switch (te){
  case barlint:{        ncomp=Lbt->ncomp;    break; }
  case barlintax:{      ncomp=Lbat->ncomp;   break; }
  case barquadt:{       ncomp=Qbt->ncomp;    break; }
  case barquadtax:{     ncomp=Qbat->ncomp;   break; }
  case trlint:{         ncomp=Ltt->ncomp;    break; }
  case trlaxisym:{      ncomp=Ltat->ncomp;   break; }
  case quadlint:{       ncomp=Lqt->ncomp;    break; }
  case quadquadt:{      ncomp=Qqt->ncomp;    break; }
  case quadquadtax:{    ncomp=Qqat->ncomp;   break; }
  case quadlaxisym:{    ncomp=Lqat->ncomp;   break; }
  case lineartett:{     ncomp=Ltett->ncomp;  break; }
  case linearhext:{     ncomp=Lht->ncomp;    break; }
  case quadratichext:{  ncomp=Qht->ncomp;    break; }
    
  default:{
    print_err("unknown element type is required",__FILE__,__LINE__,__func__);
  }
  }
  return ncomp;
}

/**
   function returns number of edges of element
   
   @param eid - element id
   
   JK, 22.2.2002
*/
long transtop::give_ned (long eid)
{
  long ned;
  elemtypet te;
  
  te = give_elem_type (eid);

  switch (te){
  case barlint:{      ned=Lbt->ned;    break; }
  case barlintax:{    ned=Lbat->ned;   break; }
  case barquadt:{     ned=Qbt->ned;    break; }    
  case barquadtax:{   ned=Qbat->ned;    break; }    
  case trlint:{       ned=Ltt->ned;    break; }
  case trlaxisym:{    ned=Ltat->ned;   break; }
  case quadlint:{     ned=Lqt->ned;    break; }
  case quadlaxisym:{  ned=Lqat->ned;   break; }
  case quadquadt:{    ned=Qqt->ned;    break; }
  case quadquadtax:{  ned=Qqat->ned;   break; }
  case lineartett:{   ned=Ltett->ned;  break; }
  case linearhext:{   ned=Lht->ned;    break; }
  case quadratichext:{ned=Qht->ned;    break; }

  default:{
    print_err("unknown element type is required",__FILE__,__LINE__,__func__);
  }
  }
  return ned;
  
}

/**
   function returns number of nodes on one element edge
   
   @param eid - number of element
   
   30.12.2001
*/
long transtop::give_nned (long eid)
{
  long nned;
  elemtypet te;
  
  te = give_elem_type (eid);
  
  switch (te){
  case barlint:{         nned=Lbt->nned;    break; }
  case barlintax:{       nned=Lbat->nned;   break; }
  case barquadt:{        nned=Qbt->nned;    break; }    
  case barquadtax:{      nned=Qbat->nned;    break; }    
  case trlint:{          nned=Ltt->nned;     break; }
  case trlaxisym:{       nned=Ltat->nned;    break; }
  case quadlint:{        nned=Lqt->nned;     break; }
  case quadquadt:{       nned=Qqt->nned;     break; }
  case quadquadtax:{     nned=Qqat->nned;     break; }
  case quadlaxisym:{     nned=Lqat->nned;    break; }
  case lineartett:{      nned=Ltett->nned;   break; }
  case linearhext:{      nned=Lht->nned;     break; }
  case quadratichext:{   nned=Qht->nned;     break; }
  default:{
    print_err("unknown element type is required",__FILE__,__LINE__,__func__);
  }
  }
  return nned;
}

/**
   function returns number of surfaces of element
   
   @param eid - element id
   
   JK, 9.3.2002
*/
long transtop::give_nsurf (long eid)
{
  long nsurf;
  elemtypet te;
  
  te = give_elem_type (eid);

  switch (te){
  case barlint:{         nsurf=0;     break; }
  case barlintax:{       nsurf=0;     break; }
  case barquadt:{        nsurf=0;     break; }
  case barquadtax:{      nsurf=0;     break; }
  case trlint:{          nsurf=0;     break; }
  case trlaxisym:{       nsurf=0;     break; }
  case quadlint:{        nsurf=0;     break; }
  case quadquadt:{       nsurf=0;     break; }
  case quadquadtax:{     nsurf=0;     break; }
  case quadlaxisym:{     nsurf=0;     break; }

  case lineartett:{      nsurf=Ltett->nsurf;   break; }
  case linearhext:{      nsurf=Lht->nsurf;     break; }
  case quadratichext:{   nsurf=Qht->nsurf;     break; }
  default:{
    print_err("unknown element type is required",__FILE__,__LINE__,__func__);
  }
  }
  return nsurf;
  
}

/**
   function returns number of nodes on one surface of element
   
   @param eid - element id
   
   JK, 24.8.2004
*/
long transtop::give_nnsurf (long eid)
{
  long nnsurf;
  elemtypet te;
  
  te = give_elem_type (eid);

  switch (te){
  case lineartett:{      nnsurf=Ltett->nnsurf;   break; }
  case linearhext:{      nnsurf=Lht->nnsurf;     break; }
  case quadratichext:{   nnsurf=Qht->nnsurf;     break; }
  default:{
    print_err("unknown element type is required",__FILE__,__LINE__,__func__);
  }
  }
  return nnsurf;
  
}

/**
   function returns number of nodes of element
   
   @param eid - element id
   
   JK, 22.2.2002
*/
long transtop::give_nne (long eid)
{
  long nne;
  elemtypet te;
  
  te = give_elem_type (eid);

  switch (te){
  case barlint:{          nne=Lbt->nne;    break; }
  case barlintax:{        nne=Lbat->nne;   break; }
  case barquadt:{         nne=Qbt->nne;    break; }
  case barquadtax:{       nne=Qbat->nne;   break; }
  case trlint:{           nne=Ltt->nne;    break; }
  case trlaxisym:{        nne=Ltat->nne;   break; }
  case quadlint:{         nne=Lqt->nne;    break; }
  case quadquadt:{        nne=Qqt->nne;    break; }
  case quadquadtax:{      nne=Qqat->nne;   break; }
  case quadlaxisym:{      nne=Lqat->nne;   break; }
  case lineartett:{       nne=Ltett->nne;  break; }
  case linearhext:{       nne=Lht->nne;    break; }
  case quadratichext:{    nne=Qht->nne;    break; }
  default:{
    print_err("unknown element type is required",__FILE__,__LINE__,__func__);
  }
  }
  return nne;
  
}

/**
   function returns number of integration points of element
   for conductivity %matrix

   @param eid - element id
   @param ri,ci - row and column indices
   
   JK, 22.2.2002
*/
long transtop::give_nip (long eid,long ri,long ci)
{
  long nip;
  elemtypet te;
  
  te = give_elem_type (eid);

  switch (te){
  case barlint:{          nip=Lbt->nip[ri][ci];     break; }
  case barlintax:{        nip=Lbat->nip[ri][ci];    break; }
  case barquadt:{         nip=Qbt->nip[ri][ci];     break; }
  case barquadtax:{       nip=Qbat->nip[ri][ci];    break; }
  case trlint:{           nip=Ltt->nip[ri][ci];     break; }
  case trlaxisym:{        nip=Ltat->nip[ri][ci];    break; }
  case quadlint:{         nip=Lqt->nip[ri][ci];     break; }
  case quadquadt:{        nip=Qqt->nip[ri][ci];     break; }
  case quadquadtax:{      nip=Qqat->nip[ri][ci];    break; }
  case quadlaxisym:{      nip=Lqat->nip[ri][ci];    break; }
  case lineartett:{       nip=Ltett->nip[ri][ci];   break; }
  case linearhext:{       nip=Lht->nip[ri][ci];     break; }
  case quadratichext:{    nip=Qht->nip[ri][ci];     break; }
  default:{
    print_err("unknown element type is required",__FILE__,__LINE__,__func__);
  }
  }
  return nip;
  
}

/**
   function returns degree of polynomial expansion
   
   @param eid - number of element
   
   TKr, 20.1.2003
   
*/
long transtop::give_degree (long eid)
{
  long deg;
  elemtypet te;
  
  te = give_elem_type (eid);
  
  switch (te){
  case barlint:{         deg=1;  break;  }
  case barlintax:{       deg=1;  break;  }
  case barquadt:{        deg=2;  break;  }
  case barquadtax:{      deg=2;  break;  }
  case trlint:{          deg=1;  break;  }
  case trlaxisym:{       deg=1;  break;  }
  case quadlint:{        deg=1;  break;  }
  case quadquadt:{       deg=2;  break;  }
  case quadquadtax:{     deg=2;  break;  }
  case quadlaxisym:{     deg=1;  break;  }
  case lineartett:{      deg=1;  break;  }
  case linearhext:{      deg=1;  break;  }
  case quadratichext:{   deg=2;  break;  }
  default:{
    print_err("unknown element type is required",__FILE__,__LINE__,__func__);
  }
  }
  return deg;
}

/**
   function returns dimension of problem
   
   @param eid - number of element
   
   JK, 23.2.2002
   modified by TKr 20.1.2033
*/
long transtop::give_dimension (long eid)
{
  long dim;
  elemtypet te;
  
  te = give_elem_type (eid);
  
  switch (te){
  case barlint:{         dim=1;  break;  }
  case barlintax:{       dim=1;  break;  }
  case barquadt:{        dim=1;  break;  }
  case barquadtax:{      dim=1;  break;  }
  case trlint:{          dim=2;  break;  }
  case trlaxisym:{       dim=2;  break;  }
  case quadlint:{        dim=2;  break;  }
  case quadquadt:{       dim=2;  break;  }
  case quadquadtax:{     dim=2;  break;  }
  case quadlaxisym:{     dim=2;  break;  }
  case lineartett:{      dim=3;  break;  }
  case linearhext:{      dim=3;  break;  }
  case quadratichext:{   dim=3;  break;  }
  default:{
    print_err("unknown element type is required",__FILE__,__LINE__,__func__);
  }
  }
  return dim;
}


/**
   function returns node numbers on required element edge
   
   @param eid - element id
   @param enid - end node id
   @param nodes - array containing end nodes
   
   JK, 15.11.2008
*/
void transtop::give_end_nodes (long eid,long enid,long *nodes)
{
  long nne;
  elemtypet te;
  ivector nod,endnod;
  
  //  type of element
  te = give_elem_type (eid);
  //  number of nodes on element
  nne = give_nne (eid);
  
  allocv (nne,nod);
  allocv (1,endnod);
  
  //  element nodes
  give_elemnodes (eid,nod);
  
  switch (te){
  case barlint:{
    endnod[0]=enid;
    break;
  }
  case barlintax:{
    endnod[0]=enid;
    break;
  }
  case barquadt:{
    endnod[0]=enid;
    break;
  }
  case barquadtax:{
    endnod[0]=enid;
    break;
  }
  case trlint:{
    break;
  }
  case trlaxisym:{
    break;
  }
  case quadlint:{
    break;
  }
  case quadlaxisym:{
    break;
  }
  case quadquadt:{ 
    break;
  }
  case lineartett:{
    break;
  }
  case linearhext:{
    break;
  }
  case quadratichext:{
    break;
  }
  default:{
    print_err("unknown element type is required",__FILE__,__LINE__,__func__);
  }
  }
  
  nodes[0]=nod[endnod[0]];
  
  destrv (nod);  destrv (endnod);
}


/**
   function returns node numbers on required element edge
   
   @param eid - element id
   @param edid - edge id
   @param nodes - array containing edge nodes
   
   5.3.2002
*/
void transtop::give_edge_nodes (long eid,long edid,long *nodes)
{
  long i,nne,nned;
  elemtypet te;
  ivector nod,edgenod;
  
  //  type of element
  te = give_elem_type (eid);
  //  number of nodes on element
  nne = give_nne (eid);
  //  number of nodes on one edge
  nned = give_nned (eid);
  
  allocv (nne,nod);
  allocv (nned,edgenod);
  
  //  element nodes
  give_elemnodes (eid,nod);
  
  switch (te){
  case barlint:{
    edgenod[0]=edid;
    break;
  }
  case barlintax:{
    break;
  }
  case barquadt:{
    break;
  }
  case barquadtax:{
    break;
  }
  case trlint:{
    lintriangle_edgnod (edgenod.a,edid);
    break;
  }
  case trlaxisym:{
    lintriangle_edgnod (edgenod.a,edid);
    break;
  }
  case quadlint:{
    linquadrilat_edgnod (edgenod.a,edid);
    break;
  }
  case quadlaxisym:{
    linquadrilat_edgnod (edgenod.a,edid);
    break;
  }
  case quadquadt:{ 
    quadquadrilat_edgnod (edgenod.a,edid);
    break;
  }
  case lineartett:{
    break;
  }
  case linearhext:{
    break;
  }
  case quadratichext:{
    break;
  }
  default:{
    print_err("unknown element type is required",__FILE__,__LINE__,__func__);
  }
  }
  
  for (i=0;i<nned;i++){
    nodes[i]=nod[edgenod[i]];
  }
  
  destrv (nod);  destrv (edgenod);
}

/**
   function returns node numbers on required element surface
   
   @param eid - element id
   @param surfid - surface id
   @param nodes - array containing surface nodes
   
   JK, 24.8.2004
*/
void transtop::give_surface_nodes (long eid,long surfid,long *nodes)
{
  long i,nne,nnsurf;
  elemtypet te;
  ivector nod,surfnod;
  
  //  type of element
  te = give_elem_type (eid);
  //  number of nodes on element
  nne = give_nne (eid);
  //  number of nodes on one surface
  nnsurf = give_nnsurf (eid);
  
  allocv (nne,nod);
  allocv (nnsurf,surfnod);
  
  //  element nodes
  give_elemnodes (eid,nod);
  
  switch (te){
  case barlint:{
    break;
  }
  case barlintax:{
    break;
  }
  case barquadt:{
    break;
  }
  case barquadtax:{
    break;
  }
  case trlint:{
    break;
  }
  case trlaxisym:{
    break;
  }
  case quadlint:{
    break;
  }
  case quadlaxisym:{
    break;
  }

  case lineartett:{
    break;
  }
  case linearhext:{
    linhexahedral_surfnod (surfnod.a,surfid);
    break;
  }
  case quadratichext:{
    quadhexahedral_surfnod (surfnod.a,surfid);
    break;
  }
  default:{
    print_err("unknown element type is required",__FILE__,__LINE__,__func__);
  }
  }
  
  for (i=0;i<nnsurf;i++){
    nodes[i]=nod[surfnod[i]];
  }
  
  destrv (nod);  destrv (surfnod);
}

/**
   function determinates the number of boundary objects and number of nodes on them
   boundary objects are end nodes in 1D, edges in 2D and surfaces in 3D
   
   @param eid - element id
   @param bco - number of boundary objects
   @param ncompbco - number of components on one boundary object
   
   JK, 15.11.2008
*/
void transtop::give_nbobjects (long eid,long &bco,long &ncompbco)
{
  
  switch (elements[eid].te){
    //  1D elements
  case barlint:{
    bco=Lbt->nen;
    ncompbco=1;
    break;
  }
  case barlintax:{
    bco=Lbat->nen;
    ncompbco=1;
    break;
  }
  case barquadt:{
    bco=Qbt->nen;
    ncompbco=1;
    break;
  }
  case barquadtax:{
    bco=Qbat->nen;
    ncompbco=1;
    break;
  }
    
    //  2D elements
  case trlint:{
    bco=Ltt->ned;
    ncompbco=Ltt->nned;
    break;
  }
  case trlaxisym:{
    bco=Ltat->ned;
    ncompbco=Ltat->nned;
    break;
  }
  case quadlint:{
    bco=Lqt->ned;
    ncompbco=Lqt->nned;
    break;
  }
  case quadlaxisym:{
    bco=Lqat->ned;
    ncompbco=Lqat->nned;
    break;
  }
  case quadquadt:{
    bco=Qqt->ned;
    ncompbco=Qqt->nned;
    break;
  }
  case quadquadtax:{
    bco=Qqat->ned;
    ncompbco=Qqat->nned;
    break;
  }
   
    //  3D elements
  case lineartett:{
    bco=Ltett->nsurf;
    ncompbco=Ltett->nnsurf;
    break;
  }
  case linearhext:{
    bco=Lht->nsurf;
    ncompbco=Lht->nnsurf;
    break;
  }
  case quadratichext:{
    bco=Qht->nsurf;
    ncompbco=Qht->nnsurf;
    break;
  }
  default:{
    print_err("unknown element type is required",__FILE__,__LINE__,__func__);
  }
  }
  
}

/**
   function assembles nodes on boundary objects (end node, edge, surface)
   
   @param eid - element id
   @param boid - boundary object id
   @param nod - array of node numbers
   
   JK, 15.11.2008
*/
void transtop::give_bonodes (long eid,long boid,long *nod)
{
  
  switch (elements[eid].te){
    //  1D elements
  case barlint:
  case barlintax:
  case barquadt:
  case barquadtax:{
    give_end_nodes (eid,boid,nod);
    break;
  }
    
    //  2D elements
  case trlint:
  case trlaxisym:
  case quadlint:
  case quadlaxisym:
  case quadquadt:
  case quadquadtax:{
    give_edge_nodes (eid,boid,nod);
    break;
  }
    
    //  3D elements
  case lineartett:
  case linearhext:
  case quadratichext:{
    give_surface_nodes (eid,boid,nod);
    break;
  }
  default:{
    print_err("unknown element type is required",__FILE__,__LINE__,__func__);
  }
  }
  
}




void transtop::alloc_prep(long nn, long ne)
{
  nodes = new nodet[nn];
  elements = new elementt[ne];
}

/**
   function returns integral
   
   @param eid - number of element
   
*/
/*
double transtop::give_integral (long eid,vector &nodval)
{
  double value;
  elemtypet te;
  
  te = give_elem_type (eid);
  
  switch (te){
  case barlint:{      value = Lbt->total_integral(eid,nodval);   break;}
  case barlintax:{    value = Lbat->total_integral(eid,nodval);  break;}
  case barquadt:{     value = Qbt->total_integral(eid,nodval);   break;}
  case barquadtax:{   value = Qbat->total_integral(eid,nodval);  break;}
  case quadlint:{     value = Lqt->total_integral(eid,nodval);   break;}
  case quadquadt:{    value = Qqt->total_integral(eid,nodval);   break;}
  case quadquadtax:{  value = Qqat->total_integral(eid,nodval);  break;}
  default:{
    print_err("unknown element type is required",__FILE__,__LINE__,__func__);
  }
  }
  return value;
}
*/

/**
   function allocates all needed arrays on nodes
   
   JK, 10.7.2008 - revision
*/
void transtop::alloc_nodes (void)
{
  long i,j,nne,ncomp, ncompo, ncompeqo, ipp;
  ivector nod;
  
  //  loop over elements
  for (i=0;i<ne;i++){
    nne = give_nne (i);
    allocv (nne,nod);
    give_elemnodes (i,nod);
    ncomp=give_ncomp (i); 
    ncompo=Tm->givencompother ();
    ipp = Tt->elements[i].ipp[0][0];
    ncompeqo=Tm->givencompeqother (ipp,0);
    for (j=0;j<nne;j++){
      //  array of gradients
      if (Tp->gradcomp==1){
	if (nodes[nod[j]].gradient==NULL){
	  nodes[nod[j]].alloc_grad(ncomp);
	  nodes[nod[j]].nullgrad();
	}
      }
      //  array of fluxes
      if (Tp->fluxcomp==1){
	if (nodes[nod[j]].flux==NULL){
	  nodes[nod[j]].alloc_flux(ncomp);
	  nodes[nod[j]].nullflux();
	}
      }
      //  array of other values
      if (Tp->othercomp==1){
	if (nodes[nod[j]].other==NULL){
	  nodes[nod[j]].alloc_other(ncompo);
	  nodes[nod[j]].nullother();
	}
      }
      //  array of eqother values
      if (Tp->eqothercomp==1){
	if (nodes[nod[j]].eqother==NULL){
	  nodes[nod[j]].alloc_eqother(ncompeqo);
	  nodes[nod[j]].nulleqother();
	}
      }
    }
    destrv (nod);
  }
  
  //  actual nodal values
  if (Tp->nvs==1){
    for (i=0;i<nn;i++){
      nodes[i].alloc_nodval ();
    }
  }
  //  previous nodal values
  if (Tp->pnvs==1){
    for (i=0;i<nn;i++){
      nodes[i].alloc_nodvalp ();
    }
  }
  //  initial nodal values
  if (Tp->invs==1){
    for (i=0;i<nn;i++){
      nodes[i].alloc_nodvali ();
    }
  }
  //  time derivatives of actual nodal values
  if (Tp->tdnvs==1){
    for (i=0;i<nn;i++){
      nodes[i].alloc_nodvalt ();
    }
  }
}

/**
   functions allocates end nodes
   end nodes are allocated in problems with discontinuities
   
   JK, 17.10.2008
*/
void transtop::alloc_enodes ()
{
  if (Gtt->nen>0){
    enodes = new endnodet [Gtt->nen];
  }
}

/**
   functions allocates edges
   edges are allocated in problems with discontinuities
   
   JK, 8.8.2008
*/
void transtop::alloc_edges ()
{
  if (Gtt->nged>0){
    edges = new edget [Gtt->nged];
  }
}

/**
   function initializes end node parameters and arrays
   
   JK, 17.10.2008
*/
void transtop::enodes_init ()
{
  long i;
  
  for (i=0;i<Gtt->nen;i++){
    enodes[i].init (i);
  }
}

/**
   function initializes edge parameters and arrays
   
   JK, 8.8.2008
*/
void transtop::edge_init ()
{
  long i;
  
  for (i=0;i<Gtt->nged;i++){
    edges[i].init (i);
  }
}

/**
   function computes jumps at discontinuity
   jumps are saved on end nodes and edges
   
   @param rhs - right hand side %vector
   
   JK, 8.8.2008
*/
void transtop::compute_jumps (double *rhs)
{
  long i,j,nm,*ncn1,*ncn2,*mcn;
  
  //  loop over end nodes
  for (i=0;i<Gtt->nen;i++){
    enodes[i].compute_jump (i);
    
    //  number of multipliers between first nodes
    nm=Gtt->endnodes[i].ndofn;
    ncn1 = new long [nm];
    ncn2 = new long [nm];
    mcn = new long [nm];
    
    
    Gtt->give_endnode_code_numbers (i,ncn1,ncn2,mcn);
    
    for (j=0;j<nm;j++){
      if (mcn[j]>0)
	rhs[mcn[j]-1]+=enodes[i].jump[j];
    }
    delete [] mcn;
    delete [] ncn2;
    delete [] ncn1;
  }

  //  loop over edges
  for (i=0;i<Gtt->nged;i++){
    edges[i].compute_jump (i);
    
    //  number of multipliers between first nodes
    nm=Gtt->gedges[i].ndofnf;
    ncn1 = new long [nm];
    ncn2 = new long [nm];
    mcn = new long [nm];
    
    
    Gtt->give_edge_code_numbers (i,1,ncn1,ncn2,mcn);
    
    for (j=0;j<nm;j++){
      if (mcn[j]>0)
	rhs[mcn[j]-1]+=edges[i].jumpfn[j];
    }
    delete [] mcn;
    delete [] ncn2;
    delete [] ncn1;


    //  number of multipliers between last nodes
    nm=Gtt->gedges[i].ndofnl;
    ncn1 = new long [nm];
    ncn2 = new long [nm];
    mcn = new long [nm];
    
    
    Gtt->give_edge_code_numbers (i,2,ncn1,ncn2,mcn);
    
    for (j=0;j<nm;j++){
      if (mcn[j]>0)
	rhs[mcn[j]-1]+=edges[i].jumpln[j];
    }
    delete [] mcn;
    delete [] ncn2;
    delete [] ncn1;

  }
  
}


/**
   function establishes %vector of initial nodal values
   
   JK, 5.3.2006
*/
void transtop::initial_nodval ()
{
  long i,j,k,ndofe,lcid,*cn;
  double *r;
  
  //  load case id must be equal to zero
  lcid=0;
  
  for (i=0;i<ne;i++){
    
    j=Gtt->leso[i];
    k=Gtt->gelements[i].auxinf;
    
    if (j==1 && k==0){
      //  added element
      //  number of DOFs on element
      ndofe = give_ndofe (i);
      
      r = new double [ndofe];
      cn = new long [ndofe];
      
      //  
      give_code_numbers (i,cn);
      //  nodal values on element
      elemvalues (lcid,i,r,cn,ndofe);
      //  definition of initial nodal values
      elements[i].initnodvalues (r,ndofe);
      
      Gtt->gelements[i].auxinf=1;
      
      delete [] cn;
      delete [] r;
    }
  }
  
}

/**
   function saves nodal unknowns from the array lhs to nodes
   function is used in problems with growing number of nodes and elements
   
   @param lhs - array of nodal values
   @param lhsi - array of initial nodal values
   @param tdlhs - array of time derivatives of nodal values
   
   JK, 2.6.2006
*/
void transtop::lhs_save (double *lhs,double *lhsi,double *tdlhs)
{
  long i,j,ndofn;
  long *cn;

  for (i=0;i<nn;i++){
    //  number of DOFs on node
    ndofn = give_ndofn (i);
    cn = new long [ndofn];
    give_node_code_numbers (i,cn);
    
    for (j=0;j<ndofn;j++){
      nodes[i].nodval[j]=0.0;
      nodes[i].nodvali[j]=0.0;
      nodes[i].nodvalt[j]=0.0;
      if (cn[j]>0){
	nodes[i].nodval[j]=lhs[cn[j]-1];
	nodes[i].nodvali[j]=lhsi[cn[j]-1];
	nodes[i].nodvalt[j]=tdlhs[cn[j]-1];
      }
    }   
    delete [] cn;
  }
  
}

/**
   function restores nodal unknowns from nodes to the array lhs
   function is used in problems with growing number of nodes and elements
   
   @param lhs - array of nodal values
   @param lhsi - array of initial nodal values
   @param tdlhs - array of time derivatives of nodal values

   JK, 2.6.2006
*/
void transtop::lhs_restore (double *lhs,double *lhsi,double *tdlhs)
{
  long i,j,ndofn;
  long *cn;

  for (i=0;i<nn;i++){
    ndofn = give_ndofn (i);
    cn = new long [ndofn];
    give_node_code_numbers (i,cn);
    
    for (j=0;j<ndofn;j++){
      if (cn[j]>0){
	lhs[cn[j]-1]=nodes[i].nodval[j];
	lhsi[cn[j]-1]=nodes[i].nodvali[j];
	tdlhs[cn[j]-1]=nodes[i].nodvalt[j];
      }
    }
    delete [] cn;
  }
  
}


long transtop::mesh_check(void)
{
  long i,j,err=0,kk,jj;

  //elems and nodes check
  for(i=0;i<Tt->ne;i++){
    for(j=0;j<Gtt->gelements[i].nne;j++){
      
      jj = Gtt->gelements[i].nodes[j];
      for(kk = 0; kk < Tp->ntm;kk++){
	
	if((Gtt->gelements[i].tgf) < (Gtt->gnodes[jj].tgf[kk])){

	  fprintf (stdout,"\n\n Different element node(%ld) time function number=%ld than element(%ld) time function number=%ld",j+1,Gtt->gnodes[jj].tgf[kk]+1,i+1,Gtt->gelements[i].tgf+1);
	  fprintf (stdout,"\n Node number=%ld, element number=%ld\n\n",Gtt->gelements[i].nodes[j]+1,i+1);
	  err = 1;

	  return err;
	}
	
      }
    }
  }
  
  return err;
}

/**
   function defines types of materials on nodes
   this is necessary for problems with discontinuities
   
   JK, 16.11.2007
*/
/*
void transtop::node_materials ()
{
  long i,j,mid,nne;
  ivector nodes;
  
  for (i=0;i<ne;i++){
    //  number nodes on element
    nne = give_nne (i);
    //  allocation of array nodes
    allocv (nne,nodes);
    //  element nodes
    give_elemnodes (i,nodes);
    //  id of material defined on element
    mid = elements[i].idm[0];
    for (j=0;j<nne;j++){
      Gtt->gnodes[nodes[j]].ai=mid;
    }
    destrv (nodes);
  }
}
*/

/*
void transtop::jump_initiation (long nbn,seqselnodes *selnodfeti)
{
  long i,j,k,ns,nsnsd;
  
  //  total number of interface/boundary nodes
  tnbn = nbn;

  //  jnmap[i][0]=j - the i-th interface/boundary node has local number j on the subdomain with lower id
  //  jnmap[i][1]=j - the i-th interface/boundary node has local number j on the subdomain with higher id
  jnmap = new long* [tnbn];
  for (i=0;i<tnbn;i++){
    jnmap[i] = new long [2];
    
    jnmap[i][0] = -1;
    jnmap[i][1] = -1;
  }
  
  //  number of subdomains
  ns = Gtt->stop->ns;
  for (i=0;i<ns;i++){
    //  number of selected nodes on subdomain
    nsnsd = selnodfeti->nsndom[i];
    for (j=0;j<nsnsd;j++){
      k=selnodfeti->gnn[i][j];
      if (jnmap[k][0]==-1)
	jnmap[k][0] = selnodfeti->lsnl[i][j];
      else
	jnmap[k][1] = selnodfeti->lsnl[i][j];
    }
  }
}
*/
